package anonvpn.anon_next.core;

import HTTPClient.HTTPClientSocketFactory;
import anon.crypto.JAPCertificate;
import anon.crypto.SignatureVerifier;
import anon.crypto.CryptoUtil;
import anon.util.ResourceLoader;
import anon.util.IResourceLoaderHelper;
import anonvpn.anon_next.core.gui.IVPNInterface;
import anonvpn.anon_next.core.gui.IVPNInterfaceFactory;
import anonvpn.anon_next.core.networking.IConnectivityMonitor;
import anonvpn.anon_next.core.networking.IConnectivityMonitorFactory;
import anonvpn.anon_next.core.networking.INetworkFactory;
import anonvpn.anon_next.core.networking.RegularHTTPClientSocketFactory;
import anonvpn.anon_next.core.notification.Event;
import anonvpn.anon_next.core.notification.NotificationDispatcher;
import anonvpn.anon_next.core.notification.Observer;
import anonvpn.anon_next.core.persistence.IConfig;
import anonvpn.anon_next.core.persistence.IPersistenceFactory;
import logging.ILog;
import logging.LogHolder;
import logging.LogLevel;
import logging.LogType;

/**
 * Created by martin on 5/22/17.
 */

public class ANONVPNService implements Observer
	{

		private final NotificationDispatcher mNotification;
		private final INetworkFactory mNetworkFactory;
		private final IPersistenceFactory mPersistenceFactory;
		private final IVPNInterfaceFactory mGUIFactory;
		private final HTTPClientSocketFactory mMixSocketFactory;
		private final InfoService mInfoService;
		private final IConnectivityMonitor mConnectivityMonitor;

		private final IConfig mConfig;
		private final IVPNInterface mGUI;

		private CascadeConnectionManager mCascadeManager = null;

		/**
		 * Launches the os-independent VPN service.
		 *
		 * @param networkFactory
		 * @param persistenceFactory
		 * @param guiFactory
		 */
		public ANONVPNService(INetworkFactory networkFactory,
							  IPersistenceFactory persistenceFactory,
							  IVPNInterfaceFactory guiFactory,
							  IConnectivityMonitorFactory connFactory,
							  HTTPClientSocketFactory mixSocketFactory,
							  IResourceLoaderHelper resourceLoderHelper,
							  ILog logger)
			{
				//set the logging
				LogHolder.setLogInstance(logger);
				if(CascadeConnectionManager.USE_DEBUG_LOG)
					{
						logger.setLogLevel(LogLevel.DEBUG);
						logger.setLogType(LogType.ALL);
						LogHolder.setDetailLevel(LogHolder.DETAIL_LEVEL_HIGHEST);
					}

				//should be probabbly moved to a better place....
				ResourceLoader.setReourceLoaderHelper(resourceLoderHelper); //sets a resource loader to load certificates etc...
				SignatureVerifier.getInstance().setCheckSignatures(true); //enable certificate check

				//add the paths for various certifcates (and loads them...)
				CryptoUtil.addDefaultCertificates("acceptedInfoServiceCAs/", JAPCertificate.CERTIFICATE_TYPE_ROOT_INFOSERVICE);
				CryptoUtil.addDefaultCertificates("acceptedInfoServices/", JAPCertificate.CERTIFICATE_TYPE_INFOSERVICE);
				CryptoUtil.addDefaultCertificates("acceptedMixCAs/", JAPCertificate.CERTIFICATE_TYPE_ROOT_MIX);
				CryptoUtil.addDefaultCertificates("acceptedPIs/", JAPCertificate.CERTIFICATE_TYPE_PAYMENT);
				CryptoUtil.addDefaultCertificates("acceptedTaCTemplates/", JAPCertificate.CERTIFICATE_TYPE_TERMS_AND_CONDITIONS);

				//end


				mNetworkFactory = networkFactory;
				mMixSocketFactory = mixSocketFactory;
				mPersistenceFactory = persistenceFactory;
				mGUIFactory = guiFactory;
				mNotification = new NotificationDispatcher();
				mConfig = mPersistenceFactory.getConfiguration();
				mInfoService = new InfoService(mNotification, mConfig, new RegularHTTPClientSocketFactory(), mMixSocketFactory);
				mConnectivityMonitor = connFactory.getConnectivityMonitor(mNotification);
				mGUI = mGUIFactory.getGUIInterface(mNotification, mConfig, mInfoService);

				mNotification.subscribe(Event.VPN_ENABLED, this);
				mNotification.subscribe(Event.VPN_DISABLED, this);
						}

		@Override
		public void notify(Event event)
			{
				if (event == Event.VPN_ENABLED)
					{
						mCascadeManager = new CascadeConnectionManager(mNotification, mNetworkFactory, mConfig, mConnectivityMonitor, mMixSocketFactory);
					}
				else if(event == Event.VPN_DISABLED && mCascadeManager != null)
					{
						mCascadeManager.notify(Event.VPN_DISABLED);
					}
			}

		public IVPNInterface getGUIInterface()
			{
				return mGUI;
			}
	}
