package anonvpn.anon_next.core;

import java.util.Enumeration;
import java.util.Hashtable;

import HTTPClient.HTTPClientSocketFactory;
import anon.infoservice.Database;
import anon.infoservice.InfoServiceDBEntry;
import anon.infoservice.InfoServiceHolder;
import anon.infoservice.MixCascade;
import anon.infoservice.StatusInfo;
import anonvpn.anon_next.core.notification.Event;
import anonvpn.anon_next.core.notification.NotificationDispatcher;
import anonvpn.anon_next.core.notification.Observer;
import anonvpn.anon_next.core.persistence.IConfig;

/**
 * Created by martin on 5/22/17.
 */

public class InfoService implements Observer
	{

		private final HTTPClientSocketFactory mProtectedSocketFactory;
		private final HTTPClientSocketFactory mRegularSocketFactory;
		private final IConfig mConfig;

		private boolean mConnectedToCascade = false;
		private boolean mVPNEnabled = false;

		public InfoService(NotificationDispatcher notification, IConfig config, HTTPClientSocketFactory regularSocketFactory, HTTPClientSocketFactory protectedSocketFactory)
			{
				this.mProtectedSocketFactory = protectedSocketFactory;
				this.mRegularSocketFactory = regularSocketFactory;
				this.mConfig = config;

				notification.subscribe(Event.CASCADE_CONNECTED, this);
				notification.subscribe(Event.CASCADE_DISCONNECTED, this);
				notification.subscribe(Event.VPN_ENABLED, this);
				notification.subscribe(Event.VPN_DISABLED, this);
			}

		@Override
		public void notify(Event event)
			{

				if (event == Event.VPN_ENABLED)
					{

						mVPNEnabled = true;

					}
				else if (event == Event.VPN_DISABLED)
					{

						mVPNEnabled = false;

					}
				else if (event == Event.CASCADE_CONNECTED)
					{

						mConnectedToCascade = true;

					}
				else if (event == Event.CASCADE_DISCONNECTED)
					{

						mConnectedToCascade = false;

					}
			}

		/**
		 * Retrieves the list of current mix cascades from the Info Service.
		 * <p>
		 * TODO: Currently a stub.
		 *
		 * @return
		 */
		public Hashtable fetchMixCascades(boolean bWithStatus,boolean bOnlyVPN)
			{

				InfoServiceHolder is = InfoServiceHolder.getInstance();

				if (!mVPNEnabled)
					{
						is.setSocketFactory(mRegularSocketFactory);
					}
				else
					{
						if (mConnectedToCascade)
							{
								is.setSocketFactory(mRegularSocketFactory);
							}
						else
							{
								is.setSocketFactory(mProtectedSocketFactory);
							}
					}

				InfoServiceDBEntry service = new InfoServiceDBEntry(mConfig.getString("PrimaryInfoService.Host", "141.76.46.139"), mConfig.getInt("PrimaryInfoService.Port", 80));
				service.setConnectionTimeout(20000);
				is.setChangeInfoServices(false);
				is.setNumberOfAskedInfoServices(1);
				is.setPreferredInfoService(service);

				Hashtable cascades = is.getMixCascades();
				if (cascades != null)
					{
						Enumeration enumer = cascades.elements();
						while (enumer.hasMoreElements())
							{
								MixCascade cascade = (MixCascade) enumer.nextElement();
								if(bOnlyVPN)
									{
										if(!cascade.isVPNSupported())
											{
												cascades.remove(cascade.getId());
												continue;
											}
									}
								if(bWithStatus)
									{
										StatusInfo statusInfo = cascade.fetchCurrentStatus();
										// Store the statusInfo, so that we can fetch it later usings cascade.getStatusInfo()
										Database.getInstance(StatusInfo.class).update(statusInfo, false);
									}
							}
					}
				return cascades;
			}
	}
