package anonvpn.anon_next.core.entities;


final public class IPv4Packet
	{

		public static final int IP_CHECKSUM_FIELD_POSITION = 10;
		public static final int IP_LEN_FIELD_POSITION = 2;
		public static final int IP_HEADER_LENGTH = 20;
		public static final int SRC_IP_POS = 12;
		public static final int DST_IP_POS = 16;
		public static final int SRC_PORT_POS = 20;
		public static final int DST_PORT_POS = 22;


		public static final int UDP_CHECKSUM_FIELD_POSITION = 26;
		public static final int UDP_LEN_FIELD_POSITION = 24;
		public static final int UDP_PAYLOAD_START_POSITION = 28;
		private final byte[] m_arbytesPktBuf;


		public static final int FIN = 0x01;
		public static final int SYN = 0x02;
		public static final int RST = 0x04;
		public static final int PSH = 0x08;
		public static final int ACK = 0x10;
//		public static final int URG = 0x20;

		public IPv4Packet(final byte[] pktBuf)
			{
				m_arbytesPktBuf = pktBuf;
			}

		public final byte[] getRawPacket()
			{
				return m_arbytesPktBuf;
			}

		public int getLength()

			{
				return ((m_arbytesPktBuf[2] << 8)| (0x00FF & m_arbytesPktBuf[3]))&0x00FFFF ;
			}

		public boolean isUDP()
			{
				return m_arbytesPktBuf[9] == 17;
			}

		public boolean isTCP()
			{
				return m_arbytesPktBuf[9] == 6;
			}

		public int getDstAddress()
		{

			return ((m_arbytesPktBuf[16]&0x00FF)<<24) | (m_arbytesPktBuf[17]&0x00FF)<<16 | (m_arbytesPktBuf[18]&0x00FF)<<8 | ( m_arbytesPktBuf[19]&0x00FF);
		}

		public String getDstAddressStr()
		{
			return getIpAddress(m_arbytesPktBuf,16);
		}

		public int getSrcAddress()
		{
			return ((m_arbytesPktBuf[12]&0x00FF)<<24) | (m_arbytesPktBuf[13]&0x00FF)<<16 | (m_arbytesPktBuf[14]&0x00FF)<<8 | ( m_arbytesPktBuf[15]&0x00FF);
		}

		public String getSrcAddressStr()
		{
			return getIpAddress(m_arbytesPktBuf,12);
		}

		private static String getIpAddress(byte[] rawBytes,int offset) {
       StringBuilder ipAddress = new StringBuilder();
       ipAddress.append(rawBytes[offset] & 0xFF);
       ipAddress.append(".");
       ipAddress.append(rawBytes[offset+1] & 0xFF);
       ipAddress.append(".");
       ipAddress.append(rawBytes[offset+2] & 0xFF);
       ipAddress.append(".");
       ipAddress.append(rawBytes[offset+3] & 0xFF);
       return ipAddress.toString();
   }
		
		public boolean isFIN()
		{
			if(isTCP()) return (m_arbytesPktBuf[33] & FIN) == FIN;
			return false;
		}

		public boolean isSYN()
		{
			if(isTCP()) return (m_arbytesPktBuf[33] & SYN) == SYN;
			return false;
		}

		public boolean isRST()
		{
			if(isTCP()) return (m_arbytesPktBuf[33] & RST) == RST;
			return false;
		}

		public boolean isPSH()
		{
			if(isTCP()) return (m_arbytesPktBuf[33] & PSH) == PSH;
			return false;
		}

		public boolean isACK()
		{
			if(isTCP()) return (m_arbytesPktBuf[33] & ACK) == ACK;
			return false;
		}

//		public boolean isURG()
//		{
//			return (m_arbytesPktBuf[13] & URG) == URG;
//		}

		public String toString()
			{
				String s="IPv4Packet: Len: "+getLength();
				s+=" Content: ";
				for(int i=0;i<getLength();i++)
					{
						s+=String.format("%02x:",m_arbytesPktBuf[i]);
					}
				return s;
			}

	}
