package anonvpn.anon_next.core.networking;

import java.io.IOException;

import anonvpn.anon_next.core.entities.IPv4Packet;
import anonvpn.anon_next.core.persistence.IConfig;

public abstract class AbstractTunnelingDevice implements ITunneling
	{
		IConfig m_Config;

		public AbstractTunnelingDevice(IConfig a_config)
			{
				m_Config = a_config;
			}

		public void writePacket(IPv4Packet pkt) throws IOException
			{
				if (m_Config != null)
					m_Config.addReceivedBytes(pkt.getLength());
				writePacket_implementation(pkt);
			}

		public void write(byte[] ar, int off, int len) throws IOException
			{
				write_implementation(ar, off, len);
				if (m_Config != null)
					m_Config.addReceivedBytes(len);
			}

		public IPv4Packet readPacket() throws IOException
			{
				IPv4Packet p = readPacket_implementation();
				if (m_Config != null)
					m_Config.addSentBytes(p.getLength());
				return p;
			}

		abstract protected void writePacket_implementation(IPv4Packet pkt) throws IOException;

		abstract protected void write_implementation(byte[] ar, int off, int len) throws IOException;

		abstract protected IPv4Packet readPacket_implementation() throws IOException;

	}
