package anonvpn.anon_next.core.networking;

import java.io.InputStream;

import logging.LogHolder;
import logging.LogLevel;
import logging.LogType;


final public class ReceiverThread extends Thread
	{

		private final ITunneling m_TunDevice;
		private final InputStream m_streamMixIn;
		private final int m_iMTU;
		private volatile boolean m_bRun = false;

		public ReceiverThread(ITunneling tunDevice, InputStream mixIn)
			{
				m_TunDevice = tunDevice;
				m_streamMixIn = mixIn;
				m_iMTU = tunDevice.getMTU();

				setName("VPN IP Receiver Thread");
				setDaemon(true);
				m_bRun = true;
			}

		public void terminate()
			{
				m_bRun = false;
				try
					{
						m_streamMixIn.close();
					}
				catch (Exception e1)
					{
					}
				interrupt();
				try
					{
						join();
					}
				catch (Exception e)
					{
					}
			}

		@Override
		public void run()
			{
				int lengthIn = 0;
				int posIn = 0;
				int remaining = 4;
				byte[] arPacketIn = new byte[m_iMTU];
				while (m_bRun)
					{
						try
							{
								// Read the incoming packet from the tunnel.
								if (remaining > 0)
									{
										int l = m_streamMixIn.read(arPacketIn, posIn, remaining);
										if(l<0)
											{
												m_bRun=false;
												break;
											}
										remaining -= l;
										posIn += l;
									}
								if (posIn == 4 && lengthIn == 0)
									{
										lengthIn = ((arPacketIn[2] & 0xFF) << 8) | (arPacketIn[3] & 0xFF);
										remaining = lengthIn - 4;
										int l = m_streamMixIn.read(arPacketIn, posIn, remaining);
										if(l<0)
											{
												m_bRun=false;
												break;
											}
										remaining -= l;
										posIn += l;
									}
								if (lengthIn > 0)
									{
										if (remaining > 0)
											{
												int l = m_streamMixIn.read(arPacketIn, posIn, remaining);
												if(l<0)
													{
														m_bRun=false;
														break;
													}
												remaining -= l;
												posIn += l;
											}
										else
											{
												m_TunDevice.write(arPacketIn, 0, lengthIn);
												posIn = 0;
												remaining = 4;
												lengthIn = 0;
											}
									}
							}
						catch (Exception e)
							{
								m_bRun = false;
								//System.out.println("VPN Receiver Thread: Failed to handle packet: " + e.getMessage());
							}
					}
				LogHolder.log(LogLevel.DEBUG,LogType.NET,"RECEIVER TERMINATED");
			}
	}
