package anonvpn.anon_next.core.networking;

import HTTPClient.HTTPClientSocketFactory;
import anon.client.AnonClient;
import anonvpn.anon_next.core.CascadeConnectionManager;
import anonvpn.anon_next.core.entities.IPv4Packet;
import anonvpn.anon_next.core.networking.TcpSocks.DNSResolver;
import anonvpn.anon_next.core.networking.TcpSocks.TcpStreamManager;
import anonvpn.anon_next.core.networking.TcpSocks.UdpStreamManager;
import logging.LogHolder;
import logging.LogLevel;
import logging.LogType;

import java.io.OutputStream;

final public class SenderThread extends Thread
	{
		private final ITunneling m_TunDevice;
		private final OutputStream m_streamMixOut;
		private volatile boolean m_bRun = false;

		private final TcpStreamManager tcpManager;
		private final UdpStreamManager udpManager;
		private final static int m_iIP_DNS = 0x0a000001;
		private final DNSResolver m_DnsResolver;

		public SenderThread(ITunneling tunDevice, OutputStream mixOut, AnonClient a_AnonClient,
				HTTPClientSocketFactory mMixSocketFactory)
			{
				m_TunDevice = tunDevice;
				m_streamMixOut = mixOut;
				setName("VPN Sender Thread");
				setDaemon(true);
				m_bRun = true;
				m_DnsResolver = new DNSResolver(m_TunDevice);

				tcpManager = TcpStreamManager.getInstance();
				tcpManager.setAnonClient(a_AnonClient);
				tcpManager.setDnsResolver(m_DnsResolver);
				tcpManager.setMixSocketFactory(mMixSocketFactory);
				tcpManager.setTunDevice(m_TunDevice);

				udpManager = new UdpStreamManager();
				udpManager.setAnonClient(a_AnonClient);
				udpManager.setDnsResolver(m_DnsResolver);
				udpManager.setMixSocketFactory(mMixSocketFactory);
				udpManager.setTunDevice(m_TunDevice);
				udpManager.setSrcIP(m_TunDevice.getIP());
			}

		public void terminate()
			{
				m_bRun = false;
				interrupt();
				m_DnsResolver.stop();
				udpManager.close();
				try
					{
						join();
					}
				catch (InterruptedException e)
					{
					}
			}

		@Override
		public void run()
			{
				LogHolder.log(LogLevel.DEBUG,LogType.NET,"VPN - Sender Thread started");

				IPv4Packet currentPkt = null;
runloop:
				while (m_bRun)
					{
						try
							{
								// Read packet from tunneling device
								currentPkt = m_TunDevice.readPacket();
								if (!m_bRun)
									{
										break;
									}
								if (currentPkt.isUDP())
									{

										if (CascadeConnectionManager.USE_FAKE_DNS && currentPkt.getDstAddress() == m_iIP_DNS)
											{
												m_DnsResolver.sendResponse(currentPkt);
												continue;
											}
										else if (CascadeConnectionManager.USE_UDP_SOCKS/*&& currentPkt.getDstAddress() == m_iIP_DNS*/ )
											{
												udpManager.handlePacket(currentPkt.getRawPacket());
												continue;
											}
									}

								else if (CascadeConnectionManager.USE_TCP_SOCKS && currentPkt.isTCP())
									{
										tcpManager.handlePacket(currentPkt);
										continue;
									}

								//fallback - IP or TCP/UDP not handled in an optimised way

								int index = 0;
								int length = currentPkt.getLength();
								while (length > 0)
									{
										///@todo: fix mix block size
										int i = Math.min(790/*m_streamMixOut.m_iOutputBlockSize*/, length);
										try
											{
												m_streamMixOut.write(currentPkt.getRawPacket(), index, i);
											}
										catch (Exception e)
											{
												e.printStackTrace();
												///TODO: Correct error handling
												m_bRun=false;
												break runloop;
											}
										index += i;
										length -= i;
									}
							}

						//						m_streamMixOut.write(currentPkt.getRawPacket(),0,currentPkt.getLength());
						//										System.out.println("SENDER THREAD - SUCCESSFULLY WROTE PACKET!");

						catch (Exception e)
							{
								//System.out.println("VPN Sender Thread Failed to handle packet: " + e.getMessage());
								//e.printStackTrace();
								///TODO: Correct error handling
								LogHolder.log(LogLevel.DEBUG,LogType.NET,"VPN Sender Thread Failed to handle packet: " + e.getMessage(),e);
								m_bRun=false;						
							}
					}
				udpManager.close();
				LogHolder.log(LogLevel.DEBUG,LogType.NET,"VPN - Sender Thread stopped");
			}
	}
