package anonvpn.anon_next.core.networking.TcpSocks;

import java.util.Random;

import anonvpn.anon_next.core.entities.IPv4Packet;

final public class BitUtils
	{
	
		public static int getUnsignedShort(short value)
			{
				return value & 0xFFFF;
			}

		public static int getUnsignedByte(byte value)
			{
				return value & 0x00FF;
			}
		final public static int getUnsignedShort(byte[] value, int pos)
			{
				int v = value[pos];
				v <<= 8;
				v |= (value[pos + 1] & 0x00FF);
				return v & 0x00FFFF;
			}

		public static long getUnsignedInt(int value)
			{
				return value & 0xFFFFFFFFL;
			}

		public static byte[] longToBytes(long l)
			{
				byte[] result = new byte[8];
				for (int i = 7; i >= 0; i--)
					{
						result[i] = (byte) (l & 0xFF);
						l >>= 8;
					}
				return result;
			}

		public final static long bytesToLong(byte[] b, int pos)
			{
				long result = 0;
				for (int i = 0; i < 8; i++)
					{
						result <<= 8;
						result |= (b[i + pos] & 0xFF);
					}
				return result;
			}

		public static final byte[] intToBytes(int value)
			{
				return new byte[] { (byte) (value >>> 24), (byte) (value >>> 16), (byte) (value >>> 8), (byte) value };
			}

		public static final void intToBytes(int value, byte[] to, int ito)
			{
				to[ito] = (byte) (value >>> 24);
				to[ito + 1] = (byte) (value >>> 16);
				to[ito + 2] = (byte) (value >>> 8);
				to[ito + 3] = (byte) (value);
			}

		public static int bytesToInt(byte[] b, int pos)
			{
				int result = 0;
				for (int i = 0; i < 4; i++)
					{
						result <<= 8;
						result |= (b[i + pos] & 0xFF);
					}
				return result;
			}

		// only for debugging purposes
		public static String printPacket(String msg, byte[] packet)
			{
				String s = msg;
				s += "Len: " + packet.length;
				//        if(packet[9] == 6) s+=" TCP ";
				//        if(packet[9] == 17) s+=" UDP ";

				s += " Content: \t";
				for (int i = 0; i < packet.length; i++)
					{
						s += String.format("%02x:", packet[i]);
					}
				return s;
			}

		public static void calcIPHeaderChecksum(byte[] buf)
			{

				int sum = 0;
				int data;

				// Handle all pairs
				for (int len = 0; len < 20; len += 2)
					{
						// Corrected to include @Andy's edits and various comments on Stack Overflow
						data = (((buf[len] << 8) & 0xFF00) | ((buf[len + 1]) & 0xFF));
						sum += data;
					}

				// Final 1's complement value correction to 16-bits

				sum = ((sum >> 16) & 0xffff) + (sum & 0xffff);
				if(sum>0xFFFF)
					{
						sum++;
//						sum&=0xFFFF;
					}
				sum =  (~sum & 0xffff);

				buf[IPv4Packet.IP_CHECKSUM_FIELD_POSITION] = (byte) ((sum >> 8) & 0x0FF);
				buf[IPv4Packet.IP_CHECKSUM_FIELD_POSITION + 1] = (byte) (sum & 0x00FF);

			}

		public static long calcIPHeaderChecksumOld(byte[] buf)
			{

				long sum = 0;
				long data;

				// Handle all pairs
				for (int len = 0; len < 20; len += 2)
					{
						// Corrected to include @Andy's edits and various comments on Stack Overflow
						data = (((buf[len] << 8) & 0xFF00) | ((buf[len + 1]) & 0xFF));
						sum += data;
						// 1's complement carry bit correction in 16-bits (detecting sign extension)
						if ((sum & 0xFFFF0000) > 0)
							{
								sum = sum & 0xFFFF;
								sum += 1;
							}
					}

				// Final 1's complement value correction to 16-bits
				sum = ~sum;
				sum = sum & 0xFFFF;

				buf[IPv4Packet.IP_CHECKSUM_FIELD_POSITION] = (byte) ((sum >> 8) & 0x0FF);
				buf[IPv4Packet.IP_CHECKSUM_FIELD_POSITION + 1] = (byte) (sum & 0x00FF);

				return sum;

			}

		public static void computeTCPChecksum(byte[] pseudoIPHeader, byte[] packet, int payloadLen)
			{
				int i = 0;
				int sum = 0;
				int data;
				packet[36] = packet[37] = 0;
				//        System.arraycopy(pseudoIPHeader, 0, packet, payloadLen + 40, 12);
				for (i = 0; i < 12; i += 2)
					{
						data = (((pseudoIPHeader[i] << 8) & 0xFF00) | ((pseudoIPHeader[i + 1]) & 0xFF));
						sum += data;
					}

				int length = payloadLen + 20;
				i = 20;

				// loop through all 16-bit words unless there's 0 or 1 byte left.
				while (length > 1)
					{
						data = (((packet[i] << 8) & 0xFF00) | ((packet[i + 1]) & 0xFF));
						sum += data;
						i += 2;
						length -= 2;
					}

				if (length > 0)
					{ // ie. there are 8 bits of data remaining.
						sum += (packet[i] << 8 & 0xFF00); // create a 16 bit word where the 8 lsb are 0's and add it to the sum.
					}

				sum = (sum >> 16 & 0xffff) + (sum & 0xffff);
				if(sum>0xFFFF)
					{
						sum++;
//						sum&=0xFFFF;
					}
				
				sum = (~sum & 0xffff);

				packet[36] = (byte) ((sum >> 8) & 0x0FF);
				packet[37] = (byte) (sum & 0x00FF);
			}

		public static long getUnsignedInt(byte[] b, int pos)
			{
				int result = 0;
				for (int i = 0; i < 4; i++)
					{
						result <<= 8;
						result |= (b[i + pos] & 0xFF);
					}
				return result & 0xFFFFFFFFL;
			}

		public static void main(String argv[])
			{
				byte[] pseudo = new byte[12];
				byte[] packet = new byte[1500];
				Random r = new Random();
				r.nextBytes(pseudo);
				r.nextBytes(packet);
				long l = System.currentTimeMillis();
				for (int i = 0; i < 10000000; i++)
					{
						BitUtils.computeTCPChecksum(pseudo, packet, packet.length-40);
					}
				System.out.println("TCPChecksum takes: " + (System.currentTimeMillis() - l));

				byte[] ip1=new byte[20];
				byte[] ip2=new byte[20];
				
				for (int i = 0; i < 100000000; i++)
					{
						r.nextBytes(ip1);
						System.arraycopy(ip1, 0, ip2, 0, 20);
						BitUtils.calcIPHeaderChecksum(ip1);
						int b1 = ip1[IPv4Packet.IP_CHECKSUM_FIELD_POSITION];
						int b2 = ip1[IPv4Packet.IP_CHECKSUM_FIELD_POSITION+1];
						BitUtils.calcIPHeaderChecksumOld(ip2);
						boolean b = (b1 == ip2[IPv4Packet.IP_CHECKSUM_FIELD_POSITION] && b2 == ip2[IPv4Packet.IP_CHECKSUM_FIELD_POSITION+1]);
						if (!b)
							{
								System.out.println(b);
								
							}
					}
			}

	
	}