package anonvpn.anon_next.core.networking.TcpSocks;

import anon.client.AnonTCPChannel;
import anonvpn.anon_next.core.entities.IPv4Packet;
import anonvpn.anon_next.core.networking.ITunneling;
import logging.LogHolder;
import logging.LogLevel;
import logging.LogType;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Random;

/**
 * Created by nod on 8/8/17.
 */

final class TcpStream implements Runnable
	{

		private String m_strThreadName;

		private final static int TCP_IP_HEADER_LENGTH = 40;
		private final long m_dstIpPort;

		private final static Random ms_Random = new Random();

		private volatile boolean m_bRun = true;

		private final AnonTCPChannel m_anonTCPChannel;
		private final ITunneling m_TunDevice;
		public TcpStatus status;
		private long ackNumberToDevice, seqNumberToDevice = 0;
		private long lastPushSeq = 0;

		private InputStream m_inputstreamFromMix;
		private OutputStream os;

		//    Packet refPacket = null;
		byte[] refPkt = null;
		byte[] pseudoHeader = new byte[12];

		byte[] packet;

		private long m_isentWindowPos;
		private final Object m_oSyncWindowSize = new Object();

		private enum TcpStatus {
			SYN_RECEIVED, ESTABLISHED, CLOSE_WAIT, FIN_WAIT, CLOSING
		}

		public TcpStream(long dstIpPorts, AnonTCPChannel chan, ITunneling a_TunDevice)
			{
				m_dstIpPort = dstIpPorts;
				m_anonTCPChannel = chan;
				m_TunDevice = a_TunDevice;
				refPkt = new byte[m_TunDevice.getMTU()];
				LogHolder.log(LogLevel.DEBUG,LogType.NET,"Created VPN-TCP-Stream for DstIPPort: "+m_dstIpPort);
			}

		private void initPacket(byte[] buffer)
			{

				System.arraycopy(buffer, 0, refPkt, 0, 40);
				// swap IPs
				System.arraycopy(buffer, 12, refPkt, 16, 4);
				System.arraycopy(buffer, 16, refPkt, 12, 4);
				// swap ports
				System.arraycopy(buffer, 20, refPkt, 22, 2);
				System.arraycopy(buffer, 22, refPkt, 20, 2);

				// reset IP-flags
				refPkt[6] = 0;
				refPkt[3] = 40; //Toal Len --> set to 40 for easier checkum update;
				refPkt[4] = 0; // Identification
				refPkt[5] = 0; // Identification
				refPkt[6] = 0; // Flags & Offset
				refPkt[7] = 0; // Offset
				refPkt[8] = 64; // TTL

				refPkt[10] = 0; // For Header Checksum calculation (for len = 40)
				refPkt[11] = 0; // For Header Checksum calculation (for len = 40)

				BitUtils.calcIPHeaderChecksum(refPkt);

				// TCP header

				ackNumberToDevice = BitUtils.getUnsignedInt(BitUtils.bytesToInt(refPkt, 24)) + 1;
				seqNumberToDevice = ms_Random.nextInt(Short.MAX_VALUE + 1);

				//        System.arraycopy(BitUtils.intToBytes((int) seqNumberToDevice), 0, refPkt, 24, 4);
				//        System.arraycopy(BitUtils.intToBytes((int) ackNumberToDevice), 0, refPkt, 28, 4);

				refPkt[24] = (byte) (seqNumberToDevice >>> 24);
				refPkt[25] = (byte) (seqNumberToDevice >>> 16);
				refPkt[26] = (byte) (seqNumberToDevice >>> 8);
				refPkt[27] = (byte) seqNumberToDevice;

				refPkt[28] = (byte) (ackNumberToDevice >>> 24);
				refPkt[29] = (byte) (ackNumberToDevice >>> 16);
				refPkt[30] = (byte) (ackNumberToDevice >>> 8);
				refPkt[31] = (byte) ackNumberToDevice;

				refPkt[32] = 0x50; // header size 20
				refPkt[33] = 0; // no flags
				refPkt[34] = refPkt[35] = (byte) 0xff; // window size 0xffff
				refPkt[36] = refPkt[37] = refPkt[38] = refPkt[39] = 0; // chksum = 0 urgent pointer = 0

				//constructing the pseudo ip header
				System.arraycopy(refPkt, 12, pseudoHeader, 0, 8);
				pseudoHeader[8] = 0;
				pseudoHeader[9] = 6;

			}

		private void updatePacket(byte flags, long seqNr, long ackNr, byte[] payload, int payloadLen)
			{
				///ToDo: Remove this copy...
				if (payloadLen > 0)
					System.arraycopy(payload, 0, refPkt, 40, payloadLen);
				int totalLen = 40 + payloadLen;
				refPkt[2] = (byte) (totalLen >>> 8);
				refPkt[3] = (byte) totalLen;

				refPkt[24] = (byte) (seqNr >>> 24);
				refPkt[25] = (byte) (seqNr >>> 16);
				refPkt[26] = (byte) (seqNr >>> 8);
				refPkt[27] = (byte) seqNr;

				refPkt[28] = (byte) (ackNr >>> 24);
				refPkt[29] = (byte) (ackNr >>> 16);
				refPkt[30] = (byte) (ackNr >>> 8);
				refPkt[31] = (byte) ackNr;

				refPkt[33] = flags;

				refPkt[10] = refPkt[11] = 0;
				BitUtils.calcIPHeaderChecksum(refPkt);

				// tcp chksum
				pseudoHeader[10] = (byte) ((payloadLen + 20) >>> 8);
				pseudoHeader[11] = (byte) (payloadLen + 20);

				refPkt[36] = refPkt[37] = 0;
				BitUtils.computeTCPChecksum(pseudoHeader, refPkt, payloadLen);
			}

		public void handlePacket(IPv4Packet pkt/*Packet packet*/) throws IOException
			{

				byte[] rawPacket = pkt.getRawPacket();
				int packetHeaderSize = 20 + ((rawPacket[32] & 0xF0) >> 2);

				int payloadLen = pkt.getLength() - packetHeaderSize;

				// Exclude resends
				if (lastPushSeq >= BitUtils.getUnsignedInt(rawPacket, 24))
					{
						return;
					}

				if (payloadLen > 0 && status != TcpStatus.FIN_WAIT && os != null)
					{

						try
							{
								os.write(rawPacket, packetHeaderSize, payloadLen);
							}
						catch (Exception e)
							{
								//                System.out.println(this.TNAME+" PLEN: " + payloadLen);
								System.out.println("Status is: " + status.toString());
								e.printStackTrace();
								if (status == TcpStatus.ESTABLISHED || status == TcpStatus.SYN_RECEIVED)
									{
										sendFinAck();
										System.out.println("Sending FIN|ACK");
									}
							}

						lastPushSeq = BitUtils.getUnsignedInt(rawPacket, 24);

						ackNumberToDevice = lastPushSeq + payloadLen;
						sendAck();

					}

				//        System.out.println(TNAME+" HANDLING: " + packet.toString());

				// Flag handling
				if (pkt.isSYN())
					{
						// check if its the first SYN
						if (status == null)
							{
								//                refPacket = packet;
								initPacket(pkt.getRawPacket());

								m_inputstreamFromMix = m_anonTCPChannel.getInputStream();
								os = m_anonTCPChannel.getOutputStream();

								//                    System.out.println(TNAME+" SYN RECEIVED");
								status = TcpStatus.SYN_RECEIVED;
								sendSynAck();
							}

					}
				else if (pkt.isFIN())
					{
						if (status == TcpStatus.ESTABLISHED)
							{
								ackNumberToDevice++;
								status = TcpStatus.CLOSE_WAIT;
								sendFinAck();
								//                System.out.println(TNAME+" FIN/ACK RECEIVED, CHANGING STATUS TO CLOSE_WAIT");
							}
						else if (status == TcpStatus.FIN_WAIT)
							{
								if (!pkt.isACK())
									{
										ackNumberToDevice++;
										sendAck();
										status = TcpStatus.CLOSING;
									}
								else
									{
										ackNumberToDevice++;
										sendAck();
										close();
									}
							}
						else
							{
								sendRst();
								close();
							}
					}
				else if (pkt.isRST())
					{
						//            sendRst();
						close();
					}
				else if (pkt.isACK())
					{

						if (status == TcpStatus.SYN_RECEIVED)
							{
								status = TcpStatus.ESTABLISHED;
							}
						else if (status == TcpStatus.CLOSE_WAIT || status == TcpStatus.CLOSING)
							{
								close();
							}
						synchronized (m_oSyncWindowSize)
							{
								m_isentWindowPos = BitUtils.getUnsignedShort(rawPacket, 34) + BitUtils.getUnsignedInt(rawPacket, 28);
								m_oSyncWindowSize.notify();
							}

						// TODO: out-of-order packet verification?
					}
				else
					{
						System.out.println(m_strThreadName + " TCP: Problem");
						sendRst();
						close();
					}
			}

		private synchronized void sendPsh(byte[] payload, int payloadLen)
			{
				updatePacket((byte) (IPv4Packet.PSH | IPv4Packet.ACK), seqNumberToDevice, ackNumberToDevice, payload,
						payloadLen);
				seqNumberToDevice += payloadLen;

				int len = TCP_IP_HEADER_LENGTH + payloadLen;

				try
					{
						m_TunDevice.write(refPkt, 0, len);
					}
				catch (IOException e)
					{
						e.printStackTrace();
					}
			}

		private synchronized void sendSynAck()
			{
				updatePacket((byte) (IPv4Packet.SYN | IPv4Packet.ACK), seqNumberToDevice, ackNumberToDevice, null, 0);

				seqNumberToDevice++;

				try
					{
						m_TunDevice.write(refPkt, 0, TCP_IP_HEADER_LENGTH);
					}
				catch (IOException e)
					{
						e.printStackTrace();
					}
			}

		private synchronized void sendAck()
			{
				updatePacket((byte) IPv4Packet.ACK, seqNumberToDevice, ackNumberToDevice, null, 0);

				try
					{
						m_TunDevice.write(refPkt, 0, TCP_IP_HEADER_LENGTH);
					}
				catch (IOException e)
					{
						e.printStackTrace();
					}
			}

		private synchronized void sendFinAck()
			{
				updatePacket((byte) (IPv4Packet.FIN | IPv4Packet.ACK), seqNumberToDevice, ackNumberToDevice, null, 0);
				seqNumberToDevice++;

				if (status == TcpStatus.ESTABLISHED || status == TcpStatus.SYN_RECEIVED)
					{
						status = TcpStatus.FIN_WAIT;
					}

				try
					{
						m_TunDevice.write(refPkt, 0, TCP_IP_HEADER_LENGTH);
					}
				catch (IOException e)
					{
						e.printStackTrace();
					}
			}

		private synchronized void sendRst()
			{
				updatePacket((byte) IPv4Packet.RST, seqNumberToDevice, ackNumberToDevice, null, 0);

				try
					{
						m_TunDevice.write(refPkt, 0, TCP_IP_HEADER_LENGTH);
					}
				catch (IOException e)
					{
						e.printStackTrace();
					}
			}

		private void close()
			{
				m_bRun = false;
				m_anonTCPChannel.close();
				TcpStreamManager.getInstance().closeStream(m_dstIpPort);
			}

		@Override
		public void run()
			{
				m_strThreadName = Thread.currentThread().getName();
				//				int iMTUCheck=	m_TunDevice.getMTU()-100;
				byte[] packetIn = new byte[m_TunDevice.getMTU() - TCP_IP_HEADER_LENGTH];
				int bytesToRead;
				while (m_bRun)
					{
						try
							{
								// Read packet

								synchronized (m_oSyncWindowSize)
									{
										while ((m_isentWindowPos - seqNumberToDevice) <= 0 && m_bRun)
											{
												try
													{
														m_oSyncWindowSize.wait();
													}
												catch (InterruptedException e)
													{
														//e.printStackTrace();
													}
											}
										bytesToRead = packetIn.length;
										if ((m_isentWindowPos - seqNumberToDevice) < bytesToRead)
											{
												bytesToRead = (int) (m_isentWindowPos - seqNumberToDevice); //m_isentWindowSize can only be >0 here
											}
									}
								int bytesRead = m_inputstreamFromMix.read(packetIn);
								if (bytesRead > 0)
									{
										//                        System.out.println(TNAME+" BYTES READ: " + bytesRead);
										//									if(bytesRead>iMTUCheck)
										//										System.out.println("Recevied packet size: "+bytesRead);
										sendPsh(packetIn, bytesRead);
									}
								else if (bytesRead == -1)
									{
										if (status != TcpStatus.FIN_WAIT)
											{
												sendFinAck();
											}
										m_anonTCPChannel.close();
										m_bRun = false;
									}

							}
						catch (Exception e)
							{
								//							e.printStackTrace();
								if (status == TcpStatus.SYN_RECEIVED || status == TcpStatus.ESTABLISHED || !m_bRun)
									{
										sendRst();
									}
								close();
								return;
							}
					}
			}
	}