package anonvpn.anon_next.core.networking.TcpSocks;

import HTTPClient.HTTPClientSocketFactory;
import anon.AnonChannel;
import anon.client.AnonClient;
import anon.client.AnonTCPChannel;
import anonvpn.anon_next.core.CascadeConnectionManager;
import anonvpn.anon_next.core.entities.IPv4Packet;
import anonvpn.anon_next.core.networking.ITunneling;
import logging.LogHolder;
import logging.LogLevel;
import logging.LogType;

import java.io.IOException;
import java.util.HashSet;
import java.util.Hashtable;

/**
 * Created by nod on 8/7/17.
 */

final public class TcpStreamManager
	{

		private static TcpStreamManager instance = null;

		private ITunneling m_TunDevice;
		private DNSResolver m_dnsResolver;
		private Hashtable<Long, TcpStream> m_tcpStreamHashtable = new Hashtable<>();
		private AnonClient m_AnonClient;
		private HashSet<Integer> ignoreList = new HashSet<>();

		private byte[] refPkt = new byte[40];
		private byte[] pseudoHeader = new byte[12];

		private HTTPClientSocketFactory m_MixSocketFactory;

		protected TcpStreamManager()
			{
				// Singleton
			}

		public static TcpStreamManager getInstance()
			{
				if (instance == null)
					{
						instance = new TcpStreamManager();
					}
				return instance;
			}

		public void handlePacket(IPv4Packet bytes)
			{

				try
					{

						long key = constructLongKey(bytes.getRawPacket());

						if (m_tcpStreamHashtable.containsKey(key))
							{
								m_tcpStreamHashtable.get(key).handlePacket(bytes);
							}
						else if (bytes.isSYN())
							{

								if (!m_tcpStreamHashtable.containsKey(key))
									{

										// TODO: ignores muessten eigentlich auf domainnamen basieren
										int mIP = BitUtils.bytesToInt(bytes.getRawPacket(), 16);

										if (ignoreList.contains(mIP))
											return;

										// for debugging with local SOCKS server
										AnonTCPChannel chan = CascadeConnectionManager.USE_SOCKS_WITH_CASCADE
												? (AnonTCPChannel) m_AnonClient.createChannel(AnonChannel.TCP)
												: new AnonTcpChannelDummy(m_MixSocketFactory);

										TcpStream tcp = new TcpStream(key, chan, m_TunDevice);

										byte[] ip = new byte[4];
										System.arraycopy(bytes.getRawPacket(), IPv4Packet.DST_IP_POS, ip, 0, 4);
										String hn = null;

										if (CascadeConnectionManager.USE_FAKE_DNS)
											{
												hn = m_dnsResolver.getDomainName(BitUtils.bytesToInt(ip, 0));
											}

										if (hn == null && ip[0] != 10)
											{
												hn = BitUtils.getUnsignedByte(ip[0]) + ".";
												hn += BitUtils.getUnsignedByte(ip[1]) + ".";
												hn += BitUtils.getUnsignedByte(ip[2]) + ".";
												hn += BitUtils.getUnsignedByte(ip[3]);
											}

										try
											{
												chan.connect(hn, BitUtils.getUnsignedShort(bytes.getRawPacket(), 22));
											}
										catch (Exception e)
											{
												LogHolder.log(LogLevel.EXCEPTION,LogType.NET,"Sending reset because of failure in TCPStreamManger - connect AnonTCPChannel",e);
												sendRst(bytes);
												ignoreList.add(mIP);
												return;
											}

										tcp.handlePacket(bytes);
										new Thread(tcp, hn).start();
										m_tcpStreamHashtable.put(key, tcp);
									}
							}
						else if (/*bytes.isRST() || */bytes.getLength() == 40)
							{ // no need to reset if no data
								return;
							}
						else
							{
								sendRst(bytes);
								LogHolder.log(LogLevel.DEBUG,LogType.NET,"SENDING RESET 2");
							}
					}
				catch (Exception e)
					{
						sendRst(bytes);
						LogHolder.log(LogLevel.EXCEPTION,LogType.NET,"SENDING RESET 2",e);
					}
			}

		void closeStream(long l)
			{
				m_tcpStreamHashtable.remove(l);
				//        System.out.println("MAP LENGTH: " + tcpStreamHashtable.size());

			}

		private void sendRst(IPv4Packet iPv4Packet)
			{
				//        byte[] refPkt = new byte[40];
				System.arraycopy(iPv4Packet.getRawPacket(), 0, refPkt, 0, 40);
				// swap IPs
				System.arraycopy(iPv4Packet.getRawPacket(), 12, refPkt, 16, 4);
				System.arraycopy(iPv4Packet.getRawPacket(), 16, refPkt, 12, 4);
				// swap ports
				System.arraycopy(iPv4Packet.getRawPacket(), 20, refPkt, 22, 2);
				System.arraycopy(iPv4Packet.getRawPacket(), 22, refPkt, 20, 2);

				// reset IP-flags
				refPkt[6] = 0;
				refPkt[3] = 40; //Toal Len --> set to 40 for easier checkum update;
				refPkt[4] = 0; // Identification
				refPkt[5] = 0; // Identification
				refPkt[6] = 0; // Flags & Offset
				refPkt[7] = 0; // Offset
				refPkt[8] = 64; // TTL

				refPkt[10] = 0; // For Header Checksum calculation (for len = 40)
				refPkt[11] = 0; // For Header Checksum calculation (for len = 40)

				// TCP header

				long ackNumberToDevice = BitUtils.getUnsignedInt(BitUtils.bytesToInt(refPkt, 24)) + 1;
				long seqNumberToDevice = BitUtils.getUnsignedInt(BitUtils.bytesToInt(refPkt, 28));

				//        System.arraycopy(BitUtils.intToBytes((int) seqNumberToDevice), 0, refPkt, 24, 4);
				//        System.arraycopy(BitUtils.intToBytes((int) ackNumberToDevice), 0, refPkt, 28, 4);

				refPkt[24] = (byte) (seqNumberToDevice >>> 24);
				refPkt[25] = (byte) (seqNumberToDevice >>> 16);
				refPkt[26] = (byte) (seqNumberToDevice >>> 8);
				refPkt[27] = (byte) seqNumberToDevice;

				refPkt[28] = (byte) (ackNumberToDevice >>> 24);
				refPkt[29] = (byte) (ackNumberToDevice >>> 16);
				refPkt[30] = (byte) (ackNumberToDevice >>> 8);
				refPkt[31] = (byte) ackNumberToDevice;

				refPkt[32] = 0x50; // header size 20
				refPkt[33] = (byte) IPv4Packet.RST;
				refPkt[34] = refPkt[35] = (byte) 0xff; // window size 0xffff
				refPkt[36] = refPkt[37] = refPkt[38] = refPkt[39] = 0; // chksum = 0 urgent pointer = 0

				//constructing the pseudo ip header
				System.arraycopy(iPv4Packet.getRawPacket(), 12, pseudoHeader, 0, 8); // src and dst ips
				pseudoHeader[8] = 0; // null-byte
				pseudoHeader[9] = 6; // protocol
				pseudoHeader[10] = 0; // protocol
				pseudoHeader[11] = 20; // protocol

				refPkt[10] = refPkt[11] = 0; // clear ip checksum
				BitUtils.calcIPHeaderChecksum(refPkt);

				BitUtils.computeTCPChecksum(pseudoHeader, refPkt, 0);

				try
					{
						m_TunDevice.write(refPkt, 0, 40);
					}
				catch (IOException e)
					{
						e.printStackTrace();
					}
			}

		private long constructLongKey(byte[] packet)
			{
				byte[] dstIpPorts = new byte[8];
				System.arraycopy(packet, IPv4Packet.DST_IP_POS, dstIpPorts, 0, 4);
				dstIpPorts[4] = packet[22];
				dstIpPorts[5] = packet[23];
				dstIpPorts[6] = packet[20];
				dstIpPorts[7] = packet[21];

				return BitUtils.bytesToLong(dstIpPorts, 0);
			}

		public void setTunDevice(ITunneling a_tunDevice)
			{
				m_TunDevice = a_tunDevice;
			}

		public void setAnonClient(AnonClient a_AnonClient)
			{
				m_AnonClient = a_AnonClient;
			}

		public void setMixSocketFactory(HTTPClientSocketFactory a_mixSocketFactory)
			{
				m_MixSocketFactory = a_mixSocketFactory;
			}

		public void setDnsResolver(DNSResolver a_dns)
			{
				m_dnsResolver = a_dns;
			}
	}
