package anonvpn.anon_next.core.networking.TcpSocks;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.SocketException;

import anon.client.IAnonUDPChannel;
import anonvpn.anon_next.core.entities.IPv4Packet;
import anonvpn.anon_next.core.networking.ITunneling;
//import mybouncycastle.org.bouncycastle.util.encoders.Hex;
import logging.LogHolder;
import logging.LogLevel;
import logging.LogType;

/*
 * Created by nod on 9/10/17.
 */

final class UdpStream implements Runnable
	{

		private byte[] downstreamPacket = new byte[0xFFFF + 28];

		private final static int UDP_IP_HEADER_LENGTH = 28;
		private final long dstIpPort;

		private volatile boolean m_bRun = true;

		private final IAnonUDPChannel m_anonudpChannel;
		private final ITunneling m_TunDevice;
		private volatile long lastPacket;

		private long checkSumTemplate;

		//public OutputStream os;
		private UdpStreamManager m_UdpStreamManager;

		public UdpStream(UdpStreamManager manager,int srcIP, long dstIpPorts, IAnonUDPChannel chan, ITunneling mTunDevice)
			{
				this.dstIpPort = dstIpPorts;
				m_anonudpChannel = chan;
				m_TunDevice = mTunDevice;
				m_UdpStreamManager=manager;

				// Note: this initialise the downstream packet template --> so we have to change source <--> destination
				int dstIP = (int) ((dstIpPorts >> 32) & 0x00FFFFFFFFL);
				int srcPort = (int) ((dstIpPorts >> 16) & 0x00FFFFL);
				int dstPort = (int) ((dstIpPorts) & 0x00FFFFL);
				initDownstreamPacket(dstIP, srcIP, dstPort, srcPort);
			}

		private void initDownstreamPacket(int srcIP, int dstIP, int srcPort, int dstPort)
			{
				//Create IP packet template by setting all the header fields, so that only len and checksum need to be updated
				downstreamPacket[0] = 0x45; // IP version and header size
				downstreamPacket[1] = 0; //TOS
				downstreamPacket[2] = 0; //Total Len;
				downstreamPacket[3] = 28; //Toal Len --> set to 28 for easier checksum update;
				downstreamPacket[4] = 0; // Identification
				downstreamPacket[5] = 0; // Identification
				downstreamPacket[6] = 0; // Flags & Offset
				downstreamPacket[7] = 0; // Offset
				downstreamPacket[8] = 64; // TTL
				downstreamPacket[9] = 17; // Protocol = UDP

				downstreamPacket[10] = 0; // For Header Checksum calculation (for len = 28)
				downstreamPacket[11] = 0; // For Header Checksum calculation (for len = 28)

				downstreamPacket[12] = (byte) (srcIP >> 24); // src IP
				downstreamPacket[13] = (byte) ((srcIP >> 16) & 0x00FF); // src IP
				downstreamPacket[14] = (byte) ((srcIP >> 8) & 0x00FF); // src IP
				downstreamPacket[15] = (byte) (srcIP & 0x00FF); // src IP
				downstreamPacket[16] = (byte) (dstIP >> 24); // src IP
				downstreamPacket[17] = (byte) ((dstIP >> 16) & 0x00FF); // src IP
				downstreamPacket[18] = (byte) ((dstIP >> 8) & 0x00FF); // src IP
				downstreamPacket[19] = (byte) (dstIP & 0x00FF); // src IP
				downstreamPacket[20] = (byte) ((srcPort >> 8) & 0x00FF); // src IP
				downstreamPacket[21] = (byte) (srcPort & 0x00FF); // src IP
				downstreamPacket[22] = (byte) ((dstPort >> 8) & 0x00FF); // src IP
				downstreamPacket[23] = (byte) (dstPort & 0x00FF); // src IP

				downstreamPacket[IPv4Packet.UDP_CHECKSUM_FIELD_POSITION] = 0;
				downstreamPacket[IPv4Packet.UDP_CHECKSUM_FIELD_POSITION + 1] = 0;

				checkSumTemplate = calcIPHeaderChecksum(downstreamPacket);
			}

		/**
		 * Calculate and set the IP header checksum.
		 * 
		 * @ret checksum
		 */
		public long calcIPHeaderChecksum(byte[] buf)
			{

				long sum = 0;
				long data;

				// Handle all pairs
				for (int len = 0; len < 20; len += 2)
					{
						// Corrected to include @Andy's edits and various comments on Stack Overflow
						data = (((buf[len] << 8) & 0xFF00) | ((buf[len + 1]) & 0xFF));
						sum += data;
						// 1's complement carry bit correction in 16-bits (detecting sign extension)
						if ((sum & 0xFFFF0000) > 0)
							{
								sum = sum & 0xFFFF;
								sum += 1;
							}
					}

				// Final 1's complement value correction to 16-bits
				sum = ~sum;
				sum = sum & 0xFFFF;

				buf[IPv4Packet.IP_CHECKSUM_FIELD_POSITION] = (byte) ((sum >> 8) & 0x0FF);
				buf[IPv4Packet.IP_CHECKSUM_FIELD_POSITION + 1] = (byte) (sum & 0x00FF);

				return sum;

			}

		/**
		 * Updates and sets the IP and UDP header checksums.
		 * 
		 * @checkSumTemplate the IP checksum for the template ( total len = 28)
		 */
		void updateHeaderChecksums(byte[] packet, int payloadLen, long checkSumTemplate)
			{
				// for formmular see RFC1624
				long newCheckSum = checkSumTemplate - payloadLen;

				packet[IPv4Packet.IP_CHECKSUM_FIELD_POSITION] = (byte) ((newCheckSum >> 8) & 0x00FF);
				packet[IPv4Packet.IP_CHECKSUM_FIELD_POSITION + 1] = (byte) (newCheckSum & 0x00FF);
				//            packet[IPv4Packet.UDP_CHECKSUM_FIELD_POSITION] = 0;
				//            packet[IPv4Packet.UDP_CHECKSUM_FIELD_POSITION+1] = 0;
			}

		public void handlePacket(byte[] packet) throws IOException
			{
				try
					{
						int payloadLen = (packet[IPv4Packet.UDP_LEN_FIELD_POSITION] & 0x00FF);
						payloadLen <<= 8;
						payloadLen |= (packet[IPv4Packet.UDP_LEN_FIELD_POSITION + 1] & 0x00FF);
						payloadLen -= 8;
						m_anonudpChannel.writePacket(packet, IPv4Packet.UDP_PAYLOAD_START_POSITION, payloadLen);
						//os.write(packet, IPv4Packet.UDP_PAYLOAD_START_POSITION, payloadLen);
						//os.flush();
						lastPacket = System.currentTimeMillis();
					}
				catch (Exception e)
					{
						LogHolder.log(LogLevel.DEBUG,LogType.NET,"Exception in UDPStream.handlePacket(): " + e.getMessage(),e);
					}
			}

		private synchronized void sendPacketToDevice(byte[] packet, int payloadLen)
			{
				// Update checksumms and len information
				int t = payloadLen + UDP_IP_HEADER_LENGTH; // total length
				packet[IPv4Packet.IP_LEN_FIELD_POSITION] = (byte) (t >> 8);
				packet[IPv4Packet.IP_LEN_FIELD_POSITION + 1] = (byte) (t & 0x0FF);
				t = payloadLen + 8;
				packet[IPv4Packet.UDP_LEN_FIELD_POSITION] = (byte) (t >> 8);
				packet[IPv4Packet.UDP_LEN_FIELD_POSITION + 1] = (byte) (t & 0x0FF);

				//updateHeaderChecksums(packet,payloadLen, checkSumTemplate);
				packet[IPv4Packet.IP_CHECKSUM_FIELD_POSITION] = 0;
				packet[IPv4Packet.IP_CHECKSUM_FIELD_POSITION + 1] = 0;

				/*        byte tmByte[]={0x45,0x00 ,0x00 ,(byte)0x9C ,0x4D ,(byte)0xFB ,0x40 ,0x00 ,0x40 ,0x11 ,(byte)0xD8 ,0x53 ,0x0A ,0x00 ,0x00 ,0x01 ,0x0A ,0x00 ,0x00 ,0x02};
				tmByte[IPv4Packet.IP_CHECKSUM_FIELD_POSITION] =0;
				tmByte[IPv4Packet.IP_CHECKSUM_FIELD_POSITION+1] =0;
				calcIPHeaderChecksum(tmByte);
				  */
				calcIPHeaderChecksum(packet);
				int len = UDP_IP_HEADER_LENGTH + payloadLen;
				try
					{
						m_TunDevice.write(packet, 0, len);

						//if(packet.udpHeader.sourcePort==53)
							{
								//System.out.println("DNS UDP packet content:");
								//System.out.println(Hex.toHexString(packet, 0, len));
							}

					}
				catch (IOException e)
					{
						e.printStackTrace();
					}
			}

		public void close(boolean bRemoveFromTimingList)
			{
				m_anonudpChannel.close();
				if (bRemoveFromTimingList)
					m_UdpStreamManager.removeStream(dstIpPort);
				m_bRun = false;
			}

		public long getLastTimeStamp()
			{
				return lastPacket;
			}

		@Override
		public void run()
			{
				while (m_bRun)
					{
						try
							{

								int bytesRead = m_anonudpChannel.readPacket(downstreamPacket, IPv4Packet.UDP_PAYLOAD_START_POSITION, 0xFFFF);

								if (bytesRead > 0)
									{
										//                        System.out.println(TNAME+" BYTES READ: " + bytesRead);
										sendPacketToDevice(downstreamPacket, bytesRead);
										lastPacket = System.currentTimeMillis();
									}
								else
									{

										if (bytesRead == -1)
											{
												close(true);
											}

										return;
									}

							}
						catch (Exception e)
							{
								close(true);
							}
					}
			}
	}
