package anonvpn.anon_next.core.networking.TcpSocks;

import java.io.IOException;
import java.util.Hashtable;

import HTTPClient.HTTPClientSocketFactory;
import anon.AnonChannel;
import anon.client.AnonClient;
import anon.client.IAnonUDPChannel;
import anonvpn.anon_next.core.CascadeConnectionManager;
import anonvpn.anon_next.core.entities.IPv4Packet;
import anonvpn.anon_next.core.networking.ITunneling;
import logging.LogHolder;
import logging.LogLevel;
import logging.LogType;

/**
 * Created by nod on 9/10/17.
 */

final public class UdpStreamManager
	{
		private ITunneling m_TunDevice;
		private DNSResolver dns;
		private Hashtable<Long, UdpStream> udpStreamHashtable = new Hashtable<>();
		private AnonClient mAnonClient;
		private UdpTimer udpTimer;

		private HTTPClientSocketFactory mMixSocketFactory;

		private int m_srcIP;

		public UdpStreamManager()
			{
				udpTimer = new UdpTimer();
				udpTimer.start();
			}

		public void setSrcIP(int srcIP)
			{
				m_srcIP = srcIP;

			}

/*		public static UdpStreamManager getInstance()
			{
				if (instance == null)
					instance = new UdpStreamManager();
				return instance;
			}
*/
		
		public void close()
		{
			if(udpTimer!=null)
				{
					udpTimer.stop();
					udpTimer=null;
				}
		}
		
		public void handlePacket(byte[] packet)
			{

				try
					{
						//key is Dst Addr + Src Port + Dst Port
						long key = BitUtils.bytesToLong(packet, IPv4Packet.DST_IP_POS);

						if (udpStreamHashtable.containsKey(key))
							{
								udpStreamHashtable.get(key).handlePacket(packet);
							}

						else
							{
								LogHolder.log(LogLevel.DEBUG,LogType.NET,"Creating new VPN-UDP-Stream");
								IAnonUDPChannel chan = (IAnonUDPChannel) mAnonClient.createChannel(CascadeConnectionManager.UDP_ANON_CHANNEL_TYPE);
								if(chan==null)
									{
										LogHolder.log(LogLevel.ERR,LogType.NET,"Could not create AN.ON UDP channel!");
										return;
									}
								
								UdpStream udp = new UdpStream(this,m_srcIP, key, chan, m_TunDevice);

								//byte[] ip = IPv4Packet.getpacket.ip4Header.destinationAddress.getAddress();
								String hn = null;

								if (CascadeConnectionManager.USE_FAKE_DNS)
									{
										hn = dns.getDomainName(BitUtils.bytesToInt(packet, IPv4Packet.DST_IP_POS));
									}

								if (hn == null/* && ip[0] != 10*/)
									{
										hn = ((int)packet[IPv4Packet.DST_IP_POS]&0x00FF) + ".";
										hn += ((int)packet[IPv4Packet.DST_IP_POS+1]&0x00FF) + ".";
										hn += ((int)packet[IPv4Packet.DST_IP_POS+2]&0x00FF) + ".";
										hn += ((int)packet[IPv4Packet.DST_IP_POS+3]&0x00FF);
									}

								int port = BitUtils.getUnsignedShort(packet, IPv4Packet.DST_PORT_POS);
								chan.setDestination(hn, port);
								port = BitUtils.getUnsignedShort(packet, IPv4Packet.SRC_PORT_POS);
								chan.setSourcePort(port);
								//udp.os = chan.getOutputStream();

								udpStreamHashtable.put(key, udp);
//								System.out.println("Number of UDP streams: " + udpStreamHashtable.size());
								new Thread(udp, hn).start();
								udp.handlePacket(packet);
								udpTimer.addUdpStream(udp);

								//        System.out.println("MAP LENGTH: " + udpStreamHashtable.size());
							}
					}
				catch (IOException e)
					{
						e.printStackTrace();
					}
			}

		// this does not close the stream. Use UdpStream.close() to close stream properly
		void removeStream(long l)
			{
				if(udpTimer!=null)
					udpTimer.removeUdpStream(udpStreamHashtable.get(l));
				udpStreamHashtable.remove(l);
				//        System.out.println("MAP LENGTH: " + udpStreamHashtable.size());

			}

		public void setTunDevice(ITunneling tunDevice)
			{
				this.m_TunDevice = tunDevice;
			}

		public void setAnonClient(AnonClient mAnonClient)
			{
				this.mAnonClient = mAnonClient;
			}

		public void setMixSocketFactory(HTTPClientSocketFactory mixSocketFactory)
			{
				this.mMixSocketFactory = mixSocketFactory;
			}

		public void setDnsResolver(DNSResolver dns)
			{
				this.dns = dns;
			}

	}
