package pcap;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

/**
 * Created by sk13 on 12.10.2017.
 */

public class PCAPWriter
	{
		OutputStream m_Out=null;
		boolean m_bAutoFlush=false;

		public PCAPWriter(String filename) throws Exception
			{
				setOutputStream(new FileOutputStream(filename));
				}
		
		public PCAPWriter(OutputStream out) throws Exception 
			{
				setOutputStream(out);
			}
		
		public void enableAutoFlush(boolean b)
		{
			m_bAutoFlush=b;
		}

		public synchronized void setOutputStream(OutputStream out) throws Exception
		{
			close();
			m_Out=out;
			if(m_Out!=null)
				{
					writeGlobalHeader();
				}
		}
		
		public synchronized void dumpIPPacket(byte[] packet,int pos,int len) throws Exception
			{
				writePCAPPacket(packet,pos,len);
			}

		public void close()
			{
				try
					{
						if(m_Out!=null)
							{
								m_Out.close();
							}
					}
				catch (IOException e)
					{
						e.printStackTrace();
					}
				m_Out=null;
			}

		/**
		 * typedef struct pcap_hdr_s {
		 guint32 magic_number;   // magic number 0xa1b2c3d4
		guint16 version_major;  // major version number 2
		guint16 version_minor;  // minor version number  4
		gint32  thiszone;       // GMT to local correction 0
		guint32 sigfigs;        // accuracy of timestamps 0
		guint32 snaplen;        // max length of captured packets, in octets 0xFFFF
		guint32 network;        // data link type //LINKTYPE_IPV4	228
	} pcap_hdr_t;
		 */
		private void writeGlobalHeader() throws Exception
			{
				byte header[]=new byte[24];
				header[0]=(byte)0xa1;
				header[1]=(byte)0xb2;
				header[2]=(byte)0xc3;
				header[3]=(byte)0xd4;

				header[4]=(byte)0;
				header[5]=(byte)2;

				header[6]=(byte)0;
				header[7]=(byte)4;

				header[18]=(byte)0xFF;
				header[19]=(byte)0xFF;

				header[23]=(byte)228;

				m_Out.write(header,0,header.length);
			}

		/*
		typedef struct pcaprec_hdr_s {
		guint32 ts_sec;         // timestamp seconds //
		guint32 ts_usec;        // timestamp microseconds //
		guint32 incl_len;       // number of octets of packet saved in file //
		guint32 orig_len;       // actual length of packet //
	} pcaprec_hdr_t;*/
		private void writePCAPPacket(byte[]data,int pos,int len) throws Exception
			{
				if(m_Out==null)
					{
						return;
					}
				byte header[]=new byte[16];

				long t=System.currentTimeMillis();
				long s=t/1000;
				header[0]=(byte)((s>>24)&0x0FF);
				header[1]=(byte)((s>>16)&0x0FF);
				header[2]=(byte)((s>>8)&0x0FF);
				header[3]=(byte)((s)&0x0FF);
				s=(t%1000)*1000;
				header[4]=(byte)((s>>24)&0x0FF);
				header[5]=(byte)((s>>16)&0x0FF);
				header[6]=(byte)((s>>8)&0x0FF);
				header[7]=(byte)((s)&0x0FF);
				
				header[8]=(byte)((len>>24)&0x0FF);
				header[9]=(byte)((len>>16)&0x0FF);
				header[10]=(byte)((len>>8)&0x0FF);
				header[11]=(byte)((len)&0x0FF);

				header[12]=header[8];
				header[13]=header[9];
				header[14]=header[10];
				header[15]=header[11];
				m_Out.write(header,0,header.length);
				m_Out.write(data,pos,len);
				if(m_bAutoFlush)
					{
						m_Out.flush();
					}
			}

	}
