package anonvpn.anon_next.core;

import java.net.ConnectException;
import java.util.LinkedList;
import java.util.List;

import anon.AnonChannel;
import anon.AnonServerDescription;
import anon.AnonServiceEventListener;
import anon.IServiceContainer;
import anon.client.AnonClient;
import anon.error.AnonServiceException;
import anon.error.ServiceInterruptedException;
import anon.infoservice.MixCascade;
import anon.terms.TermsAndConditionConfirmation;

/**
 * Created by martin on 9/4/17.
 */

public class TestAnonClient extends AnonClient {

    public MixCascade mUsedCascade = null;
    public List<TestTCPChannel> mTestTCPChannels = new LinkedList<>();
    public List<TestUDPChannel> mTestUDPChannels = new LinkedList<>();
    public List<TestVPNChannel> mTestVPNChannels = new LinkedList<>();
    public boolean initialized = false;

    private List<AnonServiceEventListener> mListeners = new LinkedList<AnonServiceEventListener>();

    public static int numOfInitializationsToFail = 0;

    @Override
    public void initialize(final AnonServerDescription a_mixCascade,
                           final IServiceContainer a_serviceContainer,
                           final TermsAndConditionConfirmation termsConfirmation,
                           final boolean a_bIsReconnect) throws AnonServiceException {

        if (numOfInitializationsToFail > 0) {

            numOfInitializationsToFail -= 1;
            throw new ServiceInterruptedException(a_mixCascade);

        } else {

            mUsedCascade = ((MixCascade) a_mixCascade);
            initialized = true;
        }
    }

    @Override
    public AnonChannel createChannel(int a_type) throws ConnectException {
        AnonChannel result = null;
        switch (a_type) {
            case AnonChannel.VPN:
                result = new TestVPNChannel();
                mTestVPNChannels.add((TestVPNChannel) result);
                break;
            case AnonChannel.TCP:
                result = new TestTCPChannel(new TestChannel());
                mTestTCPChannels.add((TestTCPChannel) result);
                break;
            case AnonChannel.UDP:
                result = new TestUDPChannel(new TestChannel());
                mTestUDPChannels.add((TestUDPChannel) result);
                break;
        }
        return result;

    }

    @Override
    public boolean isConnected() {
        return true;
    }

    @Override
    public void shutdown(boolean resetTransferredBytes) {
        initialized = false;
        for (AnonServiceEventListener listener : mListeners) {
            listener.disconnected();
        }
    }

    @Override
    public void addEventListener(AnonServiceEventListener a_eventListener) {
        mListeners.add(a_eventListener);
    }
}
