package anonvpn.anon_next.core;

import java.net.Inet4Address;
import java.nio.ByteBuffer;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

public class TestPacket {

    private static short mIDCounter = 1;
    private static Random mRNG = new Random();

    public static ByteBuffer generateIPPacket(short length, int protocol, Inet4Address src, Inet4Address dst) {

        ByteBuffer result = ByteBuffer.allocate(length);
        result.put((byte) (4 << 4 | 5)); // IP version and header length
        result.put((byte) 0); // ToS
        result.putShort(length);
        result.putShort(mIDCounter++);
        result.put((byte) 0); //Flags
        result.put((byte) 0); // Fragment offset
        result.put((byte) 40); // TTL
        result.put((byte) protocol); // Protocol
        result.putShort((short) 0); // Header checksum -> TODO
        result.put(src.getAddress());
        result.put(dst.getAddress());

        return result;
    }

    public static ByteBuffer generateTCPSynPacket(Inet4Address src, Inet4Address dst, short srcPort, short dstPort) {
        ByteBuffer result = generateIPPacket((short) 60, 6, src, dst);

        // Set source port
        result.putShort(srcPort);

        // Set destination port
        result.putShort(dstPort);

        // Set sequence number
        result.putInt((int) mRNG.nextInt());

        // Set SYN flag
        result.put(33, (byte) 2);

        return result;
    }

    public static ByteBuffer generateDNSPacket(Inet4Address src, Inet4Address dst, short srcPort, short dstPort, String domain) {
        short length = (short) (40 + domain.length() + 6);
        ByteBuffer result = generateIPPacket(length, 17, src, dst);

        // Set source port
        result.putShort(srcPort);

        // Set destination port
        result.putShort(dstPort);

        // Set length
        result.putShort((short)(length - 20));

        // Set checksum
        result.putShort((short) 0);

        // Set request ID (2 bytes)
        result.putShort((short) 1000);

        // Set flags (2 bytes)
        result.putShort((short) (1 >> 9));

        // Set num queries (2 bytes)
        result.putShort((short) 1);

        // Set answers (2 bytes)
        result.putShort((short) 0);

        // Set authority answers (2 bytes)
        result.putShort((short) 0);

        // Set additional answers (2 bytes)
        result.putShort((short) 0);

        // Append domain name
        List<Byte> byteBuf = new LinkedList<Byte>();
        byte[] bytes = domain.getBytes();
        for (int i = 0; i < bytes.length; i++) {
            if (bytes[i] == '.' && byteBuf.size() > 0) {
                result.put((byte) byteBuf.size());
                for (Byte currentByte : byteBuf) {
                    result.put(currentByte);
                }
                byteBuf.clear();
            } else {
                byteBuf.add(bytes[i]);
            }
        }

        if (byteBuf.size() > 0) {
            result.put((byte) byteBuf.size());
            for (Byte currentByte : byteBuf) {
                result.put(currentByte);
            }
            byteBuf.clear();
        }
        result.put((byte) 0);

        // Append type (2 bytes)
        result.putShort((short) 1);

        // Append class (2 bytes)
        result.putShort((short) 1);
        return result;
    }

    public static ByteBuffer generateTCPSynAckPacket(int sequenceNumber) {
        return null;
    }

}
