package anonvpn.anon_next.core.networking.mocks;

import java.io.IOException;
import java.io.InputStream;
import java.util.concurrent.Semaphore;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

public class TestInputStream extends InputStream {

    Semaphore mReadSemaphore = new Semaphore(1);
    Semaphore mWriteSemaphore = new Semaphore(1); // Consider using synchronized for access to the mData member
    byte[] mData = null;
    int mDataIdx = 0;
    boolean mClosed = false;

    public TestInputStream() {
        try {
            mReadSemaphore.acquire();
        } catch (InterruptedException e) {
            System.out.println("Failed to initialize TestInputStream: " + e.getMessage());
        }
    }

    public void setData(byte[] data) {
        mWriteSemaphore.acquireUninterruptibly();
        mData = data;
        mDataIdx = 0;
        mWriteSemaphore.release();
        if (mReadSemaphore.availablePermits() == 0) {
            mReadSemaphore.release();
        }
    }

    public void close() {
        // Make sure no one is currently reading from the stream
        mWriteSemaphore.acquireUninterruptibly();
        mClosed = true;
        mReadSemaphore.release();
        mWriteSemaphore.release();
    }

    @Override
    public int read() throws IOException {
        int result = -1;
        try {
            mReadSemaphore.acquire();
            mWriteSemaphore.acquire();
            if (mClosed) {
                throw new IOException("Stream got closed!");
            }
            if (mData != null) {
                result = mData[mDataIdx];
                mDataIdx++;
            }
            mWriteSemaphore.release();
            if (mDataIdx < mData.length) {
                mReadSemaphore.release();
            }
            return result;
        } catch (InterruptedException e) {
            throw new IOException("Call to read() got interrupted while waiting for resources!");
        }
    }

}
