package anonvpn.anon_next.android.service.foreground;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Handler;
import android.os.IBinder;
import androidx.core.app.NotificationCompat;
import android.util.Log;

import anon.util.Util;
import anonvpn.anon_next.android.AndroidVpnService;
import anonvpn.anon_next.android.R;
import anonvpn.anon_next.android.service.gui.AndroidGUIInterface;
import anonvpn.anon_next.android.ui.main.MainActivity;
import anonvpn.anon_next.core.gui.IVPNInterface;
import anonvpn.anon_next.core.gui.VPNState;


public class ForegroundService extends Service
	{
		private static final String LOG_TAG = "ForegroundService";
		private static final int NOTIFICATION_ID =1 ;

		String cascade_ip = "";
		String cascade_location = "";
		String cascade_name = "";
		IVPNInterface.RxTx m_RxTx= IVPNInterface.RxTx.ALL_ZERO;
		VPNState conn_state = VPNState.DISABLED;

		private final int TRAFFIC_STATS_DELAY = 3000;
		private final Handler mTrafficStatisticsHandler = new Handler();
		private final GetTrafficStatisticsTask mTrafficStatisticsTask = new GetTrafficStatisticsTask();

		private ServiceConnection mVPNConnection = null;
		private AndroidGUIInterface mServiceInterface = null;

		private NotificationManager m_NM;
		private final String NOTIFICATION_CHANNEL_ID="ANONguardNotifications";

		private class GetTrafficStatisticsTask implements Runnable
			{
				public boolean isScheduled = false;

				public void run()
					{

						// This tasks starts right after the VPN service gets started.
						// Therefore, the service might still be starting (hence, the interface is not ready)
						// when this run-procedure gets called.
						if (mServiceInterface != null)
							{
								m_RxTx= mServiceInterface.getRxTx();
								showNotification();
								mTrafficStatisticsHandler.postDelayed(this, TRAFFIC_STATS_DELAY);
							}
					}
			}

		protected class VPNServiceConnection implements ServiceConnection
			{

				@Override
				public void onServiceConnected(ComponentName name, IBinder service)
					{
						Log.e(LOG_TAG, "Connected to service!");
						mServiceInterface = (AndroidGUIInterface) service;
						mTrafficStatisticsHandler.post(mTrafficStatisticsTask);
						mTrafficStatisticsTask.isScheduled = true;
					}

				@Override
				public void onServiceDisconnected(ComponentName name)
					{
						Log.e(LOG_TAG, "Disconnected from service!");
						mTrafficStatisticsHandler.removeCallbacks(mTrafficStatisticsTask);
						mTrafficStatisticsTask.isScheduled = false;
						mServiceInterface = null;

					}
			}

		@Override
		public void onCreate()
			{
				super.onCreate();
				m_NM = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);
				if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {


					NotificationChannel mChannel = new NotificationChannel(NOTIFICATION_CHANNEL_ID, "ANONguard", NotificationManager.IMPORTANCE_LOW);
					mChannel.enableLights(false);
					mChannel.enableVibration(false);
					//mChannel.setLockscreenVisibility(false);
					mChannel.setShowBadge(false);
					mChannel.setSound(null, null);
					m_NM.createNotificationChannel(mChannel);
				}



				Log.i(LOG_TAG, "Received Create Foreground Intent.");
			}

		@Override
		public int onStartCommand(Intent intent, int flags, int startId)
			{
//        if (intent.getAction().equals(Constants.ACTION.STARTFOREGROUND_ACTION)) {
				Log.i(LOG_TAG, "Received Start Foreground Intent.");

				if (intent != null)
					{
						cascade_ip = intent.getStringExtra("ip");
						cascade_location = intent.getStringExtra("location");
						cascade_name = intent.getStringExtra("name");
						conn_state = VPNState.valueOf(intent.getStringExtra("conn_state"));
					}
				showNotification();
				return START_STICKY;
			}

		private int getIconError()
			{
				boolean useWhiteIcon = (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.LOLLIPOP);
				return useWhiteIcon ? R.drawable.anon_jap_noconnection_silhouette : R.drawable.anon_jap_noconnection;
			}

		private int getIconWarning()
			{
				boolean useWhiteIcon = (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.LOLLIPOP);
				return useWhiteIcon ? R.drawable.anon_jap_warning_silhouette : R.drawable.anon_jap_warning;
			}

		private int getIconOK()
			{
				boolean useWhiteIcon = (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.LOLLIPOP);
				return useWhiteIcon ? R.drawable.anon_jap_silhouette : R.drawable.anon_jap;
			}

		private void showNotification()
			{

				NotificationCompat.Builder mBuilder = new NotificationCompat.Builder(this,NOTIFICATION_CHANNEL_ID).setContentTitle(getString(R.string.foregroundservice_title));
				switch (conn_state) {

					case DISABLED:
						mBuilder.setSmallIcon(getIconError()).setContentText(getString(R.string.foregroundservice_desc_vpn_disabled));

						if (mVPNConnection != null)
						{
							disconnectVPNService();
						}
						break;

					case WAITING:

						mBuilder.setSmallIcon(getIconWarning()).setContentText(getString(R.string.foregroundservice_desc_cascade_error));

						break;

					case CONNECTED:
						String message = getString(R.string.foregroundservice_desc_active_name) +
								cascade_name +  getString(R.string.foregroundservice_desc_active_location) +
								cascade_location +  getString(R.string.foregroundservice_desc_active_ip) +
								cascade_ip +  getString(R.string.foregroundservice_desc_active_Rx)  +
								Util.formatBytesValueWithUnit(m_RxTx.m_RxTotal)+"\n" +
								getString(R.string.foregroundservice_desc_active_Tx) + Util.formatBytesValueWithUnit(m_RxTx.m_TxTotal) ;
						mBuilder.setSmallIcon(getIconOK()).setStyle(new NotificationCompat.BigTextStyle().bigText(message)).setContentText(message);

						if (mVPNConnection == null)
						{
							connectToVPNService();
						}
						break;

					case CONNECTING:

						mBuilder.setSmallIcon(getIconWarning()).setContentText(getString(R.string.foregroundservice_desc_connecting));

						break;

					case DISCONNECTED:

						mBuilder.setSmallIcon(getIconWarning()).setContentText(getString(R.string.foregroundservice_desc_network));

						break;

				}

				// Creates an explicit intent for an Activity in your app
				Intent resultIntent = new Intent(this, MainActivity.class);

				PendingIntent resultPendingIntent = PendingIntent.getActivity(this, 0, resultIntent, PendingIntent.FLAG_UPDATE_CURRENT|PendingIntent.FLAG_IMMUTABLE);

				mBuilder.setContentIntent(resultPendingIntent);

				Notification notification = mBuilder.build();
				startForeground(NOTIFICATION_ID, notification);

			}

		@Override
		public void onDestroy()
			{
				super.onDestroy();
				Log.i(LOG_TAG, "In onDestroy");

				if (mServiceInterface != null)
					{
						disconnectVPNService();
					}
//        Toast.makeText(this, "Service Destroyed!", Toast.LENGTH_SHORT).show();
				m_NM.cancel(NOTIFICATION_ID);
			}

		@Override
		public IBinder onBind(Intent intent)
			{
				// Used only in case if services are bound (Bound Services).
				return null;
			}

		private void connectToVPNService()
			{

				if (mVPNConnection != null)
					{
						throw new AssertionError("Redundant call to connectToVPNService()!");
					}

				mVPNConnection = new VPNServiceConnection();
				Intent serviceIntent = new Intent(getApplicationContext(), AndroidVpnService.class);
				bindService(serviceIntent, mVPNConnection, Context.BIND_ADJUST_WITH_ACTIVITY);
			}

		private void disconnectVPNService()
			{
				unbindService(mVPNConnection);
				mServiceInterface = null;
				mVPNConnection = null;
			}
	}
