package anonvpn.anon_next.android.service.gui;

import android.content.Intent;
import android.os.Binder;
import android.util.Log;

import java.util.Hashtable;

import anon.crypto.SignatureVerifier;
import anon.infoservice.MixCascade;
import anon.infoservice.MixInfo;
import anon.util.CountryMapper;
import anonvpn.anon_next.android.AndroidVpnService;
import anonvpn.anon_next.android.service.foreground.ForegroundService;
import anonvpn.anon_next.android.ui.main.MainActivity;
import anonvpn.anon_next.core.InfoService;
import anonvpn.anon_next.core.gui.IVPNInterface;
import anonvpn.anon_next.core.gui.VPNState;
import anonvpn.anon_next.core.notification.Event;
import anonvpn.anon_next.core.notification.NotificationDispatcher;
import anonvpn.anon_next.core.notification.Observer;
import anonvpn.anon_next.core.persistence.IConfig;


public class AndroidGUIInterface extends Binder implements IVPNInterface, Observer
	{

		private final AndroidVpnService mVpnService;
		private final NotificationDispatcher mDispatcher;
		private final IConfig mConfig;
		private final InfoService mInfoService;
		private static final String LOGTAG = "AndroidGUIInterface";

		private VPNState mVPNState = VPNState.DISABLED;

		private long m_lastSentBytes = 0;
		private long m_lastReceivedBytes = 0;
		private long m_lastRxTxUpdate = 0;


		public AndroidGUIInterface(AndroidVpnService vpnService, NotificationDispatcher dispatcher, IConfig config, InfoService infoService)
			{
				this.mVpnService = vpnService;
				this.mDispatcher = dispatcher;
				this.mConfig = config;
				this.mInfoService = infoService;

				mDispatcher.subscribe(Event.CASCADE_CONNECTING, this);
				mDispatcher.subscribe(Event.CASCADE_CONNECTED, this);
				mDispatcher.subscribe(Event.CASCADE_DISCONNECTED, this);
				mDispatcher.subscribe(Event.CASCADE_LOST, this);

				// Simplifies notification of foreground service and main activity
				// after the VPN has been disabled
				mDispatcher.subscribe(Event.VPN_DISABLED, this);

				updateFGService();
			}


		@Override
		public RxTx getRxTx()
			{
				RxTx rxtx = new RxTx();
				long currentTime = System.currentTimeMillis();
				rxtx.m_TxTotal=mConfig.getSentBytes();
				rxtx.m_RxTotal=mConfig.getReceivedBytes();
				if (currentTime > m_lastRxTxUpdate + 1000L)
					{
						long dT = (currentTime - m_lastRxTxUpdate);
						rxtx.m_TxBytesPerSecond = (int)(((rxtx.m_TxTotal - m_lastSentBytes)*1000 )/ dT) ;
						rxtx.m_RxBytesPerSecond = (int)(((rxtx.m_RxTotal - m_lastReceivedBytes)*1000) / dT);
						m_lastSentBytes = rxtx.m_TxTotal;
						m_lastReceivedBytes = rxtx.m_RxTotal;
						m_lastRxTxUpdate = currentTime;
					}
				return rxtx;
			}

		public boolean hasChosenCascade()
			{
				return mConfig.getChosenMixCascade() != null;
			}

		@Override
		public void setChosenCascade(MixCascade cascade, String credential)
			{
				// Save entire cascade to mConfig to retrieve it later
				mConfig.setChosenCascadeCredential(credential);
				mConfig.setChosenMixCascade(cascade);

				Log.i("AndroidVPNService.GUI", Thread.currentThread().getId() + " - Set chosen cascade to cascade with id " + cascade.getId());

				mDispatcher.notify(Event.CASCADE_CHOSEN);
			}

		public MixCascade getChosenCascade()
			{
				return mConfig.getChosenMixCascade();
			}

		@Override
		/** Fetches the current cascades from the IS.
		 * @param bWithStatusUpdate if the current  cascades should be fetched as well.
		 * @param bOnlyVPN only returns cascades which support VPN.
		 */ public Hashtable getCascadeList(boolean bWithStatusUpdate, boolean bOnlyVPN)
			{
				return mInfoService.fetchMixCascades(bWithStatusUpdate, bOnlyVPN);
			}

		public void stopService()
			{
				mVpnService.shutdown();
				stopFGService();
				mDispatcher.notify(Event.SERVICE_SHUTDOWN);
			}

		/**
		 * Sends an intent to the Main Activity.
		 *
		 * @param code 0: Cascade connected; 1: Cascade disconnected.
		 */
		private void sendIntentToMainActivity(int code)
			{
//			Intent i = new Intent(getActivity().getApplicationContext(), MainActivity.class);
//			i.setAction(Intent.ACTION_MAIN);
//			i.setAction(Intent.CATEGORY_LAUNCHER);
//			i.putExtra("FragmentID", 0);
//			getActivity().startActivity(i);
			}

		@Override
		public void enableTunneling()
			{
				mVpnService.startTunnelingDevice();
				mDispatcher.notifyAsync(Event.VPN_ENABLED);
			}

		@Override
		public void disableTunneling()
			{
				mVpnService.stopTunnelingDevice();
				mDispatcher.notifyAsync(Event.VPN_DISABLED);
			}

		@Override
		public anonvpn.anon_next.core.gui.VPNState getServiceState() {
			return mVPNState;
		}

		private void notifyMainActivity() {
			Intent notificationIntent = null;
			notificationIntent = new Intent(mVpnService.getApplicationContext(), MainActivity.class);
			notificationIntent.putExtra("type", "notification");
			notificationIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
			mVpnService.startActivity(notificationIntent);
		}

		@Override
		public void notify(Event event)
			{
				switch(event) {

					case VPN_DISABLED:
						mVPNState = VPNState.DISABLED;
						break;

					case CASCADE_CONNECTING:
						mVPNState = VPNState.CONNECTING;
						break;

					case CASCADE_CONNECTED:
						Log.i(LOGTAG, "Event received: Cascade connected.");
						mVPNState = VPNState.CONNECTED;
						break;

					case CASCADE_DISCONNECTED:
						Log.i(LOGTAG, "Event received: Cascade disconnected.");
						mVPNState = VPNState.DISCONNECTED;
						break;

					case CASCADE_LOST:
						mVPNState = VPNState.WAITING;
						break;

					default:

						Log.e(LOGTAG, "Unexpected event!");
						throw new RuntimeException();
				}

				notifyMainActivity();
				updateFGService();
			}

		private void updateFGService()
			{
				Log.e("AndroidVPNMainActivity", "Preparing ForegroundService!");
				Intent startIntent = new Intent(mVpnService.getApplicationContext(), ForegroundService.class);
				try
					{

						MixCascade cascade = getChosenCascade();
						String ip = "";
						String location = "";
						String name = "";
						if (cascade != null)
							{
								try
									{
										int nrOfMixes = cascade.getNumberOfMixes();
										MixInfo cascadeInfo = cascade.getMixInfo(nrOfMixes - 1);

										String countryCode = "DE";
										ip = "Unknown";
										if (cascadeInfo != null) {
											ip = cascadeInfo.getFirstHostName();
											countryCode = cascadeInfo.getServiceLocation().getCountryCode();
										}
										CountryMapper c = new CountryMapper(countryCode);
										location = c.toString();
										name = getChosenCascade().getName();
									}
								catch (Exception e)
									{
										e.printStackTrace();
									}
							}
						startIntent.putExtra("ip", ip);
						startIntent.putExtra("location", location);
						startIntent.putExtra("name", name);

						startIntent.putExtra("conn_state", mVPNState.toString());
				}
			catch (Exception e)
				{
					e.printStackTrace();
				}
				mVpnService.startService(startIntent);
			}

		private void stopFGService()
			{
				Intent stopIntent = new Intent(mVpnService.getApplicationContext(), ForegroundService.class);
				mVpnService.stopService(stopIntent);
			}
	}
