package anonvpn.anon_next.android.service.networking;

import android.net.VpnService;
import android.util.Log;

import java.io.IOException;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.nio.channels.SocketChannel;

import HTTPClient.HTTPClientSocketFactory;

/**
 * Created by martin on 6/6/17.
 */

public class AndroidProtectedSocketFactory implements HTTPClientSocketFactory
	{

		/**
		 * This reference is needed in order to create sockets
		 * that are not routed into the VPN interface.
		 */
		private final VpnService mServiceRef;

		public AndroidProtectedSocketFactory(VpnService service)
			{

				mServiceRef = service;
			}

		@Override
		public Socket connect(InetAddress inetAddress, int port) throws IOException
			{
				//!!IMPORTANT!!
				//We need to create an **unconnected** socket, which we protect before we finnally connect it to the target.
				//Approach 1:
				// - just create the socket with new Socket();
				//--> this will no work, as (I guess) this will only create the Socket object -- but not the actuall socket (in terms of OS ressource)
				//
				//Approach 2:
				// - Socket sock = SocketChannel.open().socket();
				//--> this will protect the socket
				//==> but the SocketChannel socket will cause trouble on Android 7.0+
				//In fact it seems that concurrent read() / writes() to such a socket will block each other
				//
				//Therefore we have Aproach 3 below
				//--> we create the socket object an by calling the bind() function, we force the OS to actually create the socket, which we can protect afterwards
				// Hurray!
				Socket sock = new Socket();
				sock.bind(null); //necessary to create the socket file descriptor, which we want to protect

//				System.out.println("PROTECTEDSOCKFACTORY - SOCKET " + inetAddress.getHostAddress() + " : " + port);

				// Avoid socket packets being put into tunneling interface
				if (mServiceRef.protect(sock) == false)
					{
						Log.i("PROTECTEDSOCKFACTORY", "Failed to protect socket!");
						return null;
					}


				sock.connect(new InetSocketAddress(inetAddress, port));

				return sock;
			}
	}
