package anonvpn.anon_next.android.service.networking;

import android.os.ParcelFileDescriptor;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import anonvpn.anon_next.core.entities.IPv4Packet;
import anonvpn.anon_next.core.networking.AbstractTunnelingDevice;
import anonvpn.anon_next.core.networking.ITunneling;
import anonvpn.anon_next.core.persistence.IConfig;

final public class AndroidTunnelingInterface extends AbstractTunnelingDevice
	{

		private final ParcelFileDescriptor m_TunDescriptor;
		private final int m_iMTU;
		private final int m_iIP;
		private InputStream m_streamTunInput = null;
		private OutputStream m_streamTunOutput = null;
		private IPv4Packet m_IPv4Packet;
		private byte[] m_arPacketContent;


		public AndroidTunnelingInterface(IConfig a_config, ParcelFileDescriptor tunDescriptor, int MTU, int ip)
			{
				super(a_config);
				m_TunDescriptor = tunDescriptor;
				m_iMTU = MTU;
				m_iIP=ip;
				m_streamTunInput = new FileInputStream(m_TunDescriptor.getFileDescriptor());
				m_streamTunOutput = new FileOutputStream(m_TunDescriptor.getFileDescriptor());
				m_arPacketContent = new byte[m_iMTU];
				m_IPv4Packet = new IPv4Packet(m_arPacketContent);
			}

		/**
		 * TODO: Does not support fragmentation yet. This procedure assumes that
		 * the packet fits the block size of an ANON channel.
		 *
		 * @param pkt
		 * @throws IOException
		 */
		@Override
		protected void writePacket_implementation(IPv4Packet pkt) throws IOException
			{
				m_streamTunOutput.write(pkt.getRawPacket(),0,pkt.getLength());
			}

		@Override
		protected void write_implementation(byte[] ar, int off, int len) throws IOException
			{
				m_streamTunOutput.write(ar,off,len);
			}

		@Override
		protected IPv4Packet readPacket_implementation() throws IOException
			{
				m_streamTunInput.read(m_arPacketContent,0,m_iMTU);
				return m_IPv4Packet;
			}



		@Override
		public int getMTU()
			{
				return m_iMTU;
			}

		@Override
		public int getIP()
			{
				return m_iIP;
			}


	}
