package anonvpn.anon_next.android.service.persistence;

import android.content.SharedPreferences;

import org.w3c.dom.Document;

import java.io.StringReader;

import androidx.preference.PreferenceDataStore;
import anon.infoservice.MixCascade;
import anon.util.XMLUtil;
import anonvpn.anon_next.core.persistence.IConfig;


public class AndroidConfig extends PreferenceDataStore implements IConfig
	{

		private SharedPreferences mSettings = null;

		private volatile long mSentBytes = 0;
		private volatile long mReceivedBytes = 0;

		// Internal variable used to avoid redundant parsing of
		// XML in case getChosenCascade gets called multiple times
		private MixCascade mChosenCascade = null;

		public AndroidConfig(SharedPreferences prefs)
			{
				mSettings = prefs;
			}

		@Override
		public void setString(String key, String data)
			{
				mSettings.edit().putString(key, data).commit();
			}

		@Override
		public void putString(String key, String data)
			{
				setString(key, data);
			}


		@Override
		public void setInt(String key, int data)
			{
				mSettings.edit().putInt(key, data).commit();
			}

		@Override
		public String getString(String key, String defaultValue)
			{
				try
					{
						return mSettings.getString(key, defaultValue);
					}
					catch(Exception e)
						{
							return defaultValue;
						}
			}

		@Override
		public int getInt(String key, int defaultValue)
			{
				return mSettings.getInt(key, defaultValue);
			}

		@Override
		public boolean getBoolean(String key, boolean defaultValue)
			{
				try
					{
						return mSettings.getBoolean(key, defaultValue);
					}
				catch(Exception e)
					{
						return defaultValue;
					}
			}

		public void setBoolean(String key, boolean b)
			{
				mSettings.edit().putBoolean(key,b).commit();

			}

		@Override
		public MixCascade getChosenMixCascade()
			{

				MixCascade result = null;

				if (mChosenCascade == null)
					{
						String xml = this.getString("ChosenCascade.Object", "");
						if (!xml.equals(""))
							{ // something was cached in SharedPrefs
								try
									{
										Document doc = anon.util.XMLUtil.readXMLDocument(new StringReader(xml));
										result = new MixCascade(doc.getDocumentElement());
										mChosenCascade = result;
									}
								catch (Exception e)
									{
										e.printStackTrace();
									}
							}
					}
				else
					{
						result = mChosenCascade;
					}

				return result;
			}

		@Override
		public void setChosenMixCascade(MixCascade cascade)
			{

				String xml = XMLUtil.toString(cascade);
				this.setString("ChosenCascade.Object", xml);
				mChosenCascade = cascade;
			}

		@Override
		public String getChosenCascadeCredential()
			{
				return this.getString("ChosenCascade.Credential", "");
			}

		@Override
		public void setChosenCascadeCredential(String credential)
			{
				this.setString("ChosenCascade.Credential", credential);
			}

		@Override
		public synchronized void addSentBytes(int bytes)
			{
				mSentBytes += bytes;
			}

		@Override
		public synchronized void setSentBytes(long bytes)
			{
				mSentBytes = bytes;
			}

		/**
		 *
		 * @return int
		 */
		@Override
		public long getSentBytes()
			{
				return mSentBytes;
			}

		@Override
		public synchronized void addReceivedBytes(int bytes)
			{
				mReceivedBytes += bytes;
			}

		@Override
		public synchronized void setReceivedBytes(long bytes)
			{
				mReceivedBytes = bytes;
			}
		/**
		 *
		 * @return int
		 */
		@Override
		public long getReceivedBytes()
			{
				return mReceivedBytes;
			}

		public boolean getTutorialPassed()
			{
				return getBoolean("TutorialPassed", false);
			}

		public void setVPNstarted(boolean b)
			{
				setBoolean("VPNStarted", b);
			}

		public boolean getVPNstarted()
			{
				return getBoolean("VPNStarted", false);
			}

		public String getLanguage(String defaultLanguage)
			{
				return getString("prefLanguage",defaultLanguage);
			}

	}
