package anonvpn.anon_next.android.ui;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.IBinder;
import android.util.Log;

import androidx.appcompat.app.AppCompatActivity;
import anonvpn.anon_next.android.AndroidVpnService;
import anonvpn.anon_next.android.service.gui.AndroidGUIInterface;
import anonvpn.anon_next.core.gui.VPNState;

/**
 * Created by martin on 8/9/17.
 */

abstract public class VPNConnectedActivity extends AppCompatActivity
	{

		private static boolean mVPNServiceStarted = false;
		private ServiceConnection mVPNConnection = null;
		private AndroidGUIInterface mServiceInterface = null;

		protected class VPNServiceConnection implements ServiceConnection
			{

				@Override
				public void onServiceConnected(ComponentName name, IBinder service)
					{
						Log.e("AndroidVPNMainActivity", "Connected to service!");
						mServiceInterface = (AndroidGUIInterface) service;
						onVPNServiceConnected();

					}

				@Override
				public void onServiceDisconnected(ComponentName name)
					{
						Log.e("AndroidVPNMainActivity", "Disconnected from service!");
						mServiceInterface = null;
						onVPNServiceDisconnected();

					}
			}

		protected void disconnectVPNService()
			{
				//System.out.println("DISCONNECTING VPN! ----------- ");
				unbindService(mVPNConnection);
				mVPNConnection = null;
				mServiceInterface = null;
			}

		protected void stopVPNService()
			{
				if (!mVPNServiceStarted)
					{
						throw new AssertionError("Redundant call to stopVPNService()!");
					}
				Log.e("AndroidVPNMainActivity", "Stopping VPN service!");

				mServiceInterface.stopService();
				disconnectVPNService();
				mVPNServiceStarted = false;
				Log.e("AndroidVPNMainActivity", "VPN service stopped!");
			}

		protected void startVPNService()
			{
				Log.e("AndroidVPNMainActivity", "Starting VPN service!");
				mVPNConnection = new VPNServiceConnection();
				Intent serviceIntent = new Intent(getApplicationContext(), AndroidVpnService.class);

				if (!mVPNServiceStarted)
					{
						startService(serviceIntent);
						mVPNServiceStarted = true;
					}

				if (mServiceInterface != null)
					{
						throw new AssertionError("Redundant call to startVPNService()!");
					}

				bindService(serviceIntent, mVPNConnection, Context.BIND_ADJUST_WITH_ACTIVITY);
			}

		public boolean isVPNConnected()
			{
				return mServiceInterface != null;
			}

		public boolean isTunnelingEnabled()
			{
				return (isVPNConnected() && getServiceInterface().getServiceState() != VPNState.DISABLED);
			}

		public AndroidGUIInterface getServiceInterface()
			{
				return mServiceInterface;
			}

		abstract protected void onVPNServiceConnected();

		abstract protected void onVPNServiceDisconnected();

	}
