package anonvpn.anon_next.android.ui.main;


import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.widget.SwitchCompat;
import androidx.fragment.app.Fragment;
import anon.infoservice.MixCascade;
import anon.infoservice.MixInfo;
import anon.util.CountryMapper;
import anon.util.Util;
import anonvpn.anon_next.android.R;
import anonvpn.anon_next.android.service.gui.AndroidGUIInterface;
import anonvpn.anon_next.android.ui.HelpActivity;
import anonvpn.anon_next.core.gui.IVPNInterface;

/**
 * A simple {@link Fragment} subclass.
 */
final public class AnonFragment extends Fragment
	{

		// TODO: Consider creating a global singleton for access to preferences instead of having a separate
		//       reference per activity/fragment etc.
		private SharedPreferences mPreferences;

		ProgressBar stat;
		private TextView m_tvStatus;
		private TextView m_tvLocation;
		private View m_viewContent;
		SwitchCompat sw;

		private final String LOGTAG = "AnonFragment.java";

		private final Handler m_TrafficStatisticsHandler = new Handler();
		private final GetTrafficStatisticsTask m_TrafficStatisticsTask = new GetTrafficStatisticsTask();
		private final static int TRAFFIC_STATS_DELAY = 3000;

		private class AnonButtonChangeListener implements CompoundButton.OnCheckedChangeListener
			{

				private void showConfirmationDialog()
					{

						final AlertDialog dialog = new AlertDialog.Builder(getContext()).create();
						dialog.setTitle(getString(R.string.dialog_title_deact));
						dialog.setMessage(getString(R.string.dialog_message_deact));

						dialog.setButton(AlertDialog.BUTTON_POSITIVE, getString(R.string.dialog_b_yes), new DialogInterface.OnClickListener()
							{
								@Override
								public void onClick(DialogInterface dialogInterface, int i)
									{
										stopVPNTunneling();
										dialog.dismiss();
									}
							});

						dialog.setButton(AlertDialog.BUTTON_NEGATIVE, getString(R.string.dialog_b_no), new DialogInterface.OnClickListener()
							{
								@Override
								public void onClick(DialogInterface dialogInterface, int i)
									{
										sw.setChecked(true);
										dialog.dismiss();
									}
							});

						dialog.setCanceledOnTouchOutside(false);
						dialog.show();
					}

				@Override
				public void onCheckedChanged(CompoundButton buttonView, boolean isChecked)
					{
						if (isChecked)
							{
								// If the user canceled the confirmation dialog, the slide button changes
								// to checked although the VPN connection is still active
								if (((MainActivity) getActivity()).isTunnelingEnabled() == false)
									{

										if (!((MainActivity) getActivity()).getServiceInterface().hasChosenCascade())
											{
												System.out.println("NO CASCADE HAS BEEN CHOSEN YET");
												Intent i = new Intent(getActivity().getApplicationContext(), MainActivity.class);
												i.setAction(Intent.ACTION_MAIN);
												i.setAction(Intent.CATEGORY_LAUNCHER);
												i.putExtra(MainActivity.STR_INTENT_EXTRA_FRAGMENTID, MainActivity.FRAGMENTID_CONNECTION);
												getActivity().startActivity(i);
												buttonView.setChecked(false);

											}
										else
											{
												startVPNTunneling();
											}

									}
							}
						else
							{
								// Check if the user canceled the passphrase form
								if (((MainActivity) getActivity()).isTunnelingEnabled())
									{
										showConfirmationDialog();
									}
							}
					}
			}

		private final class GetTrafficStatisticsTask implements Runnable
			{
				private AndroidGUIInterface guiInterface = null;
				public boolean isScheduled = false;

				public void run()
					{
						MainActivity a=(MainActivity) getActivity();
						guiInterface = null;
						if(a!=null)
							guiInterface=a.getServiceInterface();

						// This tasks starts right after the VPN service gets started.
						// Therefore, the service might still be starting (hence, the interface is not ready)
						// when this run-procedure gets called.
						if (guiInterface != null)
							{
								IVPNInterface.RxTx rxtx = guiInterface.getRxTx();
								updateTrafficStatistics(rxtx);
							}
						m_TrafficStatisticsHandler.postDelayed(this, TRAFFIC_STATS_DELAY);
					}

			}

		private void startVPNTunneling()
			{

				((MainActivity) getActivity()).enableVPNTunneling();
			}

		private void stopVPNTunneling()
			{
				final ProgressDialog dialog = new ProgressDialog(getContext());
				dialog.setMessage(getString(R.string.vpn_tunneling_shutdown_progress_notification));
				dialog.setCancelable(false);
				dialog.setIndeterminate(true);
				dialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
				class StopVPNTunnelingAsyncTask extends AsyncTask<Object,Object,Object>
					{

						@Override
						protected void onPreExecute()
							{
								dialog.show();
							}
						@Override
						protected Object doInBackground(Object... objects)
							{
								MainActivity a=(MainActivity) getActivity();
								if(a!=null)
									a.disableVPNTunneling();
								return null;
							}

						@Override
						protected void onPostExecute(Object o)
							{
								dialog.dismiss();
								updateStatus();
						}
					}
				new StopVPNTunnelingAsyncTask().execute(null,null,null);
				dialog.show();
			}

		private void updateTrafficStatistics(IVPNInterface.RxTx rxtx)
			{
				final TextView up = m_viewContent.findViewById(R.id.anon_tv_upload);
				final TextView down = m_viewContent.findViewById(R.id.anon_tv_download);
				final TextView upBytes = m_viewContent.findViewById(R.id.tvUploadTraffic);
				final TextView downBytes = m_viewContent.findViewById(R.id.tvDownloadTraffic);

				up.setText(Util.formatKbitPerSecValueWithUnit(rxtx.m_TxBytesPerSecond/125)); // Byte/s --> kbit/s --> x*8/1000 =x/125
				down.setText(Util.formatKbitPerSecValueWithUnit(rxtx.m_RxBytesPerSecond/125));
				upBytes.setText(Util.formatBytesValueWithUnit(rxtx.m_TxTotal));
				downBytes.setText(Util.formatBytesValueWithUnit(rxtx.m_RxTotal));
			}


		@Override
		public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
			{

				mPreferences = getContext().getSharedPreferences(getContext().getString(R.string.sp_global), Context.MODE_PRIVATE);

				// Inflate the layout for this fragment
				m_viewContent = inflater.inflate(R.layout.fragment_anon, container, false);

				m_tvStatus = m_viewContent.findViewById(R.id.anon_tv_status);
				ImageView anonHelp = m_viewContent.findViewById(R.id.anon_iv_anonymometer_help);

				//assign Location
				m_tvLocation = m_viewContent.findViewById(R.id.anon_tv_location);

				anonHelp.setOnClickListener(new View.OnClickListener()
					{
						@Override
						public void onClick(View view)
							{
								Intent helpIntent = new Intent(getContext(), HelpActivity.class);
								getContext().setTheme(R.style.AppTheme);
								getContext().startActivity(helpIntent);
							}
					});

				//Anon-Button
				sw = m_viewContent.findViewById(R.id.anon_switch);
				//Do something on checked changed of ANON Switch
				sw.setOnCheckedChangeListener(new AnonButtonChangeListener());

				Log.e("ANONFragment", "ONCREATEVIEW");

				updateStatus();

				return m_viewContent;
			}

		@Override
		public void onResume()
			{
				super.onResume();
				updateStatus();
			}

		@Override
		public void onPause()
			{
				super.onPause();
				m_TrafficStatisticsHandler.removeCallbacks(m_TrafficStatisticsTask);
				m_TrafficStatisticsTask.isScheduled = false;
			}

		public void updateStatus()
			{

				MainActivity main = (MainActivity) getActivity();

				if (main != null) {
					AndroidGUIInterface service = main.getServiceInterface();

					if (service != null) {

						switch (service.getServiceState()) {

							case CONNECTING:
								if (!sw.isChecked()) {
									sw.setChecked(true);
								}
								m_tvStatus.setText(getContext().getString(R.string.start_state_establish));
								m_tvStatus.setTextColor(getResources().getColor(R.color.connectivity_problem));
								break;

							case CONNECTED:
								IVPNInterface.RxTx rxtx = service.getRxTx();
								updateTrafficStatistics(rxtx);
								if (!m_TrafficStatisticsTask.isScheduled) {
									m_TrafficStatisticsHandler.postDelayed(m_TrafficStatisticsTask, TRAFFIC_STATS_DELAY);
									m_TrafficStatisticsTask.isScheduled = true;
								}
								m_tvStatus.setText(getContext().getString(R.string.start_state_active));
								m_tvStatus.setTextColor(getResources().getColor(R.color.connectivity_ok));
								((TextView) m_viewContent.findViewById(R.id.anon_tv_tip)).setText("");
								MixCascade cascade = service.getChosenCascade();

								MixInfo cascadeInfo = cascade.getMixInfo(cascade.getNumberOfMixes() - 1);
								String countryCode = "DE";
								if (cascadeInfo != null) {
									countryCode = cascadeInfo.getServiceLocation().getCountryCode();
								}
								CountryMapper c = new CountryMapper(countryCode);
								String location = c.toString();
								m_tvLocation.setText(location);
								break;

							case WAITING:
								m_tvStatus.setText(getContext().getString(R.string.start_state_noservice));
								m_tvStatus.setTextColor(getResources().getColor(R.color.connectivity_fatal));
								break;

							case DISCONNECTED:
								m_tvStatus.setText(getContext().getString(R.string.start_state_nocon));
								m_tvStatus.setTextColor(getResources().getColor(R.color.connectivity_fatal));
								break;

							case DISABLED:
								Log.e("ANONFragment", "VPN is not running!");
								sw.setChecked(false);

								m_TrafficStatisticsHandler.removeCallbacks(m_TrafficStatisticsTask);
								m_TrafficStatisticsTask.isScheduled = false;
								updateTrafficStatistics(IVPNInterface.RxTx.ALL_ZERO);

								m_tvStatus.setText(getContext().getString(R.string.start_state_deactive));
								m_tvStatus.setTextColor(getResources().getColor(R.color.connectivity_off));
								((TextView) m_viewContent.findViewById(R.id.anon_tv_tip)).setText("");
								((ProgressBar) m_viewContent.findViewById(R.id.anon_iv_statusbar)).setProgress(0);
								m_tvLocation.setText(getContext().getString(R.string.start_loc_unknown));
								break;

						}
					}
				}
			}
	}
