package anonvpn.anon_next.android.ui.main;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;

import org.osmdroid.api.IMapController;
import org.osmdroid.tileprovider.tilesource.XYTileSource;
import org.osmdroid.util.BoundingBox;
import org.osmdroid.util.GeoPoint;
import org.osmdroid.views.MapView;
import org.osmdroid.views.overlay.ItemizedIconOverlay;
import org.osmdroid.views.overlay.ItemizedOverlay;
import org.osmdroid.views.overlay.OverlayItem;
import org.osmdroid.views.overlay.Polyline;
import org.w3c.dom.Document;

import java.io.StringReader;
import java.util.ArrayList;

import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import anon.infoservice.MixCascade;
import anon.infoservice.MixInfo;
import anonvpn.anon_next.android.R;

/**
 * A simple {@link Fragment} subclass.
 * Activities that contain this fragment must implement the
 * ConnectDetailFragment.OnFragmentInteractionListener interface
 * to handle interaction events.
 * Use the {@link ConnectDetailFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class ConnectDetailFragment extends Fragment
	{

		private static final String KEY_CASCADEXML = "cascadeXML";
		MixCascade mCascade;
		View view;
		MapView mMapView;

		public ConnectDetailFragment()
			{
				// Required empty public constructor
			}

		public static ConnectDetailFragment newInstance(String cascadeXML)
			{
				ConnectDetailFragment fragment = new ConnectDetailFragment();
				Bundle args = new Bundle();
				args.putString(KEY_CASCADEXML, cascadeXML);
				fragment.setArguments(args);
				return fragment;
			}

		@Override
		public void onCreate(Bundle savedInstanceState)
			{
				super.onCreate(savedInstanceState);
			}

		@Override
		public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
			{
				view = inflater.inflate(R.layout.fragment_connect_detail, container, false);
				return view;
			}

		@Override
		public void onViewCreated(View view, @Nullable Bundle savedInstanceState)
			{
				super.onViewCreated(view, savedInstanceState);
				String cascadeXML = getArguments().getString(KEY_CASCADEXML);
				try
					{
						Document doc = anon.util.XMLUtil.readXMLDocument(new StringReader(cascadeXML));
						mCascade = new MixCascade(doc.getDocumentElement());
					}
				catch (Exception e)
					{
						e.printStackTrace();
					}
				TextView tv_host = view.findViewById(R.id.connectd_tv_host);
				TextView tv_location = view.findViewById(R.id.connectd_tv_location);
				TextView tv_anonlevel = view.findViewById(R.id.connectd_tv_anonlevel);
				TextView tv_users = view.findViewById(R.id.connectd_tv_user);
				TextView tv_mix = view.findViewById(R.id.connectd_tv_mix);
				TextView tv_endip = view.findViewById(R.id.connectd_tv_endip);
				tv_host.setText(mCascade.getName());
				tv_location.setText(mCascade.getMixInfo(0).getServiceLocation().getCity());
				tv_anonlevel.setText("AnonLevel"); // TODO: 20.09.17 Add Anonlevel calculation?
				tv_users.setText(String.valueOf(mCascade.getCurrentStatus().getNrOfActiveUsers()));
				tv_mix.setText(String.valueOf(mCascade.getNumberOfMixes()));
				try
					{
						tv_endip.setText(mCascade.getMixInfo(mCascade.getNumberOfMixes() - 1).getFirstHostName());
					}
				catch (Exception e)
					{
						e.printStackTrace();
					}
				setMapView();
			}

		@Override
		public void onAttach(Context context)
			{
				super.onAttach(context);

			}

		@Override
		public void onDetach()
			{
				super.onDetach();
			}

		private void setContent()
			{
//        try {
//            host.setText(this_cascade.getName());
//            end_ip.setText(this_cascade.getMixInfo(this_cascade.getNumberOfMixes() - 1).getFirstHostName());
//            setMapView();
//        } catch (Exception e) {
//            e.printStackTrace();
//        }

//        Button connect_button = findViewById(R.id.connect_b_select);
//        connect_button.setOnClickListener(new View.OnClickListener() {
//            @Override
//            public void onClick(View view) {
//                if (intent_source.equals("MainActivity")) {
//                    Intent intent = new Intent(getApplicationContext(), MainActivity.class);
//                    try {
//                        intent.putExtra("ClickedCascade.Object", XMLUtil.toString(this_cascade));
//                    } catch (NullPointerException e) {
//                        e.printStackTrace();
//                    }
//                    startActivity(intent);
//                } else if (intent_source.equals("TutorialActivity")) {
//                    Intent intent = new Intent(getApplicationContext(), TutorialActivity.class);
//                    try {
//                        intent.putExtra("ClickedCascade.Object", XMLUtil.toString(this_cascade));
//                    } catch (NullPointerException e) {
//                        e.printStackTrace();
//                    }
//                    startActivity(intent);
//                } else {
//                    throw new RuntimeException("Unexpected intent_source: " + intent_source);
//                }
//            }
//        });
			}

		private void setMapView()
			{
				//
				// Prepare map view
				//

				mMapView = (MapView) getActivity().findViewById(R.id.connectd_map);
				mMapView.setLayerType(View.LAYER_TYPE_SOFTWARE, null);

				mMapView.setTileSource(new XYTileSource("MyOSM", 0, 19, 256, ".png", new String[]{"https://sk13.xyz:45443/osm_tiles/"}));
				mMapView.setBuiltInZoomControls(true);
				mMapView.setMultiTouchControls(true);
				IMapController mapController = mMapView.getController();
				mapController.setZoom(7);
				GeoPoint startPoint = new GeoPoint(53.0383, 5.7944);
				mapController.setCenter(startPoint);
				drawChosenConnection(mCascade);
			}

		public void drawChosenConnection(MixCascade connection)
			{

				this.mMapView.getOverlays().clear();

//				MixCascade.MixDescriptor[] mixes = connection.get
				ArrayList<OverlayItem> markers = new ArrayList<>();

				//Draw new Connection
				ArrayList<GeoPoint> geoPoints = new ArrayList<>();

				// User location, TODO: Change to GPS location?
				geoPoints.add(new GeoPoint(51.0767496, 13.6321589));
				OverlayItem olitem = new OverlayItem("Your location", // Name
								"", // Description
								geoPoints.get(0));
				Drawable drawable = new BitmapDrawable(getResources(), resizeMapIcons("anon_jap", 50, 50));
				olitem.setMarker(drawable);
				markers.add(olitem);

				for (int i = 0; i < connection.getNumberOfMixes(); i++)
					{
						MixInfo mix = connection.getMixInfo(i);
						if (mix.getServiceLocation().getLatitude().equals("") || mix.getServiceLocation().getLatitude().equals(""))
							{
								Toast.makeText(getContext(), getString(R.string.connectd_label_no_coordinates), Toast.LENGTH_SHORT).show();
								break;
							}
						GeoPoint point = GeoPoint.fromDoubleString(mix.getServiceLocation().getLatitude() + ',' + mix.getServiceLocation().getLongitude(), ',');
						geoPoints.add(point);
						OverlayItem _olitem = null;
						try
							{
								_olitem = new OverlayItem(mix.getServiceLocation().getCity(), // Name
												mix.getFirstHostName(), // Description
												point);
							}
						catch (Exception e)
							{
								e.printStackTrace();
							}
						Drawable _drawable = new BitmapDrawable(getResources(), resizeMapIcons("mix", 50, 50));
						_olitem.setMarker(_drawable);
						markers.add(_olitem);
					}

				//draw Connection-Lines
				Polyline polyline;
				polyline = new Polyline();
				polyline.setPoints(geoPoints);
				polyline.setColor(Color.RED);
				polyline.setWidth(4);
//				polyline.setInfoWindow(new BasicInfoWindow(R.layout.bonuspack_bubble, mMapView));
//				polyline.setTitle("Polyline tapped!");
				mMapView.getOverlays().add(polyline);

				        /* OnTapListener for the Markers, shows a simple Toast. */
				ItemizedOverlay<OverlayItem> _locationOverlay = new ItemizedIconOverlay<OverlayItem>(markers, new ItemizedIconOverlay.OnItemGestureListener<OverlayItem>()
					{
						@Override
						public boolean onItemSingleTapUp(final int index, final OverlayItem item)
							{
								Toast.makeText(getContext(), "Location: " + item.getTitle() + "\nIP: " + item.getSnippet(), Toast.LENGTH_LONG).show();
								return true; // We 'handled' this event.
							}

						@Override
						public boolean onItemLongPress(final int index, final OverlayItem item)
							{
								Toast.makeText(getContext(), "Location: " + item.getTitle() + "\nIP: " + item.getSnippet(), Toast.LENGTH_LONG).show();
								return false;
							}
					}, getContext());

				mMapView.getOverlays().add(_locationOverlay);

				// TODO: Zoom camera to correct bounds
				//zoom MapCamera
				final BoundingBox boundingBox = BoundingBox.fromGeoPoints(geoPoints);
				mMapView.addOnFirstLayoutListener(new MapView.OnFirstLayoutListener()
					{
						public void onFirstLayout(View v, int left, int top, int right, int bottom)
							{
								mMapView.zoomToBoundingBox(boundingBox, true);
							}
					});
//        mMapView.getController().setZoom(7);
				mMapView.invalidate();
			}

		public Bitmap resizeMapIcons(String iconName, int width, int height)
			{
				Bitmap imageBitmap = BitmapFactory.decodeResource(getResources(), getResources().getIdentifier(iconName, "drawable", getContext().getPackageName()));
				Bitmap resizedBitmap = Bitmap.createScaledBitmap(imageBitmap, width, height, false);
				return resizedBitmap;
			}


		/**
		 * This interface must be implemented by activities that contain this
		 * fragment to allow an interaction in this fragment to be communicated
		 * to the activity and potentially other fragments contained in that
		 * activity.
		 * <p>
		 * See the Android Training lesson <a href=
		 * "http://developer.android.com/training/basics/fragments/communicating.html"
		 * >Communicating with Other Fragments</a> for more information.
		 */
//    public interface OnFragmentInteractionListener {
//        // Update argument type and name
//        void onFragmentInteraction(Uri uri);
//    }
	}
