package anonvpn.anon_next.android.ui.main;

import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.widget.SwitchCompat;
import androidx.recyclerview.widget.RecyclerView;
import anonvpn.anon_next.android.R;
import anonvpn.anon_next.android.ui.main.NetDataHolder;
import anonvpn.anon_next.android.ui.NetDetailActivity;

import java.util.ArrayList;

/**
 * Created by Phil on 17.11.16.
 */

public class NetRecyclerViewAdapter extends RecyclerView.Adapter<NetRecyclerViewAdapter.ViewHolder> {
    private final ArrayList<NetDataHolder> _dataList;
    private Context _context;

    public NetRecyclerViewAdapter(ArrayList<NetDataHolder> items, Context context) {
        _dataList = items;
        _context = context;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.item_net, parent, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(final ViewHolder holder, final int position) {
        final SharedPreferences preferences = _context.getSharedPreferences(_context.getString(R.string.sp_global), Context.MODE_PRIVATE);
        holder._item = _dataList.get(position);
        holder._tv_netname.setText(_dataList.get(position).getName());
        holder._iv_neticon.setImageDrawable(holder._item.getIcon());
        String currWifi = getCurrentSsid(_context);
        if(currWifi != null) {
            if (holder._item.getName().equals(currWifi)) {
                holder._view.setBackgroundColor(_context.getResources().getColor(R.color.colorAccent));
                holder._tv_netconnected.setVisibility(View.VISIBLE);
            }
            else {
                holder._view.setBackgroundColor(_context.getResources().getColor(R.color.blank));
                holder._tv_netconnected.setVisibility(View.GONE);
            }
        }
        else {
            holder._view.setBackgroundColor(_context.getResources().getColor(R.color.blank));
        }
        holder._itemswitch.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                preferences.edit().putBoolean(_context.getString(R.string.sp_netchecked) + holder._item.getName(), isChecked).apply();
                holder._item.setChecked(isChecked);
            }
        });
        holder._itemswitch.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                boolean check = preferences.getBoolean(_context.getString(R.string.sp_netchecked) + holder._item.getName(), true);
                if(check) {
                    Toast.makeText(_context, holder._item.getName() + " " + _context.getString(R.string.net_toast_notanon), Toast.LENGTH_SHORT).show();
                }
                else{
                    if(holder._item.getSecurity().equals("Nicht sicher!")){
                        AlertDialog dialog = new AlertDialog.Builder(_context).create();
                        dialog.setTitle(_context.getString(R.string.dialog_title_unsecurenet));
                        dialog.setMessage(_context.getString(R.string.dialog_message_unsecurenet));
                        dialog.setButton(AlertDialog.BUTTON_POSITIVE, _context.getString(R.string.dialog_b_yes), new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialogInterface, int i) {
                                Toast.makeText(_context, holder._item.getName() + " " + _context.getString(R.string.net_toast_notanon), Toast.LENGTH_SHORT).show();
                            }
                        });
                        dialog.setButton(AlertDialog.BUTTON_NEGATIVE, _context.getString(R.string.dialog_b_no), new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialogInterface, int i) {
                                holder._itemswitch.setChecked(true);
                            }
                        });
                        dialog.setCanceledOnTouchOutside(false);
                        dialog.show();
                    }
                    else {
                        Toast.makeText(_context, holder._item.getName() + " " + _context.getString(R.string.net_toast_notanon), Toast.LENGTH_SHORT).show();
                    }
                }
            }
        });
        holder._view.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                //_context ist Activity main und es soll von dort aus AppDetail geöffnet werden
                Intent detailIntent = new Intent(_context, NetDetailActivity.class);
                Bundle arguments = new Bundle();
                arguments.putString("title", holder._item.getName());
                arguments.putString("security", holder._item.getSecurity());
                //putDrawable
                Drawable icon = holder._item.getIcon();
                Bitmap bitmap = ((BitmapDrawable)icon).getBitmap();
                detailIntent.putExtra("Bitmap", bitmap);
                detailIntent.putExtras(arguments);
                _context.startActivity(detailIntent);
            }
        });
        boolean isChecked = preferences.getBoolean(_context.getString(R.string.sp_netchecked) + holder._item.getName(), true);
        holder._itemswitch.setChecked(isChecked);
    }

    @Override
    public int getItemCount() {
        return _dataList.size();
    }

    class ViewHolder extends RecyclerView.ViewHolder {
        final View _view;
        final TextView _tv_netname;
        final ImageView _iv_neticon;
        final SwitchCompat _itemswitch;
        final TextView _tv_netconnected;
        NetDataHolder _item;

        ViewHolder(View view) {
            super(view);
            _view = view;
            _tv_netname = (TextView) view.findViewById(R.id.net_tv_name);
            _iv_neticon = (ImageView) view.findViewById(R.id.net_iv_icon);
            _itemswitch = (SwitchCompat) view.findViewById(R.id.net_switch);
            _tv_netconnected = (TextView) view.findViewById(R.id.net_tv_home);
        }
    }

    public static String getCurrentSsid(Context context) {
        String ssid = null;
        ConnectivityManager connManager = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo networkInfo = connManager.getNetworkInfo(ConnectivityManager.TYPE_WIFI);
        if (networkInfo.isConnected()) {
            final WifiManager wifiManager = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
            final WifiInfo connectionInfo = wifiManager.getConnectionInfo();
            if (connectionInfo != null && !TextUtils.isEmpty(connectionInfo.getSSID())) {
                ssid = connectionInfo.getSSID().replace("\"", "");
            }
        }
        return ssid;
    }
}
