package anonvpn.anon_next.android.ui.settings;


import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.view.MenuItem;

import java.util.Locale;

import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import androidx.preference.Preference;
import anonvpn.anon_next.android.R;
import anonvpn.anon_next.core.CascadeConnectionManager;


public class SettingsActivity extends AppCompatActivity
	{
		@Override
		protected void onCreate(Bundle savedInstanceState)
			{
				super.onCreate(savedInstanceState);
				setContentView(R.layout.activity_settings);
				SettingsFragment fragmentSettings = new SettingsFragment(this);
				getSupportFragmentManager().beginTransaction().replace(R.id.settings_fragment, fragmentSettings, "settings").commit();

				Intent intent = getIntent();
				String action = intent.getAction();
				if (action != null && action.equals("anonguard.settings.SET_FAKE_DNS"))
					{
						CascadeConnectionManager.USE_FAKE_DNS = intent.getBooleanExtra("USE_FAKE_DNS", CascadeConnectionManager.USE_FAKE_DNS);
						setResult(RESULT_OK);
						finish();
						return;
					}

				setupActionBar();
				setTitle(getString(R.string.settings_label_title));
			}

		public static void setLocale(Activity activity, String lang, boolean bRefresh)
			{
				if(lang==null)
					return;
				Locale myLocale = new Locale(lang);
				Resources res = activity.getResources();
				DisplayMetrics dm = res.getDisplayMetrics();
				Configuration conf = res.getConfiguration();
				conf.locale = myLocale;
				res.updateConfiguration(conf, dm);
				if (bRefresh)
					{
						Intent refresh = new Intent(activity, activity.getClass());
						refresh.setFlags(refresh.getFlags() | Intent.FLAG_ACTIVITY_NO_HISTORY);
						activity.startActivity(refresh);
					}
			}

		@Override
		public boolean onOptionsItemSelected(MenuItem item)
			{
				switch (item.getItemId())
					{
						case android.R.id.home:
							onBackPressed();
							return true;
					}

				return super.onOptionsItemSelected(item);
			}

		/**
		 * Set up the {@link android.app.ActionBar}, if the API is available.
		 */
		private void setupActionBar()
			{
				ActionBar actionBar = getSupportActionBar();
				if (actionBar != null)
					{
						// Show the Up button in the action bar.
						actionBar.setDisplayHomeAsUpEnabled(true);
					}
			}


		public void initPreferenceChangeListeners(SettingsFragment fragmentSettings)
			{
				Preference pref = fragmentSettings.findPreference("prefLanguage");
				final Activity a = this;
				pref.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener()
					{
						@Override
						public boolean onPreferenceChange(Preference preference, Object newValue)
							{
								setLocale(a, newValue.toString(), true);
								return true;
							}
					});
			}
	}
