package anonvpn.anon_next.android.service.networking;

import android.net.VpnService;
import android.os.ParcelFileDescriptor;
import android.util.Log;

import java.io.OutputStream;
import java.net.Socket;

import anonvpn.anon_next.core.networking.IInternetSocket;
import anonvpn.anon_next.core.networking.INetworkFactory;
import anonvpn.anon_next.core.networking.ITunneling;
import anonvpn.anon_next.core.persistence.IConfig;
import pcap.PCAPTUNDevice;
import pcap.PCAPWriter;

/**
 * Created by martin on 5/22/17.
 */

public class AndroidNetworkFactory implements INetworkFactory
	{

		/**
		 * This reference is needed in order to create sockets
		 * that are not routed into the VPN interface.
		 */
		private final VpnService mServiceRef;
		private ParcelFileDescriptor mTunnelingDescriptor = null;
		private final int mPacketMTU;
		private int m_iIP;
		private IConfig m_Config;

		public AndroidNetworkFactory(VpnService service, int MTU, IConfig a_config)
			{

				mServiceRef = service;
				mPacketMTU = MTU;
				m_Config = a_config;
			}

		public void setTunnelingDescriptor(ParcelFileDescriptor tunnelingDescriptor, int ip)
			{
				mTunnelingDescriptor = tunnelingDescriptor;
				m_iIP = ip;
			}

		/**
		 * Assumes that prepare has been called in advance so that the VPN device can
		 * be created.
		 *
		 *
		 */
		public ITunneling getTunnelingInterface()
			{

				ITunneling result = null;

				if (mTunnelingDescriptor != null)
					{
						result = new AndroidTunnelingInterface(m_Config, mTunnelingDescriptor, mPacketMTU, m_iIP);
					}
				return result;
			}

		public IInternetSocket getInternetSocket()
			{

				AndroidInternetSocket result = null;
				Socket sock = new Socket();

				// Avoid socket packets being put into tunneling interface
				if (mServiceRef.protect(sock) == false)
					{
						Log.i("ANDROIDTUN", "Failed to protect socket!");
					}

				result = new AndroidInternetSocket(sock);
				return result;
			}
	}
