package anonvpn.anon_next.core.networking;

import org.junit.Test;

import java.util.Hashtable;
import java.util.List;

import static org.junit.Assert.*;

import anon.infoservice.MixCascade;
import anonvpn.anon_next.core.InfoService;
import anonvpn.anon_next.core.networking.mocks.TestConfig;
import anonvpn.anon_next.core.networking.mocks.TestHTTPClientSocketFactory;
import anonvpn.anon_next.core.notification.Event;
import anonvpn.anon_next.core.notification.NotificationDispatcher;

/**
 * Created by martin on 8/3/17.
 */

public class InfoServiceTest {

    @Test
    public void fetchingWithoutVPN() {

        NotificationDispatcher dispatcher = new NotificationDispatcher();
        TestHTTPClientSocketFactory testRegularFactory = new TestHTTPClientSocketFactory();
        TestHTTPClientSocketFactory testProtectedFactory = new TestHTTPClientSocketFactory();
        TestConfig testConfig = new TestConfig();
        testConfig.setString("PrimaryInfoService.Host", "127.0.0.1");

        InfoService infoService = new InfoService(dispatcher, testConfig, testRegularFactory, testProtectedFactory);

        assertEquals(0, testRegularFactory.getCreatedSockets().size());
        assertEquals(0, testProtectedFactory.getCreatedSockets().size());

        Hashtable cascades = infoService.fetchMixCascades(true,false);

        assertTrue(testRegularFactory.getCreatedSockets().size() > 0);
        assertEquals(0, testProtectedFactory.getCreatedSockets().size());

        testRegularFactory.clearCreatedSockets();
        dispatcher.notify(Event.VPN_ENABLED);
        dispatcher.notify(Event.VPN_DISABLED);
        cascades = infoService.fetchMixCascades(true,false);

        assertTrue(testRegularFactory.getCreatedSockets().size() > 0);
        assertEquals(0, testProtectedFactory.getCreatedSockets().size());
    }

    @Test
    public void fetchingWithVPN() {

        NotificationDispatcher dispatcher = new NotificationDispatcher();
        TestHTTPClientSocketFactory testRegularFactory = new TestHTTPClientSocketFactory();
        TestHTTPClientSocketFactory testProtectedFactory = new TestHTTPClientSocketFactory();
        TestConfig testConfig = new TestConfig();
        testConfig.setString("PrimaryInfoService.Host", "127.0.0.1");

        InfoService infoService = new InfoService(dispatcher, testConfig, testRegularFactory, testProtectedFactory);

        assertEquals(0, testRegularFactory.getCreatedSockets().size());
        assertEquals(0, testProtectedFactory.getCreatedSockets().size());

        dispatcher.notify(Event.VPN_ENABLED);
        Hashtable cascades = infoService.fetchMixCascades(true,false);
        assertEquals(0, testRegularFactory.getCreatedSockets().size());
        assertTrue(testProtectedFactory.getCreatedSockets().size() > 0);
        testProtectedFactory.clearCreatedSockets();

        dispatcher.notify(Event.CASCADE_CONNECTED);
        cascades = infoService.fetchMixCascades(true,false);
        assertTrue(testRegularFactory.getCreatedSockets().size() > 0);
        assertEquals(0, testProtectedFactory.getCreatedSockets().size());
        testRegularFactory.clearCreatedSockets();

        dispatcher.notify(Event.CASCADE_DISCONNECTED);
        cascades = infoService.fetchMixCascades(true,false);
        assertEquals(0, testRegularFactory.getCreatedSockets().size());
        assertTrue(testProtectedFactory.getCreatedSockets().size() > 0);
        testProtectedFactory.clearCreatedSockets();

        dispatcher.notify(Event.VPN_DISABLED);
        cascades = infoService.fetchMixCascades(true,false);
        assertTrue(testRegularFactory.getCreatedSockets().size() > 0);
        assertEquals(0, testProtectedFactory.getCreatedSockets().size());
    }
}
