package anonvpn.anon_next.android;

/**
 * Created by migue on 16.06.15.
 * handles a vpn connection
 */

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.VpnService;
import android.os.IBinder;
import android.os.ParcelFileDescriptor;
import android.util.Log;

import java.io.IOException;

import anonvpn.anon_next.android.service.gui.AndroidGUIFactory;
import anonvpn.anon_next.android.service.gui.AndroidGUIInterface;
import anonvpn.anon_next.android.service.networking.AndroidConnectivityMonitorFactory;
import anonvpn.anon_next.android.service.networking.AndroidNetworkFactory;
import anonvpn.anon_next.android.service.networking.AndroidProtectedSocketFactory;
import anonvpn.anon_next.android.service.persistence.AndroidPersistenceFactory;
import anonvpn.anon_next.android.util.AndroidResourceLoaderHelper;
import anonvpn.anon_next.core.ANONVPNService;
import anonvpn.anon_next.core.notification.Event;
import anonvpn.anon_next.core.persistence.IConfig;
import wiresharksshdumpserver.WiresharkSSHdumpServer;

public class AndroidVpnService extends VpnService
	{



		ANONVPNService anonvpnService = null;

		// This reference is needed to properly shutdown the service later on
		ParcelFileDescriptor mTunnelingDevice = null;

		// TODO: Find a way to adapt the MTU to the block size of the ANON protocol
		final private static int TUNNELING_MTU = 9000;

		AndroidNetworkFactory mNetworkFactory = null;

		private native int setBlocking(int fd_id);

		/**
		 * We expect this method to be called after VPNService.prepare has been called.
		 *
		 * @param intent
		 * @param flags
		 * @param startId
		 * @return
		 */
		@Override
		public int onStartCommand(Intent intent, int flags, int startId)
			{

				// onStartCommand may be run multiple times (e.g. if the user turns the smartphone)
				if (anonvpnService == null)
					{
						Log.e("AndroidVPNService", Thread.currentThread().getId() + "Setting up tunneling device!");
						AndroidPersistenceFactory persistenceFactory = null;
						try
							{
								persistenceFactory = new AndroidPersistenceFactory(getSharedPreferences(this.getString(R.string.sp_global), Context.MODE_PRIVATE));
							}
						catch (Exception e)
							{
								e.printStackTrace();
							}

						IConfig a_config = null;
						if (persistenceFactory != null)
							{
								a_config = persistenceFactory.getConfiguration();
							}
						mNetworkFactory = new AndroidNetworkFactory(this, TUNNELING_MTU, a_config);
						AndroidProtectedSocketFactory mixSocketFactory = new AndroidProtectedSocketFactory(this);
						AndroidGUIFactory guiFactory = new AndroidGUIFactory(this);

						AndroidConnectivityMonitorFactory connMonFactory = new AndroidConnectivityMonitorFactory(getApplicationContext());
						Log.e("AndroidVPNService", "Starting core service!");
						AndroidResourceLoaderHelper resourecLoaderHelper = new AndroidResourceLoaderHelper(getAssets());
						AndroidLog logger = new AndroidLog();
						anonvpnService = new ANONVPNService(mNetworkFactory, persistenceFactory, guiFactory, connMonFactory, mixSocketFactory, resourecLoaderHelper, logger);
					}

				// service running until it is explicitly stopped,return sticky.
				return START_STICKY;
			}

		public void startTunnelingDevice()
			{

				byte[] ip = new byte[4];
				ip[0] = 10;
				int iIP = 10;
				for (int i = 1; i < 4; i++)
					{
						ip[i] = (byte) (2 + (int) (Math.random() * 252));
						iIP <<= 8;
						iIP |= (ip[i] & 0x00FF);
					}
				mTunnelingDevice = setupTunnelingDevice(TUNNELING_MTU, ip);
				mNetworkFactory.setTunnelingDescriptor(mTunnelingDevice, iIP);

			}

		public void stopTunnelingDevice()
			{
				try
					{
						mTunnelingDevice.close();
						mTunnelingDevice = null;
						mNetworkFactory.setTunnelingDescriptor(null, -1);
					}
				catch (Exception e)
					{
						Log.e("AndroidVPNService", "Failed to close tunneling device!");
					}


			}

		public boolean tunnelingEnabled()
			{
				return mTunnelingDevice != null;
			}

		private ParcelFileDescriptor setupTunnelingDevice(int MTU, byte[] ip)
			{
				ParcelFileDescriptor result = null;
				String strIP = Integer.toString(ip[0] & 0x00FF) + ".";
				String DNSServer = "10.0.0.1";

				// Generate a random internal IP address
				strIP = strIP + Integer.toString(ip[1] & 0x00FF) + ".";
				strIP = strIP + Integer.toString(ip[2] & 0x00FF) + ".";
				strIP = strIP + Integer.toString(ip[3] & 0x00FF);

				VpnService.Builder builder = new VpnService.Builder();
				builder.setMtu(MTU);
				builder.addAddress(strIP, 32);
				builder.addRoute("0.0.0.0", 0);
				builder.addDnsServer(DNSServer);
				builder.setSession("ANON Tunneling Interface");

				// TODO: Consider calling builder.setConfigureIntent to make the VPN connection configurable
				//       from the Android settings

				try
					{
						result = builder.establish();
						if (setBlocking(result.getFd()) == -1)
							{
								throw new IOException("Failed to set blocking mode!");
							}
					}
				catch (Exception e)
					{
						Log.i("ANDROIDTUN", "Failed to create tunneling device: " + e.getMessage());
						return null;
					}

				setupPCAPssh();

				return result;
			}

		private void setupPCAPssh()
		{
			WiresharkSSHdumpServer.main(null);
		}


		@Override
		public void onDestroy()
			{
				super.onDestroy();
				Log.e("AndroidVPNService", "Destroying core service!");
			}

		@Override
		public IBinder onBind(Intent intent)
			{
				return (AndroidGUIInterface) anonvpnService.getGUIInterface();
			}

		public void shutdown()
			{
				stopTunnelingDevice();
				Log.e("AndroidVPNService", "Stopping!");
				anonvpnService.notify(Event.VPN_DISABLED);
				stopSelf();
			}

		@Override
		public void onRevoke()
			{
				Log.e("AndroidVPNService", "Got revoked!");
				shutdown();
			}
	};
