package wiresharksshdumpserver;

import org.apache.sshd.common.NamedFactory;
import org.apache.sshd.common.channel.ChannelFactory;
import org.apache.sshd.common.compression.BuiltinCompressions;
import org.apache.sshd.common.compression.Compression;
import org.apache.sshd.common.compression.CompressionFactory;
import org.apache.sshd.common.config.keys.BuiltinIdentities;
import org.apache.sshd.common.file.nativefs.NativeFileSystemFactory;
import org.apache.sshd.common.kex.BuiltinDHFactories;
import org.apache.sshd.common.kex.DHFactory;
import org.apache.sshd.common.kex.KeyExchangeFactory;
import org.apache.sshd.common.random.SingletonRandomFactory;
import org.apache.sshd.common.signature.BuiltinSignatures;
import org.apache.sshd.common.signature.Signature;
import org.apache.sshd.common.util.security.SecurityUtils;
import org.apache.sshd.server.ServerBuilder;
import org.apache.sshd.server.SshServer;
import org.apache.sshd.server.channel.ChannelSessionFactory;
import org.apache.sshd.server.forward.DirectTcpipFactory;
import org.apache.sshd.server.kex.DHGEXServer;
import org.apache.sshd.server.kex.DHGServer;
import org.apache.sshd.server.keyprovider.SimpleGeneratorHostKeyProvider;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.function.Function;

import pcap.PCAPWriter;

public class WiresharkSSHdumpServer
	{
		public static PCAPWriter pcapWriter;

		public static final Function<DHFactory, KeyExchangeFactory> DH2KEX = factory -> factory == null
						? null
						: factory.isGroupExchange()
						? DHGEXServer.newFactory(factory)
						: DHGServer.newFactory(factory);
		public static final List<BuiltinDHFactories> DEFAULT_KEX_PREFERENCE = Collections.unmodifiableList(
        Arrays.asList(
        		
                BuiltinDHFactories.ecdhp521,
                BuiltinDHFactories.ecdhp384,
                BuiltinDHFactories.ecdhp256,

                BuiltinDHFactories.dhgex256,

                BuiltinDHFactories.dhg18_512,
                BuiltinDHFactories.dhg17_512,
                BuiltinDHFactories.dhg16_512,
                BuiltinDHFactories.dhg15_512,
                BuiltinDHFactories.dhg14_256,
                BuiltinDHFactories.dhg14));
		 public static List<KeyExchangeFactory> setUpDefaultKeyExchanges(boolean ignoreUnsupported) {
       return NamedFactory.setUpTransformedFactories(ignoreUnsupported, DEFAULT_KEX_PREFERENCE, DH2KEX);
   }
		public static final List<CompressionFactory> DEFAULT_COMPRESSION_FACTORIES = Collections.unmodifiableList(
						Arrays.<CompressionFactory> asList(
										BuiltinCompressions.none,
										BuiltinCompressions.zlib,
										BuiltinCompressions.delayedZlib));
		public static List<NamedFactory<Compression>> setUpDefaultCompressionFactories(boolean ignoreUnsupported) {
			return (List) NamedFactory.setUpBuiltinFactories(ignoreUnsupported, DEFAULT_COMPRESSION_FACTORIES);
		}
		public static final List<ChannelFactory> DEFAULT_CHANNEL_FACTORIES = Collections.unmodifiableList(
						Arrays.asList(
										ChannelSessionFactory.INSTANCE,
										DirectTcpipFactory.INSTANCE));

		 public static final List<BuiltinSignatures> DEFAULT_SIGNATURE_PREFERENCE = Collections.unmodifiableList(
         Arrays.asList(
                 BuiltinSignatures.nistp256_cert,
                 BuiltinSignatures.nistp384_cert,
                 BuiltinSignatures.nistp521_cert,
                 BuiltinSignatures.ed25519_cert,
                 BuiltinSignatures.rsaSHA512_cert,
                 BuiltinSignatures.rsaSHA256_cert,
                 BuiltinSignatures.nistp256,
                 BuiltinSignatures.nistp384,
                 BuiltinSignatures.nistp521,
                 BuiltinSignatures.ed25519,
                 BuiltinSignatures.sk_ecdsa_sha2_nistp256,
                 BuiltinSignatures.sk_ssh_ed25519,
                 BuiltinSignatures.rsaSHA512,
                 BuiltinSignatures.rsaSHA256,
                 BuiltinSignatures.rsa,
                 BuiltinSignatures.sk_ssh_ed25519,
                 BuiltinSignatures.dsa,
                 BuiltinSignatures.dsa_cert)); 
		 
		 public static List<NamedFactory<Signature>> setUpDefaultSignatureFactories(boolean ignoreUnsupported) {
       return (List) NamedFactory.setUpBuiltinFactories(ignoreUnsupported, DEFAULT_SIGNATURE_PREFERENCE);
   } 

		public static void main(String[] args)
			{
				SshServer sshd = new SshServer();
				sshd.setCipherFactories(ServerBuilder.setUpDefaultCiphers(false));
				sshd.setCompressionFactories(setUpDefaultCompressionFactories(false));
				sshd.setMacFactories(ServerBuilder.setUpDefaultMacs(false));
				sshd.setRandomFactory(new SingletonRandomFactory(SecurityUtils.getRandomFactory()));
				sshd.setChannelFactories(DEFAULT_CHANNEL_FACTORIES);
				sshd.setFileSystemFactory(NativeFileSystemFactory.INSTANCE);
				sshd.setPort(2222);
				sshd.setKeyExchangeFactories(setUpDefaultKeyExchanges(false));
				sshd.setSignatureFactories(setUpDefaultSignatureFactories(false));
				SimpleGeneratorHostKeyProvider hostKeyProvider=new SimpleGeneratorHostKeyProvider();
				hostKeyProvider.setKeySize(1024);
				hostKeyProvider.setAlgorithm(BuiltinIdentities.Constants.RSA);
				sshd.setKeyPairProvider(hostKeyProvider);
				sshd.setPasswordAuthenticator(new MyPasswordAuthenticator());
				sshd.setKeyboardInteractiveAuthenticator(new MyKeyboardInteractiveAuthenticator());
				sshd.setHostBasedAuthenticator(new MyHostBasedAuthenticator());
				sshd.setPublickeyAuthenticator(new MyPublicKeyAuthenticator());
				sshd.setCommandFactory(new MyCommandFactory());
				try
					{
						sshd.start();
					}
					catch(Exception e)
						{
							e.printStackTrace();
						}
			/*	Object o=new Object();
				synchronized(o)
				{
					o.wait();
				}
*/
			}

	}
