package anon.anonudp.mixchannel;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Random;

import anon.anonudp.Constants;
import anon.anonudp.exception.DecryptionFailed;
import anon.anonudp.exception.PacketCreationFailed;
import anon.anonudp.mixmessage.Fragment;
import anon.anonudp.mixmessage.FragmentPool;
import anon.anonudp.mixmessage.Util;
import anon.anonudp.mixmessage.crypto.Counter;
import anon.anonudp.mixmessage.crypto.MyReplayDetection;
import anon.anonudp.mixmessage.crypto.PublicKey;
import anon.anonudp.mixpacket.DataPacket;
import anon.anonudp.mixpacket.IPacket;
import anon.anonudp.mixpacket.PacketFactory;

/*
TODO: Detect the response message ids running out
TODO: implement Channel Timeout
TODO: Use Short for Channel-ID?
 */
public class Channel
	{
		public static final int ID_SIZE = 2; // byte
		static final int HIGHEST_ID = (int) Math.pow(2, Constants.BYTE_SIZE * ID_SIZE) - 1;
		public static final HashMap table = new HashMap();

		private int id;
		private PacketFactory packetFactory;

		private Counter requestCounter;
		private MyReplayDetection responseReplay;

		private boolean initialized;

		private FragmentPool fragmentPool;

		public Channel(IPv4AndPort destination, PublicKey[] mixPublicKeys) throws IOException
			{
				this.id = Channel.randomID();


				this.packetFactory = new PacketFactory(id, destination.toBytes(), mixPublicKeys);

				this.requestCounter = new Counter();
				this.responseReplay = new MyReplayDetection();

				this.initialized = false;

				this.fragmentPool = new FragmentPool();

				Channel.table.put(new Integer(id), this);
			}

		public IPacket[] request(byte[] udpPayload) throws PacketCreationFailed
			{
				ArrayList returnPackets = new ArrayList();

				this.requestCounter.count();
				int fragmentNumber = 0;

				while (udpPayload!=null&&udpPayload.length > 0)
					{
						Fragment fragment;

						IPacket packet;

						if (this.initialized)
							{
								fragment = new Fragment(this.requestCounter.asInt(), fragmentNumber, udpPayload,
										Fragment.DATA_PAYLOAD_SIZE);

								packet = this.packetFactory.makeDataPacket(fragment);
							}
						else
							{
								fragment = new Fragment(this.requestCounter.asInt(), fragmentNumber, udpPayload,
										Fragment.INIT_PAYLOAD_SIZE);

								packet = this.packetFactory.makeInitPacket(fragment);
							}

						returnPackets.add(packet);

						udpPayload = Util.copyOfRange(udpPayload, fragment.getPayload().length, udpPayload.length);

						++fragmentNumber;
					}

				return (IPacket[]) returnPackets.toArray(new IPacket[0]);
			}

		public void response(IPacket response) throws DecryptionFailed, IllegalArgumentException,IllegalStateException
			{

				if (response.getPacketType() == IPacket.TYPE_INIT_RESPONSE)
					{//Init Response Packets do not have replay protection??
						this.initialized = true;
					}
				else
					{
						if (!this.responseReplay.isValid(response.getMessageID()))
							throw new IllegalStateException("Response Replay detected.");
						DataPacket packet = (DataPacket) response;
						//Check if transformation done right...
/*						byte channelKeys[][] = this.packetFactory.getResponseChannelKeys();
						for (int j = 0; j < channelKeys.length; j++)
							{
								packet = packetFactory.process(packet, channelKeys[j]);
							}
	*/
						packet=packetFactory.myprocess(packet);
						Fragment fragment = new Fragment(packet.getData());

						this.fragmentPool.addFragment(fragment);
					}
			}

		private static int randomID()
			{
				if (table.size() == HIGHEST_ID)
					throw new IllegalStateException("No free channel ids available.");

				Integer channelID;
				Random r = new Random();

				do
					{
						channelID = new Integer(r.nextInt(HIGHEST_ID + 1));
					}
				while (Channel.table.containsKey(channelID));

				return channelID.intValue();
			}

		static void _removeAllChannels()
			{
				Channel.table.clear();
			}

		void _setInitialized()
			{
				this.initialized = true;
			}

		public boolean hasNext()
			{
				return this.fragmentPool.hasNext();
			}

		public byte[] next()
			{
				return this.fragmentPool.next();
			}
	}
