package anon.anonudp.mixmessage.crypto;

import mybouncycastle.org.bouncycastle.crypto.engines.AESFastEngine;
import mybouncycastle.org.bouncycastle.crypto.modes.GCMBlockCipher;
//import org.bouncycastle.crypto.params.AEADParameters;
import mybouncycastle.org.bouncycastle.crypto.params.KeyParameter;

import anon.anonudp.Constants;
import anon.anonudp.exception.DecryptionFailed;
import anon.anonudp.exception.EncryptionFailed;
import anon.anonudp.mixchannel.Channel;
import anon.anonudp.mixpacket.DataPacket;
import anon.anonudp.mixpacket.IPacket;
import anon.anonudp.mixpacket.InitResponse;
import anon.client.crypto.MyAEADParameters;

final public class LinkEncryption
	{
		private static final int HEADER_SIZE = Channel.ID_SIZE + Counter.SIZE + IPacket.TYPE_BYTE_SIZE;
		public static final int OVERHEAD = Counter.SIZE + HEADER_SIZE + Util.GCM_MAC_SIZE;
		
		private final static int IV_LENGTH=12;
		
		private final GCMBlockCipher m_cipherEncrypt;
		private final byte[] m_encryptIV;
		private MyAEADParameters m_encryptKeySpec = null;
		private final Counter m_encryptCounter;
		private final byte[] m_plainHeader;

		private final GCMBlockCipher m_cipherDecrypt;
		private final byte[] m_decryptIV;
		private MyAEADParameters m_decryptKeySpec = null;
		private final byte[] m_decryptPlainLinkHeader;

		public LinkEncryption(byte[] linkKey)
			{
				m_encryptCounter = new Counter();
				m_cipherEncrypt = new GCMBlockCipher(new AESFastEngine());
				m_encryptIV = new byte[IV_LENGTH];
				//we init the encryption cipher here initially with the key. Afterwards we only set the new IV.
				m_encryptKeySpec = new MyAEADParameters(new KeyParameter(linkKey), Constants.BYTE_SIZE * Util.GCM_MAC_SIZE,
						m_encryptIV, null);
				m_cipherEncrypt.init(true, m_encryptKeySpec);
				m_plainHeader = new byte[HEADER_SIZE];

				m_cipherDecrypt = new GCMBlockCipher(new AESFastEngine());
				//we init the decryption cipher here initially with the key. Afterwards we only set the new IV.
				m_decryptIV = new byte[IV_LENGTH];
				m_decryptKeySpec = new MyAEADParameters(new KeyParameter(linkKey), Constants.BYTE_SIZE * Util.GCM_MAC_SIZE,
						m_decryptIV, null);
				m_cipherDecrypt.init(false, m_decryptKeySpec);
				m_decryptKeySpec = new MyAEADParameters(null, Constants.BYTE_SIZE * Util.GCM_MAC_SIZE, m_decryptIV, null);
				m_decryptPlainLinkHeader = new byte[HEADER_SIZE + Util.GCM_MAC_SIZE];
			}

		public int encrypt(IPacket packet,byte[] encryptedBytes) throws EncryptionFailed
			{
				try
					{
						byte[] payload = packet.getData();
	
						m_encryptCounter.count();
						m_encryptCounter.asIV(m_encryptIV);
						m_cipherEncrypt.init(true, m_encryptKeySpec);

						System.arraycopy(m_encryptIV, 0, encryptedBytes, 0, Counter.SIZE);

						int c = packet.getChannelID();
						m_plainHeader[0] = (byte) ((c >> 8) & 0x00FF);
						m_plainHeader[1] = (byte) (c & 0x00FF);

						//int outOffset = m_cipherEncrypt.processBytes(in, 0, in.length, out, 0);

						byte[] in = packet.getMessageID();
						System.arraycopy(in, 0, m_plainHeader, 2, in.length);

						m_plainHeader[2 + in.length] = packet.getPacketType();

						int outOffset = m_cipherEncrypt.processBytes(m_plainHeader, 0, HEADER_SIZE, encryptedBytes, 4);
						outOffset += m_cipherEncrypt.doFinal(encryptedBytes, outOffset + 4);

						//bos.write(out, 0, outOffset);

						System.arraycopy(payload, 0, encryptedBytes, outOffset + 4, payload.length);
						return payload.length + OVERHEAD;
					}
				catch (Exception e)
					{
						throw new EncryptionFailed("Couldn't link-encrypt the mix packet " + this.m_encryptCounter.asInt() + ".", e);
					}

			}

		/*		
				public byte[] encrypt(IPacket packet) throws EncryptionFailed
					{
						this.counter.count();
		
						m_encryptKeySpec = new AEADParameters(null, Constants.BYTE_SIZE * Util.GCM_MAC_SIZE, counter.asIV(), null);
						m_cipherEncrypt.init(true, m_encryptKeySpec);
		
						ByteArrayOutputStream bos = new ByteArrayOutputStream();
		
						try
							{
								bos.write(this.counter.asBytes());
		
								byte[] out = new byte[DataPacket.SIZE];
								byte[] in = new byte[2];
								int c = packet.getChannelID();
								in[0] = (byte) ((c >> 8) & 0x00FF);
								in[1] = (byte) (c & 0x00FF);
		
								int outOffset = m_cipherEncrypt.processBytes(in, 0, in.length, out, 0);
		
								in = packet.getMessageID();
		
								outOffset += m_cipherEncrypt.processBytes(in, 0, in.length, out, outOffset);
								outOffset += m_cipherEncrypt.processByte(packet.getPacketType(), out, outOffset);
								outOffset += m_cipherEncrypt.doFinal(out, outOffset);
		
								bos.write(out, 0, outOffset);
		
								bos.write(packet.getData());
							}
						catch (Exception e)
							{
								throw new EncryptionFailed("Couldn't link-encrypt the mix packet " + this.counter.asInt() + ".", e);
							}
		
						return bos.toByteArray();
		
					}
		*/
		public IPacket decrypt(byte[] packetBytes) throws DecryptionFailed
			{
				try
					{
						System.arraycopy(packetBytes, 0, m_decryptIV, 0, Counter.SIZE);
						m_cipherDecrypt.init(false, m_decryptKeySpec);

						byte[] payload = new byte[packetBytes.length - OVERHEAD];
						System.arraycopy(packetBytes, OVERHEAD, payload, 0, payload.length);

						int outOffset = m_cipherDecrypt.processBytes(packetBytes, Counter.SIZE, HEADER_SIZE + Util.GCM_MAC_SIZE,
								m_decryptPlainLinkHeader, 0);

						outOffset += m_cipherDecrypt.doFinal(m_decryptPlainLinkHeader, outOffset);

						int channelID = (m_decryptPlainLinkHeader[0] & 0x00FF) << 8;
						channelID |= (m_decryptPlainLinkHeader[1] & 0x00FF);
						byte messageType = m_decryptPlainLinkHeader[6];

						IPacket returnPacket = null;
						if (messageType == IPacket.TYPE_DATA)
							{
								byte[] messagePrefix = new byte[Counter.SIZE];
								System.arraycopy(m_decryptPlainLinkHeader, 2, messagePrefix, 0, messagePrefix.length);
								returnPacket = new DataPacket(channelID, messagePrefix, payload);
							}
						else if (messageType == IPacket.TYPE_INIT_RESPONSE)
							returnPacket = new InitResponse(channelID, payload);

						return returnPacket;
					}
				catch (Exception e)
					{
						throw new DecryptionFailed("Couldn't link-decrypt the mix packet ", e);
					}
			}
	}
