package anon.anonudp.mixmessage.crypto;

/*** This class implements the replay detection based on a sliding window mechanism.
 * The sliding windows is represented a bit vector. Each bit in the bit vector stands for a message id,
 * whereas the specific message id = lower bound + bit position.
 *  If a message ID is below the lower bound of the sliding window, it is reported as replay.
 *  If a message ID is between the bounds of the sliding window, it is first checked, if teh corresponding bit is set.
 *  In this case, the message ID is reported as replay.
 *  Otherwise the bit is set and the window will be slided. 
 * @author sk13
 *
 */
final public class MyReplayDetection
	{

		private int m_iLowestID = -20;
		private int m_iCurrentIDs = 0;

		public MyReplayDetection()
			{
			}

		public boolean isValid(byte[] counterPrefix)
			{
				int packetID = counterPrefix[0];
				packetID<<=8;
				packetID|=(counterPrefix[1]&0x00FF);
				packetID<<=8;
				packetID|=(counterPrefix[2]&0x00FF);
				packetID<<=8;
				packetID|=((counterPrefix[3])&0x00FF);
				return isValid(packetID);
			}
		
		public boolean isValid(int packetID)
			{
				if (packetID <= m_iLowestID)
					return false;
				int bitPos = packetID - m_iLowestID - 1;
				if (bitPos > 30)
					{
						int shift = bitPos - 30;
						m_iCurrentIDs >>= shift;
						m_iCurrentIDs |= 0x40000000;
						m_iLowestID += shift;
						return true;
					}
				else
					{
						int bit = (1 << bitPos);
						if ((m_iCurrentIDs & bit) != 0)
							{
								return false;
							}
						m_iCurrentIDs |= bit;
						m_iCurrentIDs >>= 1;
						m_iLowestID++;
						return true;
					}
			}
	}
