package anon.anonudp.mixpacket;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Arrays;

import anon.anonudp.Constants;
import anon.anonudp.mixchannel.IPv4AndPort;
import anon.anonudp.mixmessage.crypto.EccGroup713;
import anon.anonudp.mixmessage.crypto.PublicKey;

public class InitPacket implements IPacket
{
    public static final int CHANNEL_KEY_ONION_SIZE = 2 * EccGroup713.SYMMETRIC_KEY_LENGTH * Constants.MIX_SERVER_COUNT;
    public static final int PAYLOAD_SIZE = IPv4AndPort.SIZE;

    private final int channelID;
    private final byte[] messageID;

    private final PublicKey publicKey;
    private final byte[] channelKeyOnion;
    private final byte[] payloadOnion;

    public InitPacket(int channelID, byte[] messageID, PublicKey publicKey, byte[] channelKeyOnion, byte[] payloadOnion)
    {
        this.channelID = channelID;

        this.messageID = messageID;

        this.publicKey = publicKey;
        this.channelKeyOnion = channelKeyOnion;
        this.payloadOnion = payloadOnion;
    }

    PublicKey getPublicKey() {
        return publicKey;
    }

    byte[] getChannelKeyOnion() {
        return channelKeyOnion;
    }

    public byte[] getPayloadOnion() {
        return payloadOnion;
    }

    private byte[] toBytes() throws IOException {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();

        bos.write(publicKey.toBytes());

        bos.write(channelKeyOnion);

        bos.write(payloadOnion);

        return bos.toByteArray();
    }

    public byte getPacketType() {
        return IPacket.TYPE_INIT;
    }

    public byte[] getMessageID() {
        return this.messageID;
    }

    public int getChannelID() {
        return this.channelID;
    }

    public byte[] getData() throws IOException {
        return this.toBytes();
    }

    public boolean equals(Object obj) {
        if (obj instanceof InitPacket)
            return this.equals((InitPacket) obj);

        return super.equals(obj);
    }

    boolean equals(InitPacket otherPacket)
    {
        boolean is_equal = this.channelID ==otherPacket.getChannelID();
        is_equal = is_equal && Arrays.equals(this.messageID, otherPacket.getMessageID());
        is_equal = is_equal && this.publicKey == otherPacket.getPublicKey();
        is_equal = is_equal && this.channelKeyOnion == otherPacket.getChannelKeyOnion();
        is_equal = is_equal &&  this.payloadOnion == otherPacket.getPayloadOnion();

        return is_equal;
    }
}