package anon.anonudp.mixpacket;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import mybouncycastle.org.bouncycastle.crypto.InvalidCipherTextException;

import anon.anonudp.exception.DecryptionFailed;
import anon.anonudp.exception.PacketCreationFailed;
import anon.anonudp.exception.SymmetricKeyCreationFailed;
import anon.anonudp.mixmessage.Fragment;
import anon.anonudp.mixmessage.Util;
import anon.anonudp.mixmessage.crypto.CTRCipher;
import anon.anonudp.mixmessage.crypto.Counter;
import anon.anonudp.mixmessage.crypto.EccGroup713;
import anon.anonudp.mixmessage.crypto.MyCTRCipher;
import anon.anonudp.mixmessage.crypto.PrivateKey;
import anon.anonudp.mixmessage.crypto.PublicKey;

public class PacketFactory
	{
		private  int channelID;
		private  byte[] initPayload;

		private PublicKey[] publicKeys;
		private  byte[][] requestChannelKeys;
		private  byte[][] responseChannelKeys;
		private MyCTRCipher[] m_arcipherDecrypt;

		private int mixCount;
		private Counter requestCounter;

		public PacketFactory(int channelID, byte[] initPayload, PublicKey[] publicKeys)
			{
				this.requestChannelKeys = new byte[publicKeys.length][EccGroup713.SYMMETRIC_KEY_LENGTH];
				this.responseChannelKeys = new byte[publicKeys.length][EccGroup713.SYMMETRIC_KEY_LENGTH];
				m_arcipherDecrypt=new MyCTRCipher[publicKeys.length];
				for (int i = 0; i < publicKeys.length; ++i)
					{
						//all 0 for Debug....
						//this.requestChannelKeys[i] = Util.randomBytes(EccGroup713.SYMMETRIC_KEY_LENGTH);
						//this.responseChannelKeys[i] = Util.randomBytes(EccGroup713.SYMMETRIC_KEY_LENGTH);
						m_arcipherDecrypt[i]=MyCTRCipher.getCipher(responseChannelKeys[i], MyCTRCipher.DECRYPT_MODE);
					}

				this.channelID = channelID;
				this.initPayload = initPayload;

				if (publicKeys.length < 1)
					throw new IllegalArgumentException("Was given empty public key list.");

				this.publicKeys = publicKeys;

				this.mixCount = this.publicKeys.length;
				this.requestCounter = new Counter();
				this.requestCounter.count();
			}

		public PacketFactory(byte[][] symKeys)
		{
			m_arcipherDecrypt=new MyCTRCipher[symKeys.length];
			this.responseChannelKeys = new byte[symKeys.length][EccGroup713.SYMMETRIC_KEY_LENGTH];

			for (int i = 0; i < symKeys.length; ++i)
				{
					m_arcipherDecrypt[i]=MyCTRCipher.getCipher(symKeys[i], MyCTRCipher.DECRYPT_MODE);
					this.responseChannelKeys[i]=symKeys[i];
				}
			requestChannelKeys=null;
		}

		ProcessedInitPacket process(InitPacket packet, PrivateKey privateKey)
				throws SymmetricKeyCreationFailed, DecryptionFailed
			{

				/* create shared key, originally used to encrypt */
				PublicKey disposableKey = packet.getPublicKey().blind(privateKey).blind(packet.getMessageID());

				byte[] symmetricDisposableKey = disposableKey.toSymmetricKey();

				/* decrypt channel key and payload */

				CTRCipher cipher = CTRCipher.getCipher(symmetricDisposableKey, new Counter(packet.getMessageID()).asIV(),
						CTRCipher.DECRYPT_MODE);

				try
					{
						byte[] decryptedChannelOnion = cipher.decryptBuffer(packet.getChannelKeyOnion());

						byte[] requestChannelKey = new byte[EccGroup713.SYMMETRIC_KEY_LENGTH];
						byte[] responseChannelKey = new byte[EccGroup713.SYMMETRIC_KEY_LENGTH];
						byte[] encryptedChannelOnion = new byte[InitPacket.CHANNEL_KEY_ONION_SIZE
								- 2 * EccGroup713.SYMMETRIC_KEY_LENGTH];

						ByteArrayInputStream bis = new ByteArrayInputStream(decryptedChannelOnion);

						boolean decryptionFailed = bis.read(requestChannelKey) != requestChannelKey.length
								|| bis.read(responseChannelKey) != responseChannelKey.length
								|| bis.read(encryptedChannelOnion) != encryptedChannelOnion.length;

						if (decryptionFailed)
							{
								throw new DecryptionFailed(
										"The channel key onion of the packet was not big enough to extract all parts.");
							}

						ByteArrayOutputStream bos = new ByteArrayOutputStream();

						bos.write(encryptedChannelOnion);
						bos.write(Util.randomBytes(2 * EccGroup713.SYMMETRIC_KEY_LENGTH));

						byte[] processedChannelOnion = bos.toByteArray();

						if (processedChannelOnion.length != packet.getChannelKeyOnion().length)
							{
								throw new DecryptionFailed("Processed channel onion came out shorter than expected.");
							}

						byte[] processedPayloadOnion = cipher.decryptBuffer(packet.getPayloadOnion());

						/* generate next public key */

						PublicKey newElement = packet.getPublicKey().blind(disposableKey);

						return new ProcessedInitPacket(this.channelID, packet.getMessageID(), requestChannelKey, responseChannelKey,
								newElement, processedChannelOnion, processedPayloadOnion);
					}
				catch (Exception e)
					{
						throw new DecryptionFailed("Couldn't decrypt init packet.", e);
					}
			}

		public static ProcessedDataPacket process(DataPacket packet, byte[] channelKey) throws DecryptionFailed
			{
				Counter counter = new Counter(packet.getMessageID());

				CTRCipher cipher = CTRCipher.getCipher(channelKey, counter.asIV(), CTRCipher.DECRYPT_MODE);

				try
					{
						return new ProcessedDataPacket(packet.getChannelID(), packet.getMessageID(),
								cipher.decryptBuffer(packet.getData()));
					}
				catch (InvalidCipherTextException e)
					{
						throw new DecryptionFailed("Could not decrypt data packet.", e);
					}
			}

		public  DataPacket myprocess(DataPacket packet) throws DecryptionFailed
			{
				Counter counter = new Counter(packet.getMessageID());

				byte[] iv=counter.asIV();
				byte[] buff=packet.getData();
			//	byte[] plainText=  new byte[buff.length];

				try
					{
						for(int i=0;i<this.m_arcipherDecrypt.length;i++)
							{
								m_arcipherDecrypt[i].decryptBuffer(buff,buff, iv);
				//				byte[] t=plainText;
					//			plainText=buff;
						//		buff=t;
							}
						return packet;//new ProcessedDataPacket(packet.getChannelID(), packet.getMessageID(),
								//buff);
					}
				catch (InvalidCipherTextException e)
					{
						throw new DecryptionFailed("Could not decrypt data packet.", e);
					}
			}


		public InitPacket makeInitPacket(Fragment fragment) throws PacketCreationFailed
			{
				this.requestCounter.count();

				PublicKey[] disposableKeys = new PublicKey[this.publicKeys.length];

				PrivateKey privateMessageKey = new PrivateKey();
				PublicKey publicMessageKey = new PublicKey(privateMessageKey);

				/* create shared disposable keys */

				try
					{
						disposableKeys[0] = this.publicKeys[0].blind(privateMessageKey).blind(this.requestCounter.asBytes());

						for (int i = 1; i < this.publicKeys.length; ++i)
							{
								privateMessageKey = privateMessageKey.blind(disposableKeys[i - 1]);

								disposableKeys[i] = this.publicKeys[i].blind(privateMessageKey).blind(this.requestCounter.asBytes());
							}

						/* preparing "onions" */

						byte[] channelOnion = new byte[InitPacket.CHANNEL_KEY_ONION_SIZE];

						ByteArrayOutputStream bos = new ByteArrayOutputStream();
						bos.write(this.initPayload);
						bos.write(fragment.toBytes());

						byte[] payloadOnion = bos.toByteArray();

						bos.close();

						/* encrypt "onions" */

						CTRCipher cipher;
						bos = new ByteArrayOutputStream();

						for (int i = disposableKeys.length - 1; i >= 0; --i)
							{
								cipher = CTRCipher.getCipher(disposableKeys[i].toSymmetricKey(), this.requestCounter.asIV(),
										CTRCipher.ENCRYPT_MODE);

								bos.write(this.requestChannelKeys[i]);
								bos.write(this.responseChannelKeys[i]);
								bos.write(Util.copyOf(channelOnion, channelOnion.length - 2 * EccGroup713.SYMMETRIC_KEY_LENGTH));

								channelOnion = cipher.decryptBuffer(bos.toByteArray());

								payloadOnion = cipher.decryptBuffer(payloadOnion);

								bos.reset();
							}

						return new InitPacket(channelID, this.requestCounter.asBytes(), publicMessageKey, channelOnion,
								payloadOnion);
					}
				catch (Exception e)
					{
						throw new PacketCreationFailed("Couldn't create channel initialization packet.", e);
					}
			}

		public DataPacket makeDataPacket(Fragment fragment) throws PacketCreationFailed
			{
				try
					{
						byte[] encryptedFragment;
						encryptedFragment = fragment.toBytes();

						this.requestCounter.count();

						for (int i = this.mixCount - 1; i >= 0; --i)
							{
								CTRCipher cipher = CTRCipher.getCipher(this.requestChannelKeys[i], this.requestCounter.asIV(),
										CTRCipher.ENCRYPT_MODE);

								encryptedFragment = cipher.encryptBuffer(encryptedFragment);
							}

						return new DataPacket(this.channelID, this.requestCounter.asBytes(), encryptedFragment);
					}
				catch (Exception e)
					{
						throw new PacketCreationFailed("Couldn't create data packet.", e);
					}
			}

		public byte[][] getRequestChannelKeys()
			{
				return requestChannelKeys;
			}

		public byte[][] getResponseChannelKeys()
			{
				return responseChannelKeys;
			}

		public int getChannelID()
			{
				return channelID;
			}
	}
