/*
 * Copyright (c) 2006, The JAP-Team
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of the University of Technology Dresden, Germany nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package anon.client;


/**
 * This class is used to hold a data channel for use in a data chain. It holds
 * data on the processed upstream and downstream packets transmitted over the
 * channel.
 *
 * @author Stefan Lieske
 */
public class DataChainChannelListEntry {

  private AbstractDataChannel m_channel;

  private int m_processedDownstreamPackets;

  private Object m_internalSynchronization;

  private int m_processedUpstreamPackets;


  public DataChainChannelListEntry(AbstractDataChannel a_channel) {
    m_channel = a_channel;
    m_processedDownstreamPackets = 0;
    m_processedUpstreamPackets = 0;
    m_internalSynchronization = new Object();
  }


  public AbstractDataChannel getChannel() {
    return m_channel;
  }

  public int getProcessedDownstreamPackets() {
    synchronized (m_internalSynchronization) {
      return m_processedDownstreamPackets;
    }
  }

  public void incProcessedDownstreamPackets() {
    synchronized (m_internalSynchronization) {
      m_processedDownstreamPackets++;
    }
  }

  public int getProcessedUpstreamPackets() {
    synchronized (m_internalSynchronization) {
      return m_processedUpstreamPackets;
    }
  }

  public void incProcessedUpstreamPackets() {
    synchronized (m_internalSynchronization) {
      m_processedUpstreamPackets++;
    }
  }

}
