package anon.client;

import java.io.IOException;

import anon.AnonChannel;

public interface IAnonUDPChannel extends AnonChannel
	{
		 /***
		 * @param host
		 *          the domain name of the target host
		 * @param port
		 *          the target port number
		 */
		public void setDestination(String host,int port);

		public void setSourcePort(int port);

		
		
		/** Reads the payload of a whole UDP packet sent from the peer. If
		 * the provided buffer is too small a UDPPacketToLargeException is thrown.
		 *
		 * Note: Do not mix calls to getInputStream.read() and readPacket() --
		 * otherwise the UDP packet stream might get out of sync leading to a
		 * UDPPacketStreamDesynchronisationException.
		 *
		 * @param payload
		 * @param off
		 * @param len
		 * @return number of payload bytes read
		 * @throws IOException
		 * @throws UDPPacketStreamDesynchronisationException
		 * @throws UDPPacketToLargeException
		 */

		public int readPacket(byte[] b, int off, int len) throws IOException, UDPPacketStreamDesynchronisationException, UDPPacketToLargeException;

		public void writePacket(byte[] payload) throws IOException;

		public void writePacket(byte[] payload,int offset,int len) throws IOException;

		/**
		 * Sets the time to wait before actually sending some data over this channel
		 * in case the packet belonging to the underlying anonymisation protocol is
		 * not full yet.
		 * <p>
		 * This method can be used for performance optimisations / efficiency. One
		 * use case would be if you know that this channel will sent data
		 * immediately after connection establishment. Therefore you would call
		 * <code>setSentAutoFlushDelay</code> with some <code>delayvalue > 0</code>
		 * just before you call the <code>connect</code> method. If you call
		 * <code>Outputstream.write</code> within <code>delayvalue</code>
		 * milliseconds after the <code>connect</code> call then the data you intend
		 * to sent will be piggybacked on the connect request. This will save some
		 * roundtrip-time delay.
		 * <p>
		 * Additionally, using this method correctly could decrease the bandwidth
		 * overhead introduced by the underlying anonymisation protocol. Many
		 * anonymisation protocols fragment the data stream into fixed sized packets
		 * which are filled with dummy data if not enough real data is available.
		 * Therefore waiting a certain amount of time for additional real data can
		 * increase the channel utilisation.
		 *
		 * <p>
		 * Note: This method will not do anything at the moment as the
		 * implementation is not ready yet.
		 *
		 * @param msAutoFlushDelay
		 *          the amount of time in milliseconds to wait for new data before
		 *          the current sent buffer is flushed. Setting this value to
		 *          <code>0</code> will always auto flush immediately. Setting this
		 *          value to <code>-1</code> will only flush the sent buffer, if the
		 *          sent buffer is full, i.e. getOutputBufgferSize many bytes are
		 *          contained in the buffer.
		 */
		public void setSentAutoFlushDelay(int msAutoFlushDelay);
		
	}
