/*
 * Copyright (c) 2006, The JAP-Team
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of the University of Technology Dresden, Germany nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package anon.client;

import org.w3c.dom.Document;

import anon.util.XMLParseException;
import anon.util.XMLUtil;
import logging.LogHolder;
import logging.LogLevel;
import logging.LogType;
import anon.IServiceContainer;


/**
 * This abstract class provides methods to send and process XML formatted
 * control messages.
 * <p>
 * The actual processing method {@link
 * anon.client.XmlControlChannel#processXmlMessage(Document) processXmlMessage}
 * has to be implemented by subclasses.
 * </p>
 *
 * @author Stefan Lieske
 */
public abstract class XmlControlChannel extends StreamedControlChannel {

  public XmlControlChannel(int a_channelId, Multiplexer a_multiplexer, IServiceContainer a_serviceContainer,boolean bEncrypted) {
    super(a_channelId, a_multiplexer, a_serviceContainer,bEncrypted);
  }

  /**
   * This method allows sending an {@link org.w3c.dom#Document XML Document}
   * containing control information over a
   * {@link anon.client.StreamedControlChannel StreamedControlChannel}.
   *
   * @param docMsg
   * @return
   */
  public int sendXmlMessage(Document docMsg) {
    return sendByteMessage(XMLUtil.toByteArray(docMsg));
  }

  /**
   * Transforms the given byte array into a {@link org.w3c.dom.Document Document}
   * and calls the {@link anon.client.XmlControlChannel#processXmlMessage(Document)
   * processXmlMessage} method on it.
   */
  protected void processMessage(byte[] a_message) {
    try {
	    processXmlMessage(XMLUtil.toXMLDocument(a_message));
    }
    catch (XMLParseException e) {
      LogHolder.log(LogLevel.ERR, LogType.NET, "Error while parsing XML document!", e);
    }
  }

  /**
   * Method that processes the XML message received through the channel.
   *
   * @param a_document
   */
  protected abstract void processXmlMessage(Document a_document);

}
