/*
 Copyright (c) 2000, The JAP-Team
 All rights reserved.
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice,
 this list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation and/or
 other materials provided with the distribution.

 - Neither the name of the University of Technology Dresden, Germany nor the names of its contributors
 may be used to endorse or promote products derived from this software without specific
 prior written permission.


 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
package anon.client.crypto;

import java.math.BigInteger;

import org.w3c.dom.Element;
//import org.w3c.dom.Text;

import anon.ErrorCodes;
import anon.crypto.MyRSA;
import anon.crypto.MyRSAPrivateKey;
import anon.crypto.MyRSAPublicKey;
import anon.util.Base64;
import anon.util.XMLUtil;

public class ASymMixCipherPlainRSA implements IASymMixCipher
	{

		MyRSA m_RSA;
		private MyRSAPublicKey m_PublicKey;

		// protected byte[] tmpP;

		public ASymMixCipherPlainRSA()
			{
				m_RSA = new MyRSA();
				m_PublicKey = null;
				// tmpP = new byte[128];
			}

		/*
		 * (non-Javadoc)
		 * 
		 * @see anon.client.crypto.IASymMixCipher#encrypt(byte[], int, byte[], int)
		 */
		public int encrypt(byte[] from, int ifrom, byte[] to, int ito)
			{
				byte[] r = null;
				try
					{
						r = m_RSA.processBlock(from, ifrom, 128);
					}
				catch (Exception e)
					{// Hm should never happen
						return -1;
					}

				/*
				 * BigInteger P = null; if (from.length == 128) { P = new BigInteger(1,
				 * from); } else { System.arraycopy(from, ifrom, tmpP, 0, 128); P = new
				 * BigInteger(1, tmpP); }
				 */
				// BigInteger C = P.modPow(m_PublicKey.getPublicExponent(),
				// m_PublicKey.getModulus());
				// r =C.toByteArray();
				if (r.length == 128)
					{
						System.arraycopy(r, 0, to, ito, 128);
					}
				else if (r.length == 129)
					{
						System.arraycopy(r, 1, to, ito, 128);
					}
				else
					{
						for (int k = 0; k < 128 - r.length; k++)
							{
								to[ito + k] = 0;
							}
						System.arraycopy(r, 0, to, ito + 128 - r.length, r.length);
					}
				return 128;
			}

		/*
		 * (non-Javadoc)
		 * 
		 * @see anon.client.crypto.IASymMixCipher#getOutputBlockSize()
		 */
		public int getOutputBlockSize()
			{
				return 128;
			}

		public int getInputBlockSize()
			{
				return 128;
			}

		/*
		 * (non-Javadoc)
		 * 
		 * @see anon.client.crypto.IASymMixCipher#getPaddingSize()
		 */
		public int getPaddingSize()
			{
				return 0;
			}

		/*
		 * (non-Javadoc)
		 * 
		 * @see anon.client.crypto.IASymMixCipher#setPublicKey(java.math.BigInteger,
		 * java.math.BigInteger)
		 */
		public int setPublicKey(BigInteger modulus, BigInteger exponent)
			{
				if (modulus == null || exponent == null)
					{
						return ErrorCodes.E_UNKNOWN;
					}
				m_PublicKey = new MyRSAPublicKey(modulus, exponent);
				try
					{
						m_RSA.init(m_PublicKey);
					}
				catch (Exception e)
					{
						// Hm should never happen
						return ErrorCodes.E_INVALID_KEY;
					}
				return ErrorCodes.E_SUCCESS;
			}

		public int setPrivateKey(BigInteger modulus, BigInteger exponent,BigInteger privateExponent,
				BigInteger p,
				BigInteger q,
				BigInteger dP,
				BigInteger dQ,
				BigInteger qInv)
			{
				if (modulus == null || exponent == null|| privateExponent==null|| 
						 p==null|| 						 q==null|| 						 dP==null|| 					 dQ==null|| 						 qInv==null)
					{
						return ErrorCodes.E_UNKNOWN;
					}
				try
					{
						m_PublicKey = new MyRSAPublicKey(modulus, exponent);
						MyRSAPrivateKey key = new MyRSAPrivateKey(modulus, exponent, privateExponent,
								 p,						 q,						 dP,					 dQ,						 qInv);
						m_RSA.init(key);
					}
				catch (Exception e)
					{
						// Hm should never happen
						return ErrorCodes.E_INVALID_KEY;
					}
				return ErrorCodes.E_SUCCESS;
			}


		/*
		 * (non-Javadoc)
		 * 
		 * @see anon.client.crypto.IASymMixCipher#setPublicKey(org.w3c.dom.Element)
		 */
		public int setPublicKey(Element xmlKey)
			{
				try
					{
						Element elemRSAKEyValue = (Element) XMLUtil.getFirstChildByName(xmlKey,"RSAKeyValue");
						BigInteger n = getBigIntegerFromXml(elemRSAKEyValue, "Modulus");
						BigInteger e = getBigIntegerFromXml(elemRSAKEyValue, "Exponent");
						return setPublicKey(n, e);
					}
				catch (Exception e)
					{
						return ErrorCodes.E_UNKNOWN;
					}
			}

		public int setPrivateKey(Element xmlKey)
			{
				try
					{
						Element elemRSAKeyPair = null;
						//check if xmlKey is already the root of the key...
						if(xmlKey.getNodeName().equals("RSAKeyPair"))
							{
								elemRSAKeyPair=xmlKey;
							}
						else
							{
								elemRSAKeyPair=(Element) XMLUtil.getFirstChildByName(xmlKey,"RSAKeyPair");
							}
						BigInteger n = getBigIntegerFromXml(elemRSAKeyPair, "Modulus");
						BigInteger e = getBigIntegerFromXml(elemRSAKeyPair, "Exponent");
						BigInteger privateExponent= getBigIntegerFromXml(elemRSAKeyPair, "D");
						BigInteger p= getBigIntegerFromXml(elemRSAKeyPair, "P");
						BigInteger q= getBigIntegerFromXml(elemRSAKeyPair, "Q");
						BigInteger dP= getBigIntegerFromXml(elemRSAKeyPair, "DP");
						BigInteger dQ= getBigIntegerFromXml(elemRSAKeyPair, "DQ");
						BigInteger qInv= getBigIntegerFromXml(elemRSAKeyPair, "InverseQ");
						return setPrivateKey(n, e, privateExponent,  p, q, dP, dQ, qInv);
						
					}
				catch (Exception e)
					{
						return ErrorCodes.E_UNKNOWN;
					}
			}

		/*
		 * (non-Javadoc)
		 * 
		 * @see anon.client.crypto.IASymMixCipher#getPublicKey()
		 */
		public MyRSAPublicKey getPublicKey()
			{
				return m_PublicKey;
			}

		private BigInteger getBigIntegerFromXml(Element root, String name)
			{
				try
					{
						Element child = (Element) XMLUtil.getFirstChildByName(root,name);
						String value=XMLUtil.parseValue(child,(String) null);
						byte[] b = Base64.decode(value);
						return new BigInteger(1, b);
					}
				catch (Exception e)
					{
						return null;
					}
			}

		// END My Hack CRYPT */

		/*
		 * Cryptix... private Cipher cipherEnc=null; public JAPASymCipher() { try {
		 * cipherEnc=Cipher.getInstance("RSA/ECB/NONE"); } catch(Exception e) {
		 * JAPDebug.out(JAPDebug.EXCEPTION,JAPDebug.MISC,"Init Asym-Cipher:
		 * "+e.toString()); } }
		 * 
		 * public int encrypt(byte[] from,int ifrom,byte[] to,int ito) { try {
		 * cipherEnc.update(from,ifrom,128,to,ito); return 128; } catch(Exception e)
		 * { JAPDebug.out(JAPDebug.EXCEPTION,JAPDebug.MISC,"encrypt ASym-Cipher:
		 * "+e.toString()); return -1; } }
		 * 
		 * public int setPublicKey(BigInteger modulus,BigInteger exponent) { try {
		 * cipherEnc.init(Cipher.ENCRYPT_MODE,(Key)new
		 * RSAPrivateKeySpec(modulus,exponent)); return 0; } catch(Exception e) {
		 * JAPDebug.out(JAPDebug.EXCEPTION,JAPDebug.MISC,"setPublicKey ASym-Cipher:
		 * "+e.toString()); return -1; } }
		 */
	}
