package anon.client.crypto;

import java.math.BigInteger;

import org.w3c.dom.Element;

import anon.crypto.MyRSAPublicKey;

/**
 * This interface describes an asymmetrical cipher a mix uses. It holds the
 * public key of that mix and allows to query information about the
 * characteristics of that algorithm.
 */
public interface IASymMixCipher
	{
		/**
		 * Encrypts one block of data using an asymmetric encryption algorithm.
		 *
		 * @param from Byte buffer to take the plain text from.
		 * @param ifrom Offset in the plain text.
		 * @param to Byte buffer to write the cipher text to.
		 * @param ito Offset in the target buffer.
		 *
		 * @return Amount of processed bytes.
		 */
		public int encrypt(byte[] from, int ifrom, byte[] to, int ito);

		/**
		 * Returns the output block size of the asymmetric mix cipher.
		 *
		 * @return
		 */
		public int getOutputBlockSize();

		/**
		 * Returns the necessary input block size of the asymmetric mix cipher.
		 * @return
		 */
		public int getInputBlockSize();

		/**
		 * Returns the padding size used by the asymmetric mix cipher in bytes.
		 * @return
		 */
		public int getPaddingSize();

		public int setPublicKey(BigInteger modulus, BigInteger exponent);

		public int setPublicKey(Element xmlKey);

		public MyRSAPublicKey getPublicKey();

	}