/*
 * Copyright (c) 2006, The JAP-Team
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of the University of Technology Dresden, Germany nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package anon.client.crypto;

import java.util.Vector;


/**
 * Describes the cipher that is used, when sending MixPackets to a specific Mix
 * server.
 *
 * @author Stefan Lieske
 */
public interface IMixCipher {

	/**
	 * Encrypts a byte array using the underlying encryption algorithm for that
	 * mix.
	 *
	 * The given callback handlers will be called, when the packet was successfully
	 * encrypted.
	 *
	 * @param a_packet The byte array to be encrypted.
	 * @param a_virtualPacketLength The minimal length of the resulting packet.
	 * @param a_sendCallbackHandlers
	 * @return A byte array containing the result of the encryption, possibly
	 * padded with random bytes until a_virtualPacketLength bytes long.
	 */
  public byte[] encrypt(byte[] a_packet, int a_virtualPacketLength, Vector a_sendCallbackHandlers);

  /**
   * Decrypts the given byte array and saves the result in place.
   *
   * @param a_packet
   * @return true, if no errors occurred (including integrity checks)
   */
  public boolean decrypt(byte[] a_packet);

  /**
   * Returns the overhead an encryption operation with this mix brings with it.
   * This can change depending on if the message to be encrypted will be the
   * first one (keys have to be sent, etc).
   *
   * @return Number of bytes needed as overhead.
   */
  public int getNextPacketEncryptionOverhead();

}
