/*
 * Copyright (c) 2006, The JAP-Team
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of the University of Technology Dresden, Germany nor
 *     the names of its contributors may be used to endorse or promote
 *     products derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package anon.client.crypto;

//import java.security.SecureRandom;
import java.util.Vector;

import logging.LogHolder;
import logging.LogLevel;
import logging.LogType;
import anon.client.MixParameters;
import anon.util.Base64;

/**
 * @author Lukas Braune
 */
public class LastMixCipher implements IMixCipher
	{

		// private static SecureRandom ms_secureRandom;

		private boolean m_firstEncryptionPacket;

		private MixParameters m_mixParameters;

		private IDataChannelChipher m_dataChannelCipher;
		private boolean m_bDebug=false;

		/*
		 * static { // initialize the random-bytes generator ms_secureRandom = new
		 * SecureRandom(); }
		 */

		public LastMixCipher(MixParameters a_mixParameters, IDataChannelChipher a_dataChannelCipher,boolean bDebug)
			{
				m_firstEncryptionPacket = true;
				m_mixParameters = a_mixParameters;
				m_dataChannelCipher = a_dataChannelCipher;
				m_bDebug=bDebug;
			}

		/*
		 * public byte[] encrypt(byte[] a_packet, int a_virtualPacketLength, Vector
		 * a_sendCallbackHandlers) { int alreadyEncryptedBytes = 0; int
		 * alreadyOutputBytes = 0; int realPacketLength = a_packet.length; int
		 * keyOffset = 0; byte[] packet = null; byte[] encryptedPacket = null; if
		 * (m_firstEncryptionPacket) { // add the symmetric encryption key to the
		 * packet // realPacketLength = realPacketLength +
		 * m_dataChannelCipher.getKeys().length +
		 * m_mixParameters.getMixCipher().getPaddingSize(); if
		 * (a_virtualPacketLength > realPacketLength) { // // we shall encrypt some
		 * more dummy bytes to keep the stream // cipher synchronized // packet =
		 * new byte[a_virtualPacketLength]; } else { packet = new
		 * byte[realPacketLength]; } keyOffset =
		 * m_dataChannelCipher.getKeys().length; byte[] symmetricKey =
		 * m_dataChannelCipher.getKeys(); // // ensure that m < n for RSA ->
		 * symmetric key is the first part of // the message and should start with a
		 * 0-bit to be sure // symmetricKey[0] = (byte) (symmetricKey[0] & 0x7f); //
		 * now check for replay-detection int diff = (int)
		 * ((System.currentTimeMillis() / 1000) - MixParameters.m_referenceTime);
		 * 
		 * byte[] ReplayOffset = m_mixParameters.getCurrentReplayOffset(diff); if
		 * (ReplayOffset != null) { // // mix supports replay-detection -> modify
		 * the key of the // symmetric cipher with the current timestamp (we assume
		 * that // the packet is sent immediately after encryption) // for (int i =
		 * 0; i < ReplayOffset.length; i++) { symmetricKey[symmetricKey.length -
		 * ReplayOffset.length + i] = ReplayOffset[i]; } }
		 * 
		 * ReplayTimestamp timestampStructure =
		 * m_mixParameters.getReplayTimestamp(); if (timestampStructure != null) {
		 * // // mix supports replay-detection -> modify the key of the // symmetric
		 * cipher with the current timestamp (we assume that // the packet is sent
		 * immediately after encryption) // byte[] currentTimestamp =
		 * timestampStructure.getCurrentTimestamp();
		 * System.arraycopy(currentTimestamp, 0, symmetricKey, symmetricKey.length -
		 * currentTimestamp.length, currentTimestamp.length); } // all modifications
		 * are done -> initialize the cipher again //
		 * m_dataChannelCipher.setEncryptionKeysAES(symmetricKey);
		 * System.arraycopy(m_dataChannelCipher.getKeys(), 0, packet, 0,
		 * m_dataChannelCipher.getKeys().length); System.arraycopy(a_packet, 0,
		 * packet, m_dataChannelCipher.getKeys().length, a_packet.length);
		 * encryptedPacket = new byte[packet.length];
		 * ms_secureRandom.nextBytes(encryptedPacket); // do asymmetric encryption
		 * on the first part of the packet //
		 * m_mixParameters.getMixCipher().encrypt(packet, 0, encryptedPacket, 0);
		 * alreadyEncryptedBytes =
		 * m_mixParameters.getMixCipher().getInputBlockSize(); alreadyOutputBytes =
		 * m_mixParameters.getMixCipher().getOutputBlockSize();
		 * m_firstEncryptionPacket = false; } else { // do only symmetric encryption
		 * // if (a_virtualPacketLength > realPacketLength) { // // we shall encrypt
		 * some more dummy bytes to keep the stream // cipher synchronized // packet
		 * = new byte[a_virtualPacketLength]; } else { packet = new
		 * byte[realPacketLength]; } System.arraycopy(a_packet, 0, packet, 0,
		 * a_packet.length); encryptedPacket = new byte[packet.length];
		 * ms_secureRandom.nextBytes(encryptedPacket); } // do symmetric encryption
		 * // int payloadLen = ((a_packet[0] << 8) & 0x003F00) | (a_packet[1] &
		 * 0x00FF); try { m_dataChannelCipher.encryptGCM1(packet,
		 * alreadyEncryptedBytes, encryptedPacket, alreadyOutputBytes, payloadLen +
		 * 3 - alreadyEncryptedBytes + keyOffset); } catch (Exception e) {
		 * e.printStackTrace(); } if (realPacketLength < encryptedPacket.length) {
		 * // cut off the dummy bytes byte[] tempPacket = encryptedPacket;
		 * encryptedPacket = new byte[realPacketLength];
		 * System.arraycopy(tempPacket, 0, encryptedPacket, 0, realPacketLength); }
		 * return encryptedPacket; }
		 */
		public byte[] encrypt(byte[] a_packet, int a_virtualPacketLength, Vector a_sendCallbackHandlers)
			{
				int payloadLen = ((a_packet[0] << 8) & 0x003F00) | (a_packet[1] & 0x00FF);
				byte[] symEncryptedPacket = new byte[payloadLen + 3 + DataChannelCipher.getMACSize()]; // size=payloadLen+payloadHeaderLen+MAC
				byte[] symmetricKey = null;

				if (m_firstEncryptionPacket)
					{
						// add the symmetric encryption key to the packet //
						symmetricKey = m_dataChannelCipher.getKeys();
						//
						// ensure that m < n for RSA -> symmetric key is the first part of
						// the message and should start with a 0-bit to be sure
						//
						symmetricKey[0] = (byte) (symmetricKey[0] & 0x7f);

						// all modifications are done -> initialize the cipher again */
						m_dataChannelCipher.setEncryptionKeysAES(symmetricKey);
					}

				// do symmetric encryption */
				try
					{
						// Encrypt and add 16 Byte MAC at the end of the packet
						m_dataChannelCipher.encryptGCM1(a_packet, 0, symEncryptedPacket, 0, payloadLen + 3);
					}
				catch (Exception e)
					{
						e.printStackTrace();
					}

				if (!m_firstEncryptionPacket)
				{
					// only sym encryption we are done...
					return symEncryptedPacket;
				}

				/*
				 * This is the first packet we encrypt, so we need to send the sym
				 * key(s) this mix is supposed to use (and that we used above) with it.
				 *
				 * First we need to figure out the needed size for the packet we will
				 * send. If the sym encrypted part is short enough it might fit into the
				 * asym encrypted part, since it has a minimal output block size. If
				 * not, we need to append all the sym encrypted bytes that didn't fit.
				 */
				int asymInBlockSize = m_mixParameters.getMixCipher().getInputBlockSize();
				int asymOutBlockSize = m_mixParameters.getMixCipher().getOutputBlockSize();
				int asymPaddingSize = m_mixParameters.getMixCipher().getPaddingSize();

				/*
				 * create a buffer of at least the size of the asym output block size;
				 * more if the sym encrypted packet is too large
				 */

				byte[] asymOutBlock = new byte[Math.max(symmetricKey.length + symEncryptedPacket.length
						+ asymPaddingSize, asymOutBlockSize)];

				/*
				 * Create a buffer large enough to hold an input block of the asym
				 * cipher. First copy the sym key(s) and then as much of the sym
				 * encrypted packet into it as possible.
				 */

				byte[] asymInBlock = new byte[asymInBlockSize];

				// number of bytes left in the input block after adding the sym key(s)
				int asymPayloadSize = asymInBlock.length - symmetricKey.length;

				// copy the sym key(s) of this mix into the input block
				System.arraycopy(symmetricKey, 0, asymInBlock, 0, symmetricKey.length);

				// copy as many bytes of the sym encrypted packet into the asym packet as possible
				System.arraycopy(symEncryptedPacket, 0, asymInBlock, symmetricKey.length, Math.min(asymPayloadSize,symEncryptedPacket.length));

				// TODO: fill the remaining asymInBlock space (if any) with random data

				// do asymmetric encryption on the first part of the packet
				m_mixParameters.getMixCipher().encrypt(asymInBlock, 0, asymOutBlock, 0);

				if (symEncryptedPacket.length > asymPayloadSize )
					{
						/*
						 * the sym encrypted part did not fit into the asym input block so
						 * we have to append the rest (symEncryptedPacket.length -
						 * asymPayloadSize)
						 *
						 * technically asymOutBlock is no longer just asym encrypted, but...
						 */
						System.arraycopy(symEncryptedPacket, asymPayloadSize, asymOutBlock, asymOutBlockSize, symEncryptedPacket.length - asymPayloadSize);
					}

				m_firstEncryptionPacket = false;

				return asymOutBlock;
			}

		public boolean decrypt(byte[] a_packet)
			{
				try
					{
						if(m_bDebug)
						{
							LogHolder.log(LogLevel.DEBUG, LogType.CRYPTO, "LastMixDecryption - AN.ON Debug Mode Received Packet data: "+Base64.encode(a_packet, false));
						}
						m_dataChannelCipher.decryptGCM2(a_packet, 0, a_packet, 0, a_packet.length);
					}
				catch (Exception e)
					{
						LogHolder.log(LogLevel.WARNING, LogType.CRYPTO, "Decryption of MixPacket with integrity check failed: "+e.getMessage());
						return false;
					}
				return true;
			}

		public int getNextPacketEncryptionOverhead()
			{
				int overhead = DataChannelCipher.getMACSize();
				if (m_firstEncryptionPacket)
					{
						overhead += m_dataChannelCipher.getKeys().length + m_mixParameters.getMixCipher().getPaddingSize();
					}
				return overhead;
			}
	}
