/*
 Copyright (c) 2000, The JAP-Team
 All rights reserved.
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice,
 this list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation and/or
 other materials provided with the distribution.

 - Neither the name of the University of Technology Dresden, Germany nor the names of its contributors
 may be used to endorse or promote products derived from this software without specific
 prior written permission.


 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
package anon.client.crypto;

import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.Provider;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

final public class SymCipherCTR implements ISymCipher
	{
		Cipher m_aesEngine1;
		Cipher m_aesEngine2;

		byte[] m_iv1 = null;

		byte[] m_iv2 = null;

		byte[] m_aesKeys;

		public SymCipherCTR()
			{
				try
					{
						m_aesEngine1 = Cipher.getInstance("AES/CTR/NoPadding");
						m_aesEngine2 = Cipher.getInstance("AES/CTR/NoPadding");

					}
				catch (Exception e)
					{
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				m_aesKeys = null;
				m_iv1 = new byte[16];
				for (int i = 0; i < 16; i++)
					{
						m_iv1[i] = 0;
					}
				m_iv2 = new byte[16];
				for (int i = 0; i < 16; i++)
					{
						m_iv2[i] = 0;
					}
			}

		public String toString()
			{
				try
					{
						Provider p = m_aesEngine1.getProvider();
						return m_aesEngine1.getAlgorithm()+"[Provider: "+p.getName() + " (Version: " + p.getVersion() + ")]";
					}
				catch (Throwable t)
					{

					}
				return "unknown";

			}

		synchronized public int setEncryptionKeyAES(byte[] key)
			{
				return setEncryptionKeyAES(key, 0, 16);
			}

		/**
		 * 
		 * @param key
		 *          byte[]
		 * @param offset
		 *          int
		 * @param len
		 *          int if len==16 --> only the key is set; if len==32 --> key and
		 *          IV is set
		 * @return int
		 */
		synchronized public int setEncryptionKeyAES(byte[] key, int offset, int len)
			{
				try
					{
						m_aesKeys = new byte[16];
						System.arraycopy(key, offset, m_aesKeys, 0, 16);

						if (len == 16)
							{
								for (int i = 0; i < 16; i++)
									{
										m_iv1[i] = 0;
										m_iv2[i] = 0;
									}
							}
						else
							{
								for (int i = 0; i < 16; i++)
									{
										m_iv1[i] = key[i + 16 + offset];
										m_iv2[i] = key[i + 16 + offset];
									}
							}

						IvParameterSpec ivSpec1 = new IvParameterSpec(m_iv1);
						IvParameterSpec ivSpec2 = new IvParameterSpec(m_iv2);

						SecretKeySpec aesKey1 = new SecretKeySpec(m_aesKeys, "AES");
						SecretKeySpec aesKey2 = new SecretKeySpec(m_aesKeys, "AES");
						m_aesEngine1.init(Cipher.ENCRYPT_MODE, aesKey1, ivSpec1);
						m_aesEngine2.init(Cipher.DECRYPT_MODE, aesKey2, ivSpec2);

						return 0;
					}
				catch (Exception e)
					{
						m_aesKeys = null;

						return -1;
					}
			}

		/**
		 * Set both keys to different value, if keys.length==2* KEY_SIZe, else set
		 * both keys to the same value
		 * 
		 * @param keys
		 *          byte[]
		 * @return int
		 */
		synchronized public int setEncryptionKeysAES(byte[] keys)
			{
				try
					{
						if (keys.length == 16)
							{
								return setEncryptionKeyAES(keys);
							}
						else
							{
								m_aesKeys = new byte[32];
								System.arraycopy(keys, 0, m_aesKeys, 0, 32);
								for (int i = 0; i < 16; i++)
									{
										m_iv1[i] = 0;
										m_iv2[i] = 0;
									}

								IvParameterSpec ivSpec1 = new IvParameterSpec(m_iv1);
								IvParameterSpec ivSpec2 = new IvParameterSpec(m_iv2);

								SecretKeySpec aesKey1 = new SecretKeySpec(m_aesKeys, 0, 16, "AES");
								SecretKeySpec aesKey2 = new SecretKeySpec(m_aesKeys, 16, 16, "AES");
								m_aesEngine1.init(Cipher.ENCRYPT_MODE, aesKey1, ivSpec1);
								m_aesEngine2.init(Cipher.DECRYPT_MODE, aesKey2, ivSpec2);

								return 0;
							}
					}
				catch (Exception e)
					{
						m_aesKeys = null;

						return -1;
					}
			}

		/**
		 * Returns the currently used key(s) for encryption. If both keys are set,
		 * than an array of 32 byte is returned, otherwise an array of 16 bytes
		 * 
		 * @return The current key(s) used for encryption or null, if no key is set.
		 */
		public byte[] getKeys()
			{
				return m_aesKeys;
			}

		synchronized public void setIV2(byte[] buff)
			{
				for (int i = 0; i < 16; i++)
					{
						m_iv2[i] = buff[i];
					}

				IvParameterSpec ivSpec2 = new IvParameterSpec(m_iv2);
				SecretKeySpec aesKey2;
				if (m_aesKeys.length == 16)
					{
						aesKey2 = new SecretKeySpec(m_aesKeys, "AES");

					}
				else
					{
						aesKey2 = new SecretKeySpec(m_aesKeys, 16, 16, "AES");
					}
				try
					{
						m_aesEngine2.init(Cipher.DECRYPT_MODE, aesKey2, ivSpec2);
					}
				catch (InvalidKeyException e)
					{
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				catch (InvalidAlgorithmParameterException e)
					{
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
			}

		public int encryptAES1(byte[] from, int ifrom, byte[] to, int ito, int len)
			{
				try
					{
						//if(len%16==0)
						if (true)
							{
								m_aesEngine1.update(from, ifrom, len, to, ito);
							}
						else
							{
								int len1 = len & 0x0FFFFFF0;
								m_aesEngine1.update(from, ifrom, len1, to, ito);
								ifrom += len1;
								ito += len1;
								len1 = len - len1;
								byte[] tmpBuff = new byte[16];
								System.arraycopy(from, ifrom, tmpBuff, 0, len1);
								m_aesEngine1.update(tmpBuff, 0, 16, tmpBuff, 0);
								System.arraycopy(tmpBuff, 0, to, ito, len1);
							}
					}
				catch (Exception e)
					{
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				return 0;
			}

		public int encryptAES2(byte[] buff)
			{
				try
					{
						int len = buff.length;
						if (true/*len%16==0*/)
							{
								m_aesEngine2.update(buff, 0, len, buff, 0);
							}
						else
							{
								int len1 = len & 0x0FFFFFF0;
								m_aesEngine2.update(buff, 0, len1, buff, 0);
								int len2 = len - len1;
								byte[] tmpBuff = new byte[16];
								System.arraycopy(buff, len1, tmpBuff, 0, len2);
								m_aesEngine2.update(tmpBuff, 0, 16, tmpBuff, 0);
								System.arraycopy(tmpBuff, 0, buff, len1, len2);
							}
					}
				catch (Exception e)
					{
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				return 0;
			}

		public static void main(String argv[])
			{
				SymCipherCTR c = new SymCipherCTR();
				byte key[] = new byte[64];
				c.setEncryptionKeyAES(key);

				byte in[] = new byte[32];
				byte[] o1 = new byte[32];
				byte[] o2 = new byte[32];

				c.encryptAES1(in, 0, o1, 0, 17);
				c.encryptAES1(in, 17, o1, 17, 15);

				c.setEncryptionKeyAES(key);
				c.encryptAES1(in, 0, o2, 0, 32);

				c.encryptAES2(in);

				c.setEncryptionKeyAES(key);
				c.encryptAES1(o1, 0, in, 0, 17);
				c.encryptAES1(o1, 17, in, 17, 15);

				byte[] iv = new byte[16];
				for (int i = 0; i < 16; i++)
					{
						iv[i] = (byte) 0xFF;
					}
				c.setIV2(iv);
				byte[] o3 = new byte[17];
				byte[] o4 = new byte[17];
				c.encryptAES2(o3);
				c.encryptAES2(o4);
				int l = 3;
				l = l + 1;
			}

	}
