/*
 Copyright (c) 2000 - 2004, The JAP-Team
 All rights reserved.
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.

 - Neither the name of the University of Technology Dresden, Germany nor the names of its contributors
  may be used to endorse or promote products derived from this software without specific
  prior written permission.


 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */

package anon.crypto;

import java.security.SecureRandom;
import java.util.Random;

public final class MyRandom
{
	Random m_TheRandom;
	public MyRandom(Random rand)
	{
		m_TheRandom = rand;
	}

	public MyRandom()
	{
		this(new SecureRandom());
	}

	public int nextInt(int max)
	{
		int randMax = Integer.MAX_VALUE;
		randMax -= randMax % max;

		for (; ; )
		{
			int v = m_TheRandom.nextInt();
			v&=0x7FFFFFFF;
			if (v < randMax)
			{
				return v % max;
			}
		}
	}
	
	public Random getRandSource()
		{
			return m_TheRandom;
		}

	public void setSeed(long randomSeed)
		{
			m_TheRandom.setSeed(randomSeed);		
		}

	/**
	 * Reseeds the random number generator. If bReplaceSeed is true, the given seed complete replaces (rather than supplements) the seed. This means
	 * that the random number generator will generate the same sequence of numbers as another instance with the same seed.
	 * @param randomSeed the new seed, which either supplements or replaces the current seed
	 * @param bReplaceSeed if true the given seed will completely replace the current seed, otherwise it will just supplement it
	 */
	public void setSeed(long randomSeed,boolean bReplaceSeed)
		{
			if(bReplaceSeed)
				{
					if(m_TheRandom instanceof SecureRandom)
						{
							m_TheRandom=new Random(randomSeed);
						}
					else
						{
							m_TheRandom.setSeed(randomSeed);
						}
				}
			else
				{
					if(m_TheRandom instanceof SecureRandom)
						{
							m_TheRandom.setSeed(randomSeed);
						}
					else
						{
							m_TheRandom=new SecureRandom();
							m_TheRandom.setSeed(randomSeed);
						}
				}
		}

	public long nextLong()
		{
			return m_TheRandom.nextLong();
		}
}
