/*
 Copyright (c) 2000 - 2005, The JAP-Team
 All rights reserved.
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
 this list of conditions and the following disclaimer.

  - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation and/or
 other materials provided with the distribution.

  - Neither the name of the University of Technology Dresden, Germany nor the names of its contributors
 may be used to endorse or promote products derived from this software without specific
 prior written permission.


 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
package anon.crypto;

import java.util.Vector;

import mybouncycastle.org.bouncycastle.asn1.ASN1Sequence;
import anon.util.Util;

/**
 * This extension is instantiated if the type is unknown. Missing extensions should be implemented
 * when needed.
 * @author Rolf Wendolsky
 */
public final class X509UnknownExtension extends AbstractX509Extension
{
	/** The identifier is unknown and contains null. */
	public static final String IDENTIFIER = null;

	/**
	 * Creates an extension from a BouncyCastle DER sequence. For internal use only.
	 * @param a_extension a DERSequence
	 */
	X509UnknownExtension(ASN1Sequence a_extension)
	{
		super(a_extension);
	}

	/**
	 * Returns "UnknownExtension".
	 * @return "UnknownExtension"
	 */
	public String getName()
	{
		return "UnknownExtension";
	}

	/**
	 * Returns the value of this extension a a single String object in a Vector. The value does
	 * not need to be human-readable.
	 * @return a Vector containing the value of this extension as single String object
	 */
	public Vector getValues()
	{
		/* Better return the identifier, so that one can find out which extension this is,
		 * because if he does not know the extension he has no use of the containing data.
		 * In addition the identifier is properly displayed in the CertDetailsDialog
		 */
		return Util.toVector(getIdentifier()); 
		//return Util.toVector(getDEROctets());
	}
}
