/*
 Copyright (c) 2004, The JAP-Team
 All rights reserved.
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.

 - Neither the name of the University of Technology Dresden, Germany nor the names of its contributors
   may be used to endorse or promote products derived from this software without specific
  prior written permission.


 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
/*
 * based on tinySSL
 * (http://www.ritlabs.com/en/products/tinyweb/tinyssl.php)
 *
 */
package anon.crypto.tinytls;

final public class TLSHandshakeRecord extends AbstractTLSRecord
{
	public final static int HEADER_LENGTH = 4;

	/* https://tools.ietf.org/html/rfc5246#section-7.4 */
	final static int TYPE_CLIENT_HELLO = 1;
	final static int TYPE_SERVER_HELLO = 2;
	final static int TYPE_CERTIFICATE = 11;
	final static int TYPE_SERVER_KEY_EXCHANGE = 12;
	final static int TYPE_CERTIFICATE_REQUEST = 13;
	final static int TYPE_SERVER_HELLO_DONE = 14;
	final static int TYPE_CERTIFICATE_VERIFY = 15;
	final static int TYPE_CLIENT_KEY_EXCHANGE = 16;
	final static int TYPE_FINISHED = 20;

	public TLSHandshakeRecord(byte[] data, int off)
	{
		m_Header = new byte[HEADER_LENGTH];
		System.arraycopy(data, off, m_Header, 0, 4);
		m_Type = m_Header[0];
		m_dataLen = ( (m_Header[1] & 0xFF) << 16) |
			( (m_Header[2] & 0xFF) << 8) | (m_Header[3] & 0xFF);
		m_Data = new byte[m_dataLen];
		System.arraycopy(data, off + 4, m_Data, 0, m_dataLen);
	}

	public int getHeaderLength()
	{
		return HEADER_LENGTH;
	}

}
