package anon.infoservice;

import java.util.Vector;

import anon.crypto.MyRandom;

public class RandomListenerInterfaceSwitcher 
{
	private Vector m_vecListenerInterfaces;
	private Vector m_vecTried = new Vector();
	private Vector m_vecRemaining;
	private Vector m_vecTimeOutPorts;
	private int m_iPreferredPort;
	private int m_iMaxTried;
	private MyRandom m_random;
	
	public RandomListenerInterfaceSwitcher(Vector a_vecListenerInterfaces, 
			int a_iPreferredPort, int a_iMaxTried, Vector a_vecTimeOutPorts)
	{
		if (a_vecListenerInterfaces != null)
		{
			m_vecListenerInterfaces = (Vector)a_vecListenerInterfaces.clone();
		}
		else
		{
			m_vecListenerInterfaces = new Vector();
		}
		
		if (a_vecTimeOutPorts != null)
		{
			m_vecTimeOutPorts = a_vecTimeOutPorts;
		}
		else
		{
			m_vecTimeOutPorts = new Vector();
		}
		
		m_iPreferredPort = Math.min(a_iPreferredPort, 65535);
		m_iMaxTried = Math.max(1, a_iMaxTried);
		m_random = new MyRandom();
	}
	
	private boolean containsPort(Vector a_vecToCheck, int a_iPort)
	{
		for (int i = 0; i < a_vecToCheck.size(); i++)
		{
			if (a_vecToCheck.elementAt(i) instanceof Integer)
			{
				if (((Integer)a_vecToCheck.elementAt(i)).intValue() == a_iPort)
				{
					return true;
				}
			}
			else
			{
				if (((ListenerInterface)a_vecToCheck.elementAt(i)).getPort() == a_iPort)
				{
					return true;
				}
			}
		}
		return false;
	}
	
	private ListenerInterface getNextRandomInterface()
	{
		if (m_vecRemaining.size() == 0)
		{
			return null;
		}
		
		int iNext = m_random.nextInt(m_vecRemaining.size());
		ListenerInterface currentInterface = (ListenerInterface)m_vecRemaining.elementAt(iNext);
		m_vecRemaining.removeElementAt(iNext);
		return currentInterface;
	}
	
	private void resetRandomInterfaces()
	{
		if (m_vecListenerInterfaces == null)
		{
			return;
		}
		m_vecRemaining = (Vector)m_vecListenerInterfaces.clone();
		for (int i = 0; i < m_vecTried.size(); i++)
		{
			for (int j = 0; j < m_vecRemaining.size(); j++)
			{
				if (((ListenerInterface)m_vecRemaining.elementAt(j)).getPort() ==
					((Integer)m_vecTried.elementAt(i)).intValue())
				{
					m_vecRemaining.removeElementAt(j);
					break; // TODO what if we have more interfaces with the same port...? 
				}
			}
		}
	}
	
	/*
	 * Try getMaxTried() listener interfaces at maximum. 
	 * a) use a random interface with the preferred port.
	 * b) use a random interface with port 443 (if this is not the preferred port)
	 * c) use a random interface with port 80 (if this is not the preferred port)
	 * d) if not already getMaxTried() interfaces were tried: 
	 * use an interface with a random port
	 * @return the next interface or null if no more interfaces are allowed
	 */
	public synchronized ListenerInterface getNextInterface()
	{
		ListenerInterface currentInterface;
		boolean bUse = false;
		
		resetRandomInterfaces();
		while (true)
		{
			if (m_vecTried.size() >= m_iMaxTried)
			{
				return null;
			}
			
			currentInterface = getNextRandomInterface();
			if (currentInterface == null)
			{
				m_iMaxTried++;
				
				if (m_iPreferredPort > 0 && 
					!m_vecTried.contains(new Integer(m_iPreferredPort)))
				{
					// there is no interface with the preferred port...
					m_vecTried.addElement(new Integer(m_iPreferredPort));
					resetRandomInterfaces();
					continue;
				}
				else if (!m_vecTried.contains(new Integer(443)))
				{
					m_vecTried.addElement(new Integer(443));
					resetRandomInterfaces();
					continue;
				}
				else if (!m_vecTried.contains(new Integer(80)))
				{
					m_vecTried.addElement(new Integer(80));
					resetRandomInterfaces();
					continue;
				}
				
				return null;
			}
	
			
			if (!containsPort(m_vecTimeOutPorts, m_iPreferredPort) &&
				m_iPreferredPort > 0 && containsPort(m_vecListenerInterfaces, m_iPreferredPort) &&
				!m_vecTried.contains(new Integer(m_iPreferredPort)))
			{
				if (currentInterface.getPort() != m_iPreferredPort)
				{
					continue;
				}
				bUse = true;
			}
			

			if (!bUse &&
				!containsPort(m_vecTimeOutPorts, 443) &&
				containsPort(m_vecListenerInterfaces, 443) && 
				!m_vecTried.contains(new Integer(443)))
			{
				if (currentInterface.getPort() != 443)
				{
					continue;
				}
				bUse = true;
			}
			
			if (!bUse && containsPort(m_vecListenerInterfaces, 80) && 
				!containsPort(m_vecTimeOutPorts, 80) &&
				!m_vecTried.contains(new Integer(80)))
			{
				if (currentInterface.getPort() != 80)
				{
					continue;
				}
				bUse = true;
			}
			
			m_vecTried.addElement(new Integer(currentInterface.getPort()));
			
			if (!currentInterface.isValid())
			{
				m_iMaxTried++;
				continue;
			}
			
			if (m_vecTimeOutPorts.size() >=2)
			{
				m_vecTimeOutPorts.removeAllElements();
			}
			
			return currentInterface;
		}
	}
}
