package anon.infoservice.externaldatabase.jdbc;

import logging.LogHolder;
/*
Copyright (c) 2000 - 2011, The JAP-Team
All rights reserved.
Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

 - Neither the name of the University of Technology Dresden, Germany nor the names of its contributors
may be used to endorse or promote products derived from this software without specific
prior written permission.


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
*/
import logging.LogLevel;
import logging.LogType;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import anon.infoservice.externaldatabase.IEDBDatabase;
import anon.infoservice.externaldatabase.EDBException;

public class EDBDatabaseJDBC implements IEDBDatabase
	{
		private EDBConfigurationJDBC m_dbConf;
		private Object SYNC_EXTERNAL_DATABASE;

		public EDBDatabaseJDBC(EDBConfigurationJDBC dbConf)
			{
				m_dbConf = dbConf;
				SYNC_EXTERNAL_DATABASE = new Object();
			}

		public void doVacuum() throws EDBException
			{
				synchronized (SYNC_EXTERNAL_DATABASE)
					{
						if (m_dbConf==null||m_dbConf.getDatabaseURL() == null)
							{
								return;
							}

						Connection conn = null;
						try
							{
								conn = DriverManager.getConnection(m_dbConf.getDatabaseURL());
								conn.setAutoCommit(true);
								Statement stat = conn.createStatement();
								stat.executeUpdate("VACUUM;");
							}
						catch (Exception a_e)
							{
								throw new EDBException("Exception in doVacuum(): " + a_e.getMessage());
							}
						finally
							{
								try
									{
										conn.close();
									}
								catch (SQLException a_e)
									{
										LogHolder.log(LogLevel.EXCEPTION, LogType.DB, "Could not close database!", a_e);
									}
							}
					}
			}

		public void testDB() throws Exception
			{
				Class.forName(m_dbConf.getDriverClassName());
			}

		public void insert(String type, String id, String content) throws EDBException
			{
				synchronized (SYNC_EXTERNAL_DATABASE)
					{

						Connection conn = null;

						try
							{
								conn = DriverManager.getConnection(m_dbConf.getDatabaseURL());
								conn.setAutoCommit(false);
								Statement stat = conn.createStatement();
								String tableName = type;

								stat.executeUpdate("create table if not exists " + tableName + " (id text PRIMARY KEY, xml text);");
								stat.execute("DELETE FROM " + tableName + " where id = '" + id + "';");
								// REPLACE INTO is no standard SQL
								stat.executeUpdate("INSERT INTO " + tableName + " values ('" + id + "', " + "'" + content + "');");

								conn.commit();
								conn.setAutoCommit(true);

							}
						catch (SQLException a_e)
							{
								throw new EDBException("EXception in insert(): " + a_e.getMessage());
							}
						finally
							{
								try
									{
										if(conn!=null)
										conn.close();
									}
								catch (SQLException a_e)
									{
										LogHolder.log(LogLevel.EXCEPTION, LogType.DB, "Could not close database!", a_e);
									}
							}
					}
			}

		public Vector getAllTypes() throws EDBException
			{
				synchronized (SYNC_EXTERNAL_DATABASE)
				{

					Connection conn = null;
					Vector vec=new Vector();
					try
						{
							conn = DriverManager.getConnection(m_dbConf.getDatabaseURL());
							conn.setAutoCommit(false);
							Statement stat = conn.createStatement();
							ResultSet rs = stat.executeQuery("select name from sqlite_master where type = 'table';");
							while (rs.next()) 
							{
								vec.addElement(rs.getString("name"));
							}
							rs.close();
						}
					catch (SQLException a_e)
						{
							throw new EDBException("Exception in getAllTypes(): " + a_e.getMessage());
						}
					finally
						{
							try
								{
									if(conn!=null)
									conn.close();
								}
							catch (SQLException a_e)
								{
									LogHolder.log(LogLevel.EXCEPTION, LogType.DB, "Could not close database!", a_e);
								}
						}
					return vec;
				}				
			}

		public Vector getAllValuesOfType(String type) throws EDBException
			{
				synchronized (SYNC_EXTERNAL_DATABASE)
				{

					Connection conn = null;
					Vector vec=new Vector();
					try
						{
							conn = DriverManager.getConnection(m_dbConf.getDatabaseURL());
							conn.setAutoCommit(false);
							Statement stat = conn.createStatement();
							ResultSet rs = stat.executeQuery("select xml from " + type + ";");
							while (rs.next()) 
							{
								vec.addElement(rs.getString("xml"));
							}
							rs.close();
						}
					catch (SQLException a_e)
						{
							throw new EDBException("Exception in getAllTypes(): " + a_e.getMessage());
						}
					finally
						{
							try
								{
									if(conn!=null)
									conn.close();
								}
							catch (SQLException a_e)
								{
									LogHolder.log(LogLevel.EXCEPTION, LogType.DB, "Could not close database!", a_e);
								}
						}
					return vec;
				}				
			}

		public void removeType(String type) throws EDBException
			{
				synchronized (SYNC_EXTERNAL_DATABASE)
				{

					Connection conn = null;

					try
						{
							conn = DriverManager.getConnection(m_dbConf.getDatabaseURL());
							conn.setAutoCommit(false);
							Statement stat = conn.createStatement();
							stat.executeUpdate("drop table " + type + ";");
							conn.commit();
							conn.setAutoCommit(true);

						}
					catch (SQLException a_e)
						{
							throw new EDBException("Exception in removeType(): " + a_e.getMessage());
						}
					finally
						{
							try
								{
									if(conn!=null)
									conn.close();
								}
							catch (SQLException a_e)
								{
									LogHolder.log(LogLevel.EXCEPTION, LogType.DB, "Could not close database!", a_e);
								}
						}
				}				
			}

		public void remove(String type, String key) throws EDBException
			{
				synchronized (SYNC_EXTERNAL_DATABASE)
				{

					Connection conn = null;

					try
						{
							conn = DriverManager.getConnection(m_dbConf.getDatabaseURL());
							conn.setAutoCommit(false);
							Statement stat = conn.createStatement();
							stat.executeUpdate("delete from " + type + " where id = ('" + key + "');");
							conn.commit();
							conn.setAutoCommit(true);

						}
					catch (SQLException a_e)
						{
							throw new EDBException("EXception in insert(): " + a_e.getMessage());
						}
					finally
						{
							try
								{
									if(conn!=null)
									conn.close();
								}
							catch (SQLException a_e)
								{
									LogHolder.log(LogLevel.EXCEPTION, LogType.DB, "Could not close database!", a_e);
								}
						}
				}				
				
			}
	}
