/*
 Copyright (c) 2006, The JAP-Team
 All rights reserved.
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
 this list of conditions and the following disclaimer.

  - Redistributions in bisnary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation and/or
 other materials provided with the distribution.

  - Neither the name of the University of Technology Dresden, Germany nor the names of its contributors
 may be used to endorse or promote products derived from this software without specific
 prior written permission.


 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
package anon.infoservice.update;

import java.util.Hashtable;

import anon.infoservice.InfoServiceHolder;
import anon.infoservice.MixCascadeExitAddresses;


/**
 * Updates the known InfoServices. This may be done automatically (by a background thread) and manually
 * by method call. The automatic update is only done if this is allowed by the model.
 * @author Rolf Wendolsky
 */
public class ServiceExitAddressUpdater extends AbstractDatabaseUpdater
{
	private static final long UPDATE_INTERVAL_MS = 20 * 60000l; // every 20 minutes
	private static final long MIN_UPDATE_INTERVAL_MS = 60000l;

	public ServiceExitAddressUpdater(ObservableInfo a_observableInfo)
	{
		super(new DynamicUpdateInterval(UPDATE_INTERVAL_MS), a_observableInfo);
	}

	public Class getUpdatedClass()
	{
		return MixCascadeExitAddresses.class;
	}
	protected boolean doCleanup(Hashtable a_newEntries)
	{
		// do nothing
		return false;
	}


	protected Hashtable getUpdatedEntries(Hashtable a_entriesToUpdate)
	{
		Hashtable hashIS = InfoServiceHolder.getInstance().updateExitAddresses();
		
		if (getUpdateInterval() instanceof DynamicUpdateInterval)
		{
			if (hashIS == null || hashIS.size() <= 1)
			{
				// no entries where found, or too few infoservices are available
				((DynamicUpdateInterval)getUpdateInterval()).setUpdateInterval(MIN_UPDATE_INTERVAL_MS);
			}
			else
			{
				((DynamicUpdateInterval)getUpdateInterval()).setUpdateInterval(UPDATE_INTERVAL_MS);
			}
		}
		
		return hashIS;
	}

	protected Hashtable getEntrySerials() 
	{
		return new Hashtable();
	}
}
