package anon.pay.xml;

import java.net.MalformedURLException;
import java.net.URL;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.GregorianCalendar;

import logging.LogHolder;
import logging.LogLevel;
import logging.LogType;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;


import anon.crypto.IMyPrivateKey;
import anon.crypto.SignatureVerifier;
import anon.crypto.XMLSignature;
import anon.pay.PayMessage;
import anon.pay.Transaction;
import anon.util.Base64;
import anon.util.IXMLEncodable;
import anon.util.XMLParseException;
import anon.util.XMLUtil;

/**
 * This class holds a balance certificate. Can be converted to and from
 * XML
 *
 * @todo find a better internal representation for the signature
 */
public class XMLBalance implements IXMLEncodable
{
	private static final String DEFAULT_RATE_ENDDATE = "3000-01-01 00:00:00.00000000";
	private static final String DEFAULT_RATE_STARTDATE = "1970-01-01 00:00:00.00000000";
	
	/** the number of the account that this balance object belongs to*/ 
	private long m_lAccountNumber;
	
	private java.sql.Timestamp m_Timestamp;
	
	private Timestamp m_tStartDate;
	
	/** bytes that are not consumed if there are any will expire after that date */
	private java.sql.Timestamp m_ValidTime;
	
	/** the amount of money which the user has paid for the corresponding volume plan */
	private long m_lDeposit;
	
	private boolean m_bBlocked = false; 
	
	private Transaction m_transaction;
	
	/** 
	 * the amount of byte which are already consumed for the 
	 * corresponding account 
	 */
	private long m_lSpent;
	
	// is this a capped flat rate?
	private long m_lVolumeBytesMonthly;
	
	private java.sql.Timestamp m_flatEnddate;
	
	/**  the kbytes that are still left to spend for the corresponding account */
	private long m_volumeKBytesleft;
	
	private long m_volumeBytesleft;
	
	private long m_lOverusageBytes = 0;
	
	private Timestamp m_tOverusageDate = null;
	
	private double m_dFactorOverusageGeneral = 0.0;
	private double m_dFactorOverusageUser = 0.0;
	
	/** */
	//private int m_balance;
	
	private String m_message;
	
	private String m_messageText;
	
	private URL m_messageLink;
	
	private Timestamp m_tMonthlyBytesUpdatedOn = new Timestamp(0);
	
	private long m_lLastMonthRemainingTraffic;

	private Document m_docTheBalance = null;
	
	private String m_strAffiliate;

	public XMLBalance(long accountNumber,
					  long deposit, long spent,
					  Timestamp timestamp,
					  Timestamp validTime,
					  //int balance,
					  long volumeBytesleft,
					  Timestamp flatEnddate,
					  IMyPrivateKey signKey,
					  long a_lVolumeBytesMonthly,
					  Timestamp a_tMonthlyBytesUpdatedOn,
					  Timestamp a_tStartDate,
					  long a_lOverusageBytes,
					  Timestamp a_tOverusageDate,
					  double a_dFactorOverusageGeneral,
					  double a_dFactorOverusageUser,
					  long a_lLastMonthRemainingTraffic,
					  boolean a_bBlocked,
					  String a_strAffiliate,
					  Transaction a_transaction)
	{
		m_transaction = a_transaction;
		m_bBlocked = a_bBlocked;
		m_strAffiliate = a_strAffiliate;
		m_lLastMonthRemainingTraffic = a_lLastMonthRemainingTraffic;
		m_dFactorOverusageGeneral = a_dFactorOverusageGeneral;
		m_dFactorOverusageUser = a_dFactorOverusageUser;
		m_lOverusageBytes = a_lOverusageBytes;
		m_tOverusageDate = a_tOverusageDate;
		m_lDeposit = deposit;
		m_lSpent = spent;
		m_Timestamp = timestamp;
		m_tMonthlyBytesUpdatedOn = a_tMonthlyBytesUpdatedOn;
		m_lVolumeBytesMonthly = a_lVolumeBytesMonthly;
		if (m_Timestamp == null)
		{
			m_Timestamp = new Timestamp(System.currentTimeMillis());
		}
		m_ValidTime = validTime;
		if (m_ValidTime == null)
		{
			m_ValidTime = Timestamp.valueOf(DEFAULT_RATE_ENDDATE);
		}
		m_lAccountNumber = accountNumber;
		//m_balance = balance;
		m_volumeKBytesleft = volumeBytesleft / 1000;
		m_volumeBytesleft = volumeBytesleft;
		m_flatEnddate = flatEnddate;
		if (m_flatEnddate == null)
		{
			m_flatEnddate = Timestamp.valueOf(DEFAULT_RATE_ENDDATE);
		}
		m_tStartDate = a_tStartDate;
		if (m_tStartDate == null)
		{
			m_tStartDate = Timestamp.valueOf(DEFAULT_RATE_STARTDATE);
		}
		
		m_docTheBalance = XMLUtil.createDocument();
		m_docTheBalance.appendChild(internal_toXmlElement(m_docTheBalance));
		if (signKey != null) //might very well be null, when created by Database (which doesnt have access to the private key)
		{
			sign(signKey);
		}
	}

	public void sign(IMyPrivateKey signKey) //is public so we can create it first, and call sign later
	{
		try
		{
			XMLSignature.sign(m_docTheBalance, signKey, SignatureVerifier.DOCUMENT_CLASS_NONE);
		} catch (XMLParseException e)
		{
			LogHolder.log(LogLevel.DEBUG, LogType.PAY, "Could not sign XMLBalance");
		}
	}

	public void setMessage(PayMessage a_message)
	{
		if (a_message == null)
		{
			m_message = null;
			m_messageLink = null;
			m_messageText = null;
		}
		else
		{
			m_message = a_message.getShortMessage();
			m_messageLink = a_message.getMessageLink();
			m_messageText = a_message.getMessageText();
		}
		//need to recreate the internal do to include the message
		m_docTheBalance = XMLUtil.createDocument();
		m_docTheBalance.appendChild(internal_toXmlElement(m_docTheBalance));
	}



	public XMLBalance(Document doc) throws Exception
	{
		setValues(doc.getDocumentElement());
		m_docTheBalance = doc;
	}

	public XMLBalance(String xmlDoc) throws Exception
	{
		Document doc = XMLUtil.toXMLDocument(xmlDoc);
		setValues(doc.getDocumentElement());
		m_docTheBalance = doc;
	}

	public XMLBalance(Element elemBalance) throws Exception
	{
		setValues(elemBalance);
		m_docTheBalance = XMLUtil.createDocument();
		m_docTheBalance.appendChild(XMLUtil.importNode(m_docTheBalance, elemBalance, true));
	}

	private void setValues(Element elemRoot) throws Exception
	{
		if (!elemRoot.getTagName().equals("Balance") ||
			!elemRoot.getAttribute("version").equals("1.0"))
		{
			throw new Exception("Balance wrong XML format");
		}
		
		Element elem = (Element) XMLUtil.getFirstChildByName(elemRoot, "AccountNumber");
		String str = XMLUtil.parseValue(elem, (String)null);
		m_lAccountNumber = Long.parseLong(str);
		
		
		Node node = XMLUtil.getFirstChildByName(elemRoot, Transaction.XML_ELEMENT_NAME);
		if (node instanceof Element && node != null)
		{
			m_transaction = new Transaction((Element)node);
		}

		elem = (Element) XMLUtil.getFirstChildByName(elemRoot, "Blocked");
		m_bBlocked = XMLUtil.parseValue(elem, false);
		
		m_strAffiliate = XMLUtil.parseValue(
				XMLUtil.getFirstChildByName(elemRoot, "Affiliate"), null);
		if (m_strAffiliate != null && m_strAffiliate.endsWith("_null")) // fix
		{
			m_strAffiliate = null;
		}
		
		elem = (Element) XMLUtil.getFirstChildByName(elemRoot, "Deposit");
		str = XMLUtil.parseValue(elem, (String)null);
		m_lDeposit = Long.parseLong(str);

		elem = (Element) XMLUtil.getFirstChildByName(elemRoot, "Spent");
		str = XMLUtil.parseValue(elem, (String)null);
		m_lSpent = Long.parseLong(str);

		elem = (Element) XMLUtil.getFirstChildByName(elemRoot, "BalanceInCent");
		str = XMLUtil.parseValue(elem, "0");
		//m_balance = Math.max(0, Integer.parseInt(str));

		elem = (Element) XMLUtil.getFirstChildByName(elemRoot, "FlatrateEnddate");
		str = XMLUtil.parseValue(elem, DEFAULT_RATE_ENDDATE);
		m_flatEnddate = java.sql.Timestamp.valueOf(str);
		
		
		elem =  (Element)XMLUtil.getFirstChildByName(elemRoot, "OverusageBytes");
		if (XMLUtil.parseAttribute(elem, "overusageDate", null) != null)
		{
			m_tOverusageDate = Timestamp.valueOf(XMLUtil.parseAttribute(elem, "overusageDate", null));
		}
		
		
		m_lLastMonthRemainingTraffic = XMLUtil.parseAttribute(elem, "lastMonthRemainingTraffic", 0l);
		m_dFactorOverusageGeneral = XMLUtil.parseAttribute(elem, "overusageFactorGeneral", 0.0d);
		m_dFactorOverusageUser = XMLUtil.parseAttribute(elem, "overusageFactorUser", 0.0d);
		m_lOverusageBytes = XMLUtil.parseValue(elem, 0l);
		
		
		elem = (Element) XMLUtil.getFirstChildByName(elemRoot, "StartDate");
		str = XMLUtil.parseValue(elem, DEFAULT_RATE_STARTDATE);
		m_tStartDate = java.sql.Timestamp.valueOf(str);
		

		elem = (Element) XMLUtil.getFirstChildByName(elemRoot, "VolumeBytesLeft");
		m_volumeKBytesleft = XMLUtil.parseValue(elem, 0l);
		
		elem = (Element) XMLUtil.getFirstChildByName(elemRoot, "BytesLeft");
		m_volumeBytesleft = XMLUtil.parseValue(elem, 0l);
		if (m_volumeBytesleft == 0)
		{
			m_volumeBytesleft = m_volumeKBytesleft * 1000;
		}
		
		elem = (Element) XMLUtil.getFirstChildByName(elemRoot, "VolumeBytesMonthly");
		m_lVolumeBytesMonthly = XMLUtil.parseValue(elem, 0l);
		

		elem = (Element) XMLUtil.getFirstChildByName(elemRoot, "Timestamp");
		str = XMLUtil.parseValue(elem, (String)null);
		if (str != null)
		{
			m_Timestamp = java.sql.Timestamp.valueOf(str);
		}
		else
		{
			m_Timestamp = new Timestamp(System.currentTimeMillis());
		}
		elem = (Element) XMLUtil.getFirstChildByName(elemRoot, "Validtime");
		str = XMLUtil.parseValue(elem, DEFAULT_RATE_ENDDATE);
		m_ValidTime = java.sql.Timestamp.valueOf(str);

	    elem = (Element) XMLUtil.getFirstChildByName(elemRoot, "Message");
		if (elem == null)
		{
			; // no message exists for this account, that's OK
		}
		else
		{
			boolean isBase64 = XMLUtil.parseAttribute(elem, "encoded",false);
			if (isBase64)
			{
				try
				{
					str = XMLUtil.parseValue(elem, "");
					if (!str.equals("") )
					{
						m_message = Base64.decodeToString(str);
					}
					else
					{
						m_message = "";
					}
				}
				catch (Exception e)
				{
					LogHolder.log(LogLevel.DEBUG, LogType.PAY,
								  "Error while reading message: " + e + ", message (Base64) was" + str +
								  "decoded message was" + m_message);
				}
			}
			else
			{
				m_message = XMLUtil.parseValue(elem,"");
			}
		}
		elem = (Element) XMLUtil.getFirstChildByName(elemRoot, "MessageLink");
		if (elem == null)
		{
			; // no message link exists for this account, that's OK
		}
		else
		{
			str = XMLUtil.parseValue(elem,"");
			if (!str.equals(""))
			{
				try
				{
					m_messageLink = new URL(str);
				} catch (MalformedURLException mue)
				{
					LogHolder.log(LogLevel.DEBUG, LogType.PAY, "Could not get URL from messagelink string: " + str + ", reason: " + mue);
				}
			}
		}
		elem = (Element) XMLUtil.getFirstChildByName(elemRoot, "MessageText");
		if (elem == null)
		{
			; // no message text exists for this account, that's OK
		}
		else
		{
			boolean isBase64 = XMLUtil.parseAttribute(elem, "encoded",false);
			if (isBase64)
			{
				try
				{
					str = XMLUtil.parseValue(elem, "");
					if (!str.equals(""))
					{
						m_messageText = Base64.decodeToString(str);
					} else
					{
						m_messageText = "";
					}
				}
				catch (Exception e)
				{
					LogHolder.log(LogLevel.DEBUG, LogType.PAY,
								  "Error while reading message: " + e + ", message (Base64) was" + str +
								  "decoded message was" + m_message);
				}
			}
			else
			{
				m_messageText = XMLUtil.parseValue(elem,"");
			}
		}

	}

	private Element internal_toXmlElement(Document a_doc)
	{
		Element elemRoot = a_doc.createElement("Balance");
		elemRoot.setAttribute("version", "1.0");

		Element elem = a_doc.createElement("AccountNumber");
		XMLUtil.setValue(elem, m_lAccountNumber);
		elemRoot.appendChild(elem);
		
		if (m_transaction != null)
		{
			elemRoot.appendChild(m_transaction.toXmlElement(a_doc));
		}
		
		elem = a_doc.createElement("Blocked");
		XMLUtil.setValue(elem, m_bBlocked);
		elemRoot.appendChild(elem);
		
		if (m_strAffiliate != null)
		{
			elem = a_doc.createElement("Affiliate");
			XMLUtil.setValue(elem, m_strAffiliate);
			elemRoot.appendChild(elem);
		}
		
		
		elem = a_doc.createElement("Deposit");
		XMLUtil.setValue(elem, m_lDeposit);
		elemRoot.appendChild(elem);
		elem = a_doc.createElement("Spent");
		XMLUtil.setValue(elem, m_lSpent);
		elemRoot.appendChild(elem);
		//elem = a_doc.createElement("BalanceInCent");
		//XMLUtil.setValue(elem,m_balance);
		//elemRoot.appendChild(elem);
		elem = a_doc.createElement("FlatrateEnddate");
		XMLUtil.setValue(elem, m_flatEnddate.toString() );
		elemRoot.appendChild(elem);
		
		
		elem = a_doc.createElement("OverusageBytes");
		XMLUtil.setValue(elem, m_lOverusageBytes);
		if (m_tOverusageDate != null)
		{
			XMLUtil.setAttribute(elem, "overusageDate", m_tOverusageDate.toString());
		}
		
		
		XMLUtil.setAttribute(elem, "lastMonthRemainingTraffic", m_lLastMonthRemainingTraffic);
		XMLUtil.setAttribute(elem, "overusageFactorGeneral",m_dFactorOverusageGeneral);
		XMLUtil.setAttribute(elem, "overusageFactorUser",m_dFactorOverusageUser);
		elemRoot.appendChild(elem);
		
		if (m_tStartDate != null)
		{
			elem = a_doc.createElement("StartDate");
			XMLUtil.setValue(elem, m_tStartDate.toString() );
			elemRoot.appendChild(elem);
		}
		
		elem = a_doc.createElement("VolumeBytesLeft");
		XMLUtil.setValue(elem, m_volumeKBytesleft);
		elemRoot.appendChild(elem);
		elem = a_doc.createElement("BytesLeft");
		XMLUtil.setValue(elem, m_volumeBytesleft);
		elemRoot.appendChild(elem);
		
		elem = a_doc.createElement("VolumeBytesMonthly");
		XMLUtil.setValue(elem, m_lVolumeBytesMonthly);
		elemRoot.appendChild(elem);
			
		
		elem = a_doc.createElement("Timestamp");
		XMLUtil.setValue(elem, m_Timestamp.toString());
		elemRoot.appendChild(elem);
		elem = a_doc.createElement("Validtime");
		XMLUtil.setValue(elem, m_ValidTime.toString());
		elemRoot.appendChild(elem);

		elem = a_doc.createElement("Message");
		if (m_message != null)
		{
			String encodedMessage = Base64.encodeString(m_message);
			XMLUtil.setValue(elem, encodedMessage);
			XMLUtil.setAttribute(elem, "encoded",true);
		}
		elemRoot.appendChild(elem);

		elem = a_doc.createElement("MessageText");
		if (m_messageText != null)
		{
			String encodedMessageText = Base64.encodeString(m_messageText);
			XMLUtil.setAttribute(elem, "encoded", true);
			XMLUtil.setValue(elem, encodedMessageText);
		}
		elemRoot.appendChild(elem);

		elem = a_doc.createElement("MessageLink");
		if (m_messageLink != null)
		{
			XMLUtil.setValue(elem, m_messageLink.toString());
		}
		elemRoot.appendChild(elem);



		return elemRoot;
	}
	
	/**
	 * Returns the number the number of the account to which this balance belongs to
	 * @return number of the account that this balance belongs to
	 */
	public long getAccountNumber()
	{
		return m_lAccountNumber;
	}
	
	public double getOverusageFactorGeneral()
	{
		return m_dFactorOverusageGeneral;
	}
	
	public double getOverusageFactor()
	{
		return m_dFactorOverusageUser;
	}
	
	public String getAffiliate()
	{
		return m_strAffiliate;
	}
	
	public boolean isBlocked()
	{
		return m_bBlocked;
	}
	
	/**
	 * Returns the total amount of money spent by the user to buy the 
	 * corresponding volume plan.
	 * @return costs for the volume plan of the corresponding account
	 */
	public long getDeposit()
	{
		return m_lDeposit;
	}
	/**
	 * The transaction ID is only set if a transaction has been started (regardless
	 * whether it is finished or not).
	 * @return
	 */
	public Transaction getTransaction()
	{
		return m_transaction;
	}
	
	/**
	 * Returns the overall spent bytes for the corresponding account
	 * @return bytes spent overall
	 */
	public long getSpent()
	{
		return m_lSpent;
	}

	/*public int getBalance()
	{
		return m_balance;
	}*/
	
	public long getOverusageBytes()
	{
		return m_lOverusageBytes;
	}
	
	
	public Timestamp getOverusageDate()
	{
		return m_tOverusageDate;
	}
	
	public Timestamp getMonthlyBytesUpdatedOn()
	{
		return m_tMonthlyBytesUpdatedOn;
	}

	public long getVolumeBytesMonthly()
	{
		return m_lVolumeBytesMonthly;
	}
	
	public long getLastMonthRemainingTraffic()
	{
		return m_lLastMonthRemainingTraffic;
	}
	
	/**
	 * returns the current credit of the user
	 * return value will be compared to jap.pay.PaymentMainPanel WARNING_AMOUNT
	 *
	 * @return long: currently volume_bytesleft
	 */
	public long getVolumeBytesLeft()
	{
		return m_volumeBytesleft;
	}

	public Timestamp getStartDate()
	{
		return m_tStartDate;
	}
	
	public java.sql.Timestamp getFlatEnddate()
	{
		return m_flatEnddate;
	}

	public java.sql.Timestamp getTimestamp()
	{
		return m_Timestamp;
	}
	
	/**
	 * Returns the Date after that unspent byte volume
	 * will expire 
	 * @return the expire date
	 */
	public java.sql.Timestamp getValidTime()
	{
		return m_ValidTime;
	}

	/**
	 *
	 * @return PayMessage: a PayMessage object if this Balance has a message associated with it
	 *  If the short message text is null or an empty String, getMessage() will return null
	 *   (even if a link or long text exists)
	 */
	public PayMessage getMessage()
	{
		if (m_message == null || m_message.equals("") )
		{
			return null;
		}
	    else
		{
			return new PayMessage(m_message, m_messageText, m_messageLink);
		}
	}

	public MonthlyOverusage calculateMonthlyOverusage(Timestamp a_tNow)
	{
		MonthlyOverusage overusage = null;
		
		if (canDoMonthlyOverusage(a_tNow))
		{
			overusage = new MonthlyOverusage();
			
			overusage.m_tEndOfCurrentPeriod = 
				XMLBalance.calculateEndOfCurrentMonthlyPeriod(getFlatEnddate(), a_tNow);
			overusage.m_dFactor = getOverusageFactorGeneral();
			
			overusage.m_lAdditionalTraffic = getVolumeBytesMonthly() *
				XMLBalance.calculateRemainingRateMonths(getFlatEnddate(), a_tNow);
			
			if (overusage.m_lAdditionalTraffic > 0 && getLastMonthRemainingTraffic() > 0)
			{
				overusage.m_lAdditionalTraffic = overusage.m_lAdditionalTraffic - 
					getVolumeBytesMonthly() + getLastMonthRemainingTraffic();
			}
			
			overusage.m_lAdditionalTraffic = 
				(long)(((double)overusage.m_lAdditionalTraffic) / getOverusageFactorGeneral());
		}
		else if (isCurrentlyInOverusage(a_tNow))
		{
			overusage = new MonthlyOverusage();
			overusage.m_dFactor = getOverusageFactor();
			overusage.m_lAdditionalTraffic = getOverusageBytes();
			overusage.m_tEndOfCurrentPeriod = XMLBalance.calculateEndOfCurrentMonthlyPeriod(getFlatEnddate(), a_tNow);
		}
		
		return overusage;
	}


	public Element toXmlElement(Document a_doc)
	{
		try
		{
			return (Element) XMLUtil.importNode(a_doc, m_docTheBalance.getDocumentElement(), true);
		}
		catch (Exception e)
		{
			return null;
		}
	}
	
	public boolean isCurrentlyInOverusage(Timestamp a_tNow)
	{
		return getOverusageDate() != null &&
		  XMLBalance.isSameMonthlyPeriod(getOverusageDate(), a_tNow, getStartDate(), true);
	}
	
	public boolean canDoMonthlyOverusage(Timestamp a_tNow)
	{
			if (getVolumeBytesMonthly() > 0 && getStartDate() != null && !getStartDate().after(a_tNow) &&
					 XMLBalance.calculateRemainingRateMonths(getFlatEnddate(), a_tNow) > 0 &&
							  (getOverusageDate() == null ||
							  !XMLBalance.isSameMonthlyPeriod(getOverusageDate(), a_tNow, getStartDate(), true)) &&
							  getOverusageFactorGeneral() > 1.0 && getLastMonthRemainingTraffic() > 0)
			{
				return true;
			}
		
		return false;
	}
	
	
	public static boolean isSameMonthlyPeriod(Timestamp a_tUpdatedOn, Timestamp a_tNow, Timestamp a_tStartDate, boolean a_bUpdatedOnEarlier)
	{
		Calendar calendarUpdatedOn = new GregorianCalendar();
		Calendar calendarNow = new GregorianCalendar();
		Calendar calendarStart = new GregorianCalendar();
		
		
		
		if (a_tStartDate == null)
		{
			return true;
		}
		
		if (a_tUpdatedOn == null || a_tNow == null)
		{
			return false;
		}
			/*
			if (a_tStartDate == null)
			{
				calendarUpdatedOn.setTime(new Date(0));
			}
			else
			{
				calendarUpdatedOn.setTime(a_tStartDate);
			}*/
		
		
		
		calendarUpdatedOn.setTime(a_tUpdatedOn);
		calendarNow.setTime(a_tNow);		
		
		calendarStart.setTime(a_tStartDate);
		
		return isSameMonthlyPeriod(calendarUpdatedOn, calendarNow, calendarStart, a_bUpdatedOnEarlier);
	}
	
	private static boolean isSameMonthlyPeriod(Calendar calendarUpdatedOn, Calendar calendarNow, Calendar calendarStart,
			 boolean a_bUpdatedOnEarlier)
	{
		Calendar calendarNowLastMonth = new GregorianCalendar();
		calendarNowLastMonth.setTime(calendarNow.getTime());
		calendarNowLastMonth.add(Calendar.MONTH, -1);
		
		
		if ((a_bUpdatedOnEarlier && calendarUpdatedOn.getTime().getTime() > calendarNow.getTime().getTime())) 
				//(!a_bUpdatedOnEarlier && calendarUpdatedOn.getTime().getTime() < calendarNow.getTime().getTime()))
		{
			// should not happen if the clocks of PI/client are perfectly synchronized, but may happen if they are not
			return true;
		}
		
		
		// normalize all used dates to the same month; otherwise, we cannot compute with days
		calendarStart.set(Calendar.YEAR, calendarUpdatedOn.get(Calendar.YEAR));
		calendarStart.set(Calendar.MONTH, calendarUpdatedOn.get(Calendar.MONTH));
		
		
		if (calendarUpdatedOn.get(Calendar.MONTH) == calendarNow.get(Calendar.MONTH))
		{
			if (calendarUpdatedOn.get(Calendar.YEAR) != calendarNow.get(Calendar.YEAR))
			{
				return false;
			}
			
			if ((calendarUpdatedOn.get(Calendar.DAY_OF_MONTH) >= calendarStart.get(Calendar.DAY_OF_MONTH) &&
					calendarNow.get(Calendar.DAY_OF_MONTH) >= calendarStart.get(Calendar.DAY_OF_MONTH)) ||
					(calendarUpdatedOn.get(Calendar.DAY_OF_MONTH) < calendarStart.get(Calendar.DAY_OF_MONTH) &&
							calendarNow.get(Calendar.DAY_OF_MONTH) < calendarStart.get(Calendar.DAY_OF_MONTH)))
			{
				return true;
			}
		}
		else if (calendarUpdatedOn.get(Calendar.MONTH) == calendarNowLastMonth.get(Calendar.MONTH))
		{
			if (calendarUpdatedOn.get(Calendar.YEAR) != calendarNowLastMonth.get(Calendar.YEAR))
			{
				return false;
			}
			
			if (calendarUpdatedOn.get(Calendar.DAY_OF_MONTH) >= calendarStart.get(Calendar.DAY_OF_MONTH) &&
					calendarNowLastMonth.get(Calendar.DAY_OF_MONTH) < calendarStart.get(Calendar.DAY_OF_MONTH))
			{
				return true;
			}
		}
				
		return false;
	}
	
	public static int calculateRemainingRateMonths(Calendar a_calendarEnd, Calendar a_calendarNow)
	{
		return calculateRemainingRateMonths(new Timestamp(a_calendarEnd.getTime().getTime()),
				new Timestamp(a_calendarNow.getTime().getTime()));
	}
	
	public static Timestamp calculateEndOfCurrentMonthlyPeriod(Timestamp a_tEnd, Timestamp a_tNow)
	{
		if (a_tEnd == null || a_tNow == null)
		{
			return null;
		}
		Calendar a_calendarEnd = new GregorianCalendar();
		a_calendarEnd.setTime(a_tEnd);
		Calendar a_calendarNow = new GregorianCalendar();
		a_calendarNow.setTime(a_tNow);
		
		return new Timestamp(calculateEndOfCurrentMonthlyPeriod(a_calendarEnd, a_calendarNow).getTime().getTime()); 
	}
	
	public static Calendar calculateEndOfCurrentMonthlyPeriod(Calendar a_calendarEnd, Calendar a_calendarNow)
	{
		if (a_calendarEnd == null || a_calendarNow == null)
		{
			return null;
		}
		Calendar calendarEndBillingPeriod = (Calendar)a_calendarEnd.clone();
		calendarEndBillingPeriod.set(Calendar.YEAR, a_calendarNow.get(Calendar.YEAR));
		calendarEndBillingPeriod.set(Calendar.MONTH, a_calendarNow.get(Calendar.MONTH));
		
		if (a_calendarNow.get(Calendar.DAY_OF_MONTH) > calendarEndBillingPeriod.get(Calendar.DAY_OF_MONTH))
		{
			calendarEndBillingPeriod.add(Calendar.MONTH, 1);
		}
		return calendarEndBillingPeriod;
	}
	
	public static int calculateRemainingRateMonths(Timestamp a_tEnd, Timestamp a_tNow)
	{
		if (a_tEnd == null || a_tNow == null || a_tEnd.getTime() < a_tNow.getTime())
		{
			return 0;
		}
		
		Calendar calendarEnd = new GregorianCalendar();
		Calendar calendarNow = new GregorianCalendar();
		
		calendarEnd.setTime(a_tEnd);
		calendarNow.setTime(a_tNow);
		
		int iRateMonths = 0;
		
		// normalize all used dates to the same month; otherwise, we cannot compute with days
		if (calendarEnd.get(Calendar.YEAR) != calendarNow.get(Calendar.YEAR))
		{
			iRateMonths += 12 * (calendarEnd.get(Calendar.YEAR) - calendarNow.get(Calendar.YEAR));
			calendarNow.set(Calendar.YEAR, calendarEnd.get(Calendar.YEAR));
		}
		if (calendarEnd.get(Calendar.MONTH) != calendarNow.get(Calendar.MONTH))
		{
			iRateMonths += calendarEnd.get(Calendar.MONTH) - calendarNow.get(Calendar.MONTH);
			calendarNow.set(Calendar.MONTH, calendarEnd.get(Calendar.MONTH));
		}
	
		
		if (iRateMonths > 0 && calendarNow.get(Calendar.DAY_OF_MONTH) > calendarEnd.get(Calendar.DAY_OF_MONTH))
		{
			iRateMonths -= 1;
		}
		return iRateMonths;
	}

	public static Calendar calculateEndDate(Calendar a_startDate, int a_duration, int a_calendarFieldUnit)
	{
		Calendar now = (Calendar)a_startDate.clone();
		if (a_calendarFieldUnit == Calendar.DATE || a_calendarFieldUnit == Calendar.DAY_OF_MONTH)
		{
			now.add(Calendar.DATE,a_duration);
		} 
		else if (a_calendarFieldUnit == Calendar.WEEK_OF_YEAR)
		{
			now.add(Calendar.WEEK_OF_YEAR, a_duration);
			now.add(Calendar.DATE, -1);
		} 
		else if (a_calendarFieldUnit == Calendar.MONTH)
		{
			now.add(Calendar.MONTH,a_duration);
			now.add(Calendar.DATE, -1);
		}
		else if (a_calendarFieldUnit == Calendar.YEAR)
		{
			now.add(Calendar.YEAR,a_duration);
			now.add(Calendar.DATE, -1);
		}
		else
		{
			return null;
		}
		//set to just before midnight of the calculated day (so we can show the end date in JAP without time, without complaints)
		now.set(Calendar.HOUR_OF_DAY,23); // Calendar.HOUR would be am/pm!
		now.set(Calendar.MINUTE, 59);
		now.set(Calendar.SECOND, 59);
		now.set(Calendar.MILLISECOND, 0);
		return now;
	}
	
	public boolean isLastMonthOfRate(Timestamp a_tNow)
	{
		return isLastMonthOfRate(getFlatEnddate(), a_tNow, getStartDate());
	}
	
	public static boolean isLastMonthOfRate(Timestamp a_tEnd, Timestamp a_tNow, Timestamp a_tStart)
	{
		if (a_tEnd.getTime() < a_tNow.getTime())
		{
			// expired
			return false;
		}
		
		return isSameMonthlyPeriod(a_tNow, a_tEnd, a_tStart, false);
	}
	
	public static boolean isLastMonthOfRate(Calendar a_calendarEnd, Calendar a_calendarNow, Calendar a_calendarStart)
	{
		if (a_calendarEnd.getTime().getTime() < a_calendarNow.getTime().getTime())
		{
			// expired
			return false;
		}
		
		// clone the calendars in order not to change the original objects
		Calendar calendarEnd = (Calendar)a_calendarEnd.clone();
		Calendar calendarNow = (Calendar)a_calendarNow.clone();
		Calendar calendarStart = (Calendar)a_calendarStart.clone();
		
		return isSameMonthlyPeriod(calendarNow, calendarEnd, calendarStart, false);
	}
	
	public class MonthlyOverusage
	{
		public double m_dFactor;
		public Timestamp m_tEndOfCurrentPeriod;
		public long m_lAdditionalTraffic;
	}
}
