/*
 Copyright (c) 2000 - 2006, The JAP-Team
 All rights reserved.
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
 this list of conditions and the following disclaimer.

  - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation and/or
 other materials provided with the distribution.

  - Neither the name of the University of Technology Dresden, Germany nor the names of its contributors
 may be used to endorse or promote products derived from this software without specific
 prior written permission.


 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */

package anon.platform;

import java.io.InputStream;
import java.lang.reflect.Method;

/***
 * Implements some functions of java.lang.Process using reflection to deal with
 * the case java.lang.Process or the needed methods are not supported on the
 * given platform (most notable: iOS)*
 * 
 * @author sk13
 *
 */
class MyProcess
	{

		private Object m_Process = null;
		private static Class m_javalangProcessClass;

		static
			{
				try
					{
						m_javalangProcessClass = Class.forName("java.lang.Process");
					}
				catch (Exception e)
					{
						m_javalangProcessClass = null;
					}
			}

		public MyProcess(Object process)
			{
				m_Process = process;
			}

		public int exitValue()
			{
				if (m_javalangProcessClass != null)
					{
						try
							{
								Method m = m_javalangProcessClass.getMethod("exitValue", (Class[]) null);
								Object o = m.invoke(m_Process, (Object[]) null);
								return ((Integer) o).intValue();
							}
						catch (Exception e)
							{
							}
					}
				return -1;
			}

		public InputStream getInputStream()
			{
				if (m_javalangProcessClass != null)
					{
						try
							{
								Method m = m_javalangProcessClass.getMethod("getInputStream", (Class[]) null);
								Object o = m.invoke(m_Process, (Object[]) null);
								return (InputStream) o;
							}
						catch (Exception e)
							{
							}
					}
				return null;
			}

		public InputStream getErrorStream()
			{
				if (m_javalangProcessClass != null)
					{
						try
							{
								Method m = m_javalangProcessClass.getMethod("getErrorStream", (Class[]) null);
								Object o = m.invoke(m_Process, (Object[]) null);
								return ((InputStream) o);
							}
						catch (Exception e)
							{
							}
					}
				return null;
			}

		public void destroy()
			{
				if (m_javalangProcessClass != null)
					{
						try
							{
								Method m = m_javalangProcessClass.getMethod("destroy", (Class[]) null);
								Object o = m.invoke(m_Process, (Object[]) null);
							}
						catch (Exception e)
							{
							}
					}
			}

	}
