/*
 Copyright (c) 2000 - 2006, The JAP-Team
 All rights reserved.
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
 this list of conditions and the following disclaimer.

  - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation and/or
 other materials provided with the distribution.

  - Neither the name of the University of Technology Dresden, Germany nor the names of its contributors
 may be used to endorse or promote products derived from this software without specific
 prior written permission.


 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */

package anon.platform;

import java.io.IOException;
import java.lang.reflect.Method;

/***
* Implements some functions what deal with processes of java.lang.Runtime using reflection to deal with
* the case java.lang.Runtime or the needed methods are not supported on the
* given platform (most notable: iOS)*
* 
* @author sk13
*
*/



public class MyRuntime
	{

		private final static MyRuntime m_Runtime = new MyRuntime();
		private static Object m_javalangRuntime = null;
		private static Class m_javalangRuntimeClass = null;

		private MyRuntime()
			{
				try
					{
						m_javalangRuntimeClass = Class.forName("java.lang.Runtime");
						Method method = m_javalangRuntimeClass.getMethod("getRuntime", (Class[]) null);
						m_javalangRuntime = method.invoke(null, (Object[]) null);
					}
				catch (Exception e)
					{
					}
			}

		public static MyRuntime getRuntime()
			{
				return m_Runtime;
			}

		public MyProcess exec(String a_strCommand) throws IOException
			{
				if (m_javalangRuntime == null)
					throw new IOException("Processs creation not supported");
				else
					{
						try
							{
								Class[] parameterTypes = new Class[1];
								parameterTypes[0] = String.class;
								Method method = m_javalangRuntimeClass.getMethod("exec", parameterTypes);
								Object[] args = new Object[1];
								args[0] = a_strCommand;
								return new MyProcess(method.invoke(m_javalangRuntime, args));
							}
						catch (Throwable t)
							{
								throw new IOException(t.getMessage());
							}
					}
			}

		public MyProcess exec(String[] cmd) throws IOException
			{
				if (m_javalangRuntime == null)
					throw new IOException("Processs creation not supported");
				else
					{
						try
							{
								Class[] parameterTypes = new Class[1];
								parameterTypes[0] = String[].class;
								Method method = m_javalangRuntimeClass.getMethod("exec", parameterTypes);
								Object[] args = new Object[1];
								args[0] = cmd;
								return new MyProcess(method.invoke(m_javalangRuntime, args));
							}
						catch (Throwable t)
							{
								throw new IOException(t.getMessage());
							}
					}
			}

	}
