package anon.proxy;

import anon.proxy.HTTPProxyCallback.HTTPConnectionHeader;

public abstract class AbstractJonDoFoxHeaders extends AbstractHTTPConnectionListener
{
	public final static String HTTP_ENCODING_GZIP = "gzip";
	public final static String HTTP_ENCODING_DEFLATE = "deflate";
	
	//public final static String USER_AGENT_JONDOFOX = 
	public final static String USER_AGENT_JONDOFOX = "Mozilla/5.0 (Windows NT 6.1; rv:17.0) Gecko/17.0 Firefox/17.0";
	public final static String USER_AGENT_JONDOFOX_OLD = "Mozilla/5.0 (Windows NT 6.1; rv:10.0) Gecko/20100101 Firefox/10.0";
	public final static String USER_AGENT_TORBUTTON = 
		"Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.3) Gecko/20100401 Firefox/3.6.3";
	public final static String USER_AGENT_TORBUTTON_OLD = 
		"Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.7) Gecko/2009021910 Firefox/3.0.7";
		
	public final static String JONDOFOX_LANGUAGE_NEW = "en-us";
	public final static String JONDOFOX_CHARSET = "utf-8,*";
	public final static String JONDOFOX_CONTENT_TYPES = "text/html,application/xml,*/*";
	public final static String JONDOFOX_ENCODING = HTTP_ENCODING_GZIP+", "+HTTP_ENCODING_DEFLATE;
	
	public final static String HTTP_DO_NOT_TRACK = "DNT";
	//public final static String HTTP_DO_NOT_TRACK = "X-Do-Not-Track";
	//public final static String HTTP_BEHAVIORAL_AD_OPT_OUT = "X-Behavioral-Ad-Opt-Out";
	
	public AbstractJonDoFoxHeaders(int a_priority)
	{
		super(a_priority);
	}
	
	protected boolean checkJonDoFox(HTTPConnectionEvent event)
	{
		HTTPConnectionHeader connHeader = event.getConnectionHeader();
		
		String[] checkHeader;
		int iJonDoFoxRecognized = 2; // try to recognize JonDoFox by some headers
		
		boolean bProxyKeepAlive = false;
		
		// TODO calculate header signature and compare to the default Firefox signature
		
		
		checkHeader = connHeader.getRequestHeader(HTTPProxyCallback.HTTP_KEEP_ALIVE);
		if (checkHeader != null && checkHeader.length >= 1)
		{
			bProxyKeepAlive = true;
		}
		checkHeader = connHeader.getRequestHeader(HTTPProxyCallback.HTTP_PROXY_CONNECTION);
		if (checkHeader != null && checkHeader.length >= 1 && 
			checkHeader[0].toLowerCase().equals(HTTPProxyCallback.HTTP_KEEP_ALIVE))
		{
			bProxyKeepAlive = true;
		}
		
		if (event.getAnonRequest().isProxyKeepAliveEnabled())
		{
			// proxy-connection keep-alive seems to be on...
			bProxyKeepAlive = true;
		}
		
		if (bProxyKeepAlive)
		{
			iJonDoFoxRecognized--;
		}
		
		checkHeader = connHeader.getRequestHeader(HTTPProxyCallback.HTTP_USER_AGENT);
		if (checkHeader == null || checkHeader.length != 1 || 
			(!checkHeader[0].equals(USER_AGENT_JONDOFOX) &&
					!checkHeader[0].equals(USER_AGENT_JONDOFOX_OLD)))
		{
			iJonDoFoxRecognized--;
		}
		
		checkHeader = connHeader.getRequestHeader(HTTPProxyCallback.HTTP_ACCEPT_LANGUAGE);
		if (checkHeader == null || checkHeader.length != 1 || 
			!checkHeader[0].equals(JONDOFOX_LANGUAGE_NEW))
		{
			iJonDoFoxRecognized--;
		}
		
		checkHeader = connHeader.getRequestHeader(HTTPProxyCallback.HTTP_ACCEPT_ENCODING);
		if (checkHeader == null || checkHeader.length != 1 || 
			!checkHeader[0].equals(JONDOFOX_ENCODING))
		{
			iJonDoFoxRecognized--;
		}
		
		checkHeader = connHeader.getRequestHeader(HTTPProxyCallback.HTTP_ACCEPT_CHARSET);
		if (checkHeader != null && checkHeader.length >= 1)
		{
			// this header should not be sent at all
			iJonDoFoxRecognized--;
		}
		
		if (iJonDoFoxRecognized >= 0)
		{
			return true;
		}
		
		return false;
	}
	



}
