package anon.proxy;

import anon.infoservice.IBrowserIdentification;
import anon.infoservice.InfoServiceDBEntry;
import anon.proxy.HTTPProxyCallback.HTTPConnectionHeader;

public class BrowserIdentification extends AbstractJonDoFoxHeaders 
	implements IBrowserIdentification
{
	private static final int BROWSER_RECOGNITION_UNINITIALISED = -1;
	private static final int BROWSER_UNKNOWN = 0;
	private static final int BROWSER_TORBUTTON = 1;
	private static final int BROWSER_JONDOFOX = 2;
	private static final int BROWSER_INTERNET_EXPLORER = 3;
	private static final int BROWSER_FIREFOX = 4;
	private static final int BROWSER_OPERA = 5;
	private static final int BROWSER_SAFARI = 6;
	private static final int BROWSER_KONQUEROR = 7;
	private static final int BROWSER_CHROME = 8;
	
	
	private static final long[] BROWSER_OCCURENCE = new long[9];
	private static final String[] BROWSER_NAME = 
		new String[]{"other", "Tor", "JonDoFox", "Internet Explorer", "Firefox", "Opera", "Safari", "Konqueror", "Chrome"}; 
	
	
	

	
	private static boolean ms_bTestShown = false;
	private static boolean ms_bDetectedBrowser = false;
	
	
	public BrowserIdentification(int a_priority)
	{
		super(a_priority);
		InfoServiceDBEntry.setBrowserIdentification(this);
	}
	
	public boolean isBlockable()
	{
		return false;
	}
	
	public int getMostFrequentBrowser()
	{
		int iBrowser = BROWSER_RECOGNITION_UNINITIALISED;
		long iOccurence = 0;
		
		for (int i = 0; i < BROWSER_OCCURENCE.length; i++)
		{
			if (BROWSER_OCCURENCE[i] > iOccurence)
			{
				iOccurence = BROWSER_OCCURENCE[i];
				iBrowser = i;
			}
		}
		return iBrowser;
	}
	
	public String getBrowserName()
	{
		int browser = getMostFrequentBrowser();
		if (browser == BROWSER_RECOGNITION_UNINITIALISED)
		{
			return null;
		}
		return BROWSER_NAME[browser];
	}
	
	/*
	 * Weak detection
	 */
	public boolean isJonDoFoxDetected()
	{
		return BROWSER_OCCURENCE[BROWSER_JONDOFOX] > 0;
	}
	
	public void responseHeadersReceived(HTTPConnectionEvent event)
	{
		
	}
	
	public void downstreamContentBytesReceived(HTTPConnectionEvent event) 
	{
	}
	
	public void upstreamContentBytesReceived(HTTPConnectionEvent event) 
	{
	}
	
	public void requestHeadersReceived(HTTPConnectionEvent event)
	{
		if (event == null)
		{
			return;
		}

		HTTPConnectionHeader connHeader = event.getConnectionHeader();
		
		if (connHeader != null)
		{	
			if (connHeader.getRequestLine().startsWith("CONNECT"))
			{
				// filtering for HTTPS does not make much sense; we are uncovered anyway
				return;
			}
		
			countBrowserType(event, checkJonDoFox(event));
		}
	
	}
	
	private void countBrowserType(HTTPConnectionEvent event, boolean a_bJonDoFox)
	{
		if (a_bJonDoFox)
		{
			BROWSER_OCCURENCE[BROWSER_JONDOFOX] = 1;
			ms_bDetectedBrowser = true;
			return;
		}
		
		if (ms_bDetectedBrowser && !ms_bTestShown && 
			HTTPProxyCallback.isAnonymityTestDomain(event.getConnectionHeader()))
		{
			ms_bTestShown = true;
		}

		if (!ms_bTestShown && BROWSER_OCCURENCE[BROWSER_JONDOFOX] == 0 &&
			!event.getConnectionHeader().getRequestLine().startsWith("CONNECT"))
		{
			int detectedBrowser = BROWSER_RECOGNITION_UNINITIALISED;
			String strUA;
			String[] ua = event.getConnectionHeader().getRequestHeader(HTTPProxyCallback.HTTP_USER_AGENT);
			if (ua != null && ua.length > 0)
			{
				strUA = ua[0].toLowerCase();
				if (strUA.indexOf("firefox") >= 0)
				{
					if (ua[0].equals(JonDoFoxHeader.USER_AGENT_TORBUTTON) || 
						ua[0].equals(JonDoFoxHeader.USER_AGENT_TORBUTTON_OLD))
					{
						detectedBrowser = BROWSER_TORBUTTON;
					}
					else
					{
						detectedBrowser = BROWSER_FIREFOX;
					}
				}
				else if (strUA.indexOf("msie") >= 0)
				{
					detectedBrowser = BROWSER_INTERNET_EXPLORER;
				}
				else if (strUA.indexOf("opera") >= 0)
				{
					detectedBrowser = BROWSER_OPERA;
				}
				else if (strUA.indexOf("chrome") >= 0)
				{
					detectedBrowser = BROWSER_CHROME;
				}				
				else if (strUA.indexOf("konqueror") >= 0)
				{
					detectedBrowser = BROWSER_KONQUEROR;
				}
				else if (strUA.indexOf("safari") >= 0)
				{
					detectedBrowser = BROWSER_SAFARI;
				}
				else if (strUA.indexOf("httpclient") < 0) // do not count InfoService/HttpClient messages
				{
					detectedBrowser = BROWSER_UNKNOWN;
				}
				if (detectedBrowser > BROWSER_RECOGNITION_UNINITIALISED)
				{
					ms_bDetectedBrowser = true;
					BROWSER_OCCURENCE[detectedBrowser]++;
				}
			}
			
			if (detectedBrowser > BROWSER_JONDOFOX &&
				BROWSER_OCCURENCE[detectedBrowser] > 40)
			{
				event.getAnonRequest().showBrowserWarning(true);
			}
		}
	}
}
