/*
 Copyright (c) 2000, The JAP-Team
 All rights reserved.
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.

 - Neither the name of the University of Technology Dresden, Germany nor the names of its contributors
   may be used to endorse or promote products derived from this software without specific
  prior written permission.


 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
package anon.proxy;

import java.util.StringTokenizer;

import anon.proxy.HTTPProxyCallback.HTTPConnectionHeader;

public final class JonDoFoxHeader extends AbstractJonDoFoxHeaders
{
	public JonDoFoxHeader(int a_priority)
	{
		super(a_priority);
	}
	

	
	public void handleRequest(HTTPConnectionEvent event)
	{
	}

	public void downstreamContentBytesReceived(HTTPConnectionEvent event) 
	{
	}
	

	
	
	
	public void requestHeadersReceived(HTTPConnectionEvent event) 
	{
		if (event == null)
		{
			return;
		}

		HTTPConnectionHeader connHeader = event.getConnectionHeader();
		
		if (connHeader != null)
		{
			String domain;
			String domainReferer;
			String[] strReferer;
			
			if (connHeader.getRequestLine().startsWith("CONNECT"))
			{
				// filtering for HTTPS does not make much sense; we are uncovered anyway
				return;
			}
			
			
			if (checkJonDoFox(event))
			{
				return;
			}
			
			domain = connHeader.parseDomain(true);
			if (domain != null)
			{
				strReferer = connHeader.getRequestHeader(HTTPProxyCallback.HTTP_REFERER);
				if (strReferer == null || strReferer.length != 1)
				{
					domainReferer = null;
				}
				else
				{
					domainReferer = HTTPProxyCallback.parseDomain(strReferer[0], true);
				}
				
				//System.out.println(domainReferer + " : " + domain);
				
				/*
				 *  Take the basic, second level domain of the requested domain; if the referrer includes this
				 *  second level domain, then it is not deleted. Web sites may then follow the users
				 *  traces on their own domain, but this provides a far better compatibility with real
				 *  web sites.
				 */
				if (domainReferer == null || !domainReferer.equals(domain))
				{
					// hide the referrer so that the user cannot be traced from one second level domain to the next
					connHeader.removeRequestHeader(HTTPProxyCallback.HTTP_REFERER);
				}
			}
			
			/*
			if (event.getConnectionHeader().getRequestLine().startsWith("CONNECT"))
			{
				// this is a CONNECT tunneling command - ask the user what to do now
				String[] uaHeader = connHeader.getRequestHeader(HTTPProxyCallback.HTTP_USER_AGENT);
				if (uaHeader == null || uaHeader.length != 1 || uaHeader[0] == null || 
					(!uaHeader[0].equals(JONDOFOX_USER_AGENT) && !uaHeader[0].equals(JONDOFOX_USER_AGENT_NEW)))
				{
					event.setNeedsConfirmation(true);
				}
			}*/
			
			// TODO: Check whether proxy-connection keep-alive is set, and close the connection if it is
			// maybe by using connection:close from server
			//connHeader.removeRequestHeader(HTTPProxyCallback.HTTP_PROXY_CONNECTION);
			
			String[] strHeaderValue = connHeader.getRequestHeader(HTTPProxyCallback.HTTP_HOST);
			if (strHeaderValue != null && strHeaderValue.length > 0)
			{
				connHeader.replaceRequestHeader(HTTPProxyCallback.HTTP_HOST, strHeaderValue[0]);	
			}

			// The order of these headers is important. Do not change it!
			connHeader.replaceRequestHeader(HTTPProxyCallback.HTTP_USER_AGENT, USER_AGENT_JONDOFOX);
			if (!event.getConnectionHeader().getRequestLine().startsWith("CONNECT"))
			{
				connHeader.replaceRequestHeader(HTTPProxyCallback.HTTP_ACCEPT, JONDOFOX_CONTENT_TYPES);
				
				connHeader.replaceRequestHeader(HTTPProxyCallback.HTTP_ACCEPT_LANGUAGE, JONDOFOX_LANGUAGE_NEW);
				
				
				String[] clientSupportedencodings = connHeader.getRequestHeader(HTTPProxyCallback.HTTP_ACCEPT_ENCODING);
				event.getAnonRequest().setInternalEncodingRequired(
						detectInternaEncodingRequired(clientSupportedencodings));
				//will be determined when the server response is received
				event.getAnonRequest().setContentEncodings(null);
			
				
				/*
				strHeaderValue = connHeader.getRequestHeader(HTTPProxyCallback.HTTP_ACCEPT_ENCODING);
				if (strHeaderValue == null || strHeaderValue.length == 0)
				{
					// TODO Problem with IE9!! Test why decompression fails and fix it...
					connHeader.resetRequestHeader(HTTPProxyCallback.HTTP_ACCEPT_ENCODING);
				}
				else
				*/
				{
					connHeader.replaceRequestHeader(HTTPProxyCallback.HTTP_ACCEPT_ENCODING, JONDOFOX_ENCODING);
				}
				
				connHeader.removeRequestHeader(HTTPProxyCallback.HTTP_ACCEPT_CHARSET);
				
				
				
				
				/* does not yet what it should - mycookie is not set correctly!
				strHeaderValue = connHeader.getRequestHeader(HTTPProxyCallback.HTTP_COOKIE);
				if (strHeaderValue != null && strHeaderValue.length > 0)
				{
					connHeader.removeRequestHeader(HTTPProxyCallback.HTTP_COOKIE);
					for (int i = 0; i < strHeaderValue.length; i++)
					{
						connHeader.setRequestHeader(HTTPProxyCallback.HTTP_COOKIE, strHeaderValue[i]);
					}
					connHeader.setRequestHeader(HTTPProxyCallback.HTTP_COOKIE, "mycookie=test");
				}*/
				
				strHeaderValue = connHeader.getRequestHeader(HTTPProxyCallback.HTTP_REFERER);
				if (strHeaderValue != null && strHeaderValue.length > 0)
				{
					connHeader.replaceRequestHeader(HTTPProxyCallback.HTTP_REFERER, strHeaderValue[0]);	
				}
				
				
				//connHeader.replaceRequestHeader(HTTP_BEHAVIORAL_AD_OPT_OUT, "1");
				//connHeader.replaceRequestHeader("X-JonDonym-Test", "true");
				connHeader.resetRequestHeader(HTTPProxyCallback.HTTP_CONNECTION);
				connHeader.replaceRequestHeader(HTTP_DO_NOT_TRACK, "1");
				connHeader.resetRequestHeader(HTTPProxyCallback.HTTP_PROXY_CONNECTION);
				connHeader.resetRequestHeader(HTTPProxyCallback.HTTP_KEEP_ALIVE); // TODO check order!
		
				connHeader.removeRequestHeader(HTTPProxyCallback.HTTP_IE_UA_CPU);
				connHeader.removeRequestHeader(HTTPProxyCallback.HTTP_PRAGMA);
				
			}
		}
	}


	public void responseHeadersReceived(HTTPConnectionEvent event) 
	{
		AnonProxyRequest req = event.getAnonRequest();
		HTTPConnectionHeader connHeader = event.getConnectionHeader();
		if (req.isInternalEncodingRequired())
		{
			String[] contentEncodings = connHeader.getResponseHeader(HTTPProxyCallback.HTTP_CONTENT_ENCODING);
			if(contentEncodings != null)
			{
				req.setContentEncodings(contentEncodings);
				connHeader.removeResponseHeader(HTTPProxyCallback.HTTP_CONTENT_ENCODING);
				//Also remove Content-Length, because this header cannot be changed 
				//when the length of the uncompressed content is determined.
				connHeader.removeResponseHeader(HTTPProxyCallback.HTTP_CONTENT_LENGTH);
			}
			else
			{
				req.setInternalEncodingRequired(false);
			}
		}
	}

	private static boolean detectInternaEncodingRequired(String[] clientSupportedEncodings)
	{
		boolean gzipFound = false;
		boolean zlibFound = false;
		if(clientSupportedEncodings != null)
		{
			StringTokenizer valueTokenizer = null;
			for (int i = 0; i < clientSupportedEncodings.length; i++) 
			{
				valueTokenizer = new StringTokenizer(clientSupportedEncodings[i], ",");
				String token = null;
				while (valueTokenizer.hasMoreTokens()) {
					token = valueTokenizer.nextToken().trim();
					if(!gzipFound)
					{
						gzipFound = token.equals(HTTP_ENCODING_GZIP);
					}
					if(!zlibFound)
					{
						zlibFound = token.trim().equals(HTTP_ENCODING_DEFLATE);
					}
				}
			}
		}
		return !gzipFound || !zlibFound;
	}

	public void upstreamContentBytesReceived(HTTPConnectionEvent event) 
	{
	}

}
