/*
 Copyright (c) 2000, The JAP-Team
 All rights reserved.
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.

 - Neither the name of the University of Technology Dresden, Germany nor the names of its contributors
   may be used to endorse or promote products derived from this software without specific
  prior written permission.


 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
package anon.proxy;

/**
 * Used to process bytes received as a request before sending to its destination
 * or as a response from that destination, before it's given to the client.
 *
 * @see {@link ProxyCallbackHandler}
 * @see {@link AnonProxyRequest#run()}
 */

public interface ProxyCallback
{
	/** indicates that a packet should be not processed any further */
	public final static int STATUS_FINISHED = 0;

	/** indicates that packet processing should stop immediately and to delay packet sending until enuogh
	 * data for processing is collected
	 */
	public final static int STATUS_DELAY = 1;

	/** indicates that a packet can be processed any further by other registered proxy callbacks */
	public final static int STATUS_PROCESSABLE = 2;

	//processing failure should be indicated by raising exception
	//rather than by defining a status code.

	/**
	 * Processes a chunk of data, before it is sent out.
	 *
	 * @param anonRequest The request the data is from.
	 * @param buffer Data that is sent to and from the ProxyCallback object.
	 * @return
	 * @throws ProxyCallbackNotProcessableException if an error occurred during
	 * processing of the data.
	 */
	public int handleUpstreamChunk(AnonProxyRequest anonRequest, ProxyCallbackBuffer buffer) throws ProxyCallbackNotProcessableException;

	/**
	 * Processes a chunk of received data, before it is given to the client of the
	 * proxy.
	 *
	 * @param anonRequest The request the data is from.
	 * @param buffer Data that is sent to and from the ProxyCallback object.
	 * @return
	 * @throws ProxyCallbackNotProcessableException if an error occurred during
	 * processing of the data.
	 */
	public int handleDownstreamChunk(AnonProxyRequest anonRequest, ProxyCallbackBuffer buffer) throws ProxyCallbackNotProcessableException;
	/**
	 * Signals to the {@link ProxyCallback} that the given
	 * {@link AnonProxyRequest} is being closed and all the related stored data
	 * can be discarded.
	 *
	 * @param anonRequest
	 */
	public void closeRequest(AnonProxyRequest anonRequest);
}
