package anon.util;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import logging.LogHolder;
import logging.LogLevel;
import logging.LogType;

/**
 * This password reader memorizes previously given passwords and first tries to use them before asking the user for a new password.
 * @author Rolf
 *
 */
public abstract class AbstractMemorizingPasswordReader implements IMiscPasswordReader 
{
	
	private Vector pwMatches = new Vector();
	private Enumeration currentPWIteration = null;
	private Object lastAccount = null;
	private boolean skipAll = false;
	private Hashtable m_completedObjects;
	
	public AbstractMemorizingPasswordReader()
	{
		m_completedObjects = new Hashtable();
	}
	
	public final synchronized void reset()
	{
		m_completedObjects.clear();
	}
	
	public final synchronized int countCmpletedObjects()
	{
		return m_completedObjects.size();
	}
	
	public final synchronized char[] getPassword()
	{
		return ((String)m_completedObjects.elements().nextElement()).toCharArray();	
	}
	
	protected abstract void initPasswordDialog(Object a_message);
	
	protected abstract String readPassword();
	
	protected abstract boolean askForCancel();
	
	public final synchronized String readPassword(Object a_message)
	{
		String password = null;

		if(skipAll)
		{
			return null;
		}
		
		if(lastAccount == null)
		{
			lastAccount = a_message;
		}
		
		/* try to query a new account -> the last one matched */
		if(!a_message.equals(lastAccount))
		{
			pwMatches.addElement(m_completedObjects.get(lastAccount));
			currentPWIteration = null;
			currentPWIteration = pwMatches.elements();
			lastAccount = a_message;
		}
		
		if(currentPWIteration != null)
		{
			if(currentPWIteration.hasMoreElements())
			{
				return (String) currentPWIteration.nextElement();
			}
			else
			{
				currentPWIteration = null;
			}
		}
		
		initPasswordDialog(a_message);
		while (true)
		{
			try
			{
				password = readPassword();
			}
			catch (Exception a_e)
			{
				LogHolder.log(LogLevel.ALERT, LogType.MISC, "Could not read password!", a_e);
				return null;
			}
			if (password == null)
			{
				m_completedObjects.remove(a_message);
				
				if (askForCancel())
				{
					// user clicked cancel
					skipAll = true;
					break;
				}
			}
			else
			{
				m_completedObjects.put(a_message, password);
				break;
			}
		}
		return password;
	}
}
