package anon.util;

import java.io.File;
import java.util.Hashtable;

/** A class implementing this interface helps the ResourceLoder to actually load some resources, i.e. in an OS or otherwise specific way.
 * 
 * @author sk13
 *
 */

public interface IResourceLoaderHelper
	{
		/**
		 * Loads resources from a given directory (simple file, directory or zip file) that are found
		 * in the given resource search path and instantiates them as objects of a type defined by the
		 * resource instantiator. Resources with the same path name
		 * are only loaded once according to the given Hashtable.
		 * The resource search path is relative to the given directory and may either specify a single
		 * resource or, if it ends with a slash "/", a (virtual) directory in the given directory.
		 * Therefore a resource search path "/" loads all resources in the specified directory.
		 * A resource search path "certificates/" loads all resources in the (virtual) directory
		 * <i> certificates </i>.
		 * @param a_strResourceSearchPath a (virtual) path in the directory to load resources from
		 * @param a_Directory a simple file, directory or zip file, might be null - in this case some "default" path/file/URL is assumed
		 * @param a_instantiator a ResourceInstantiator that is used to instantiate the loaded resources
		 * @param a_bRecursive true if (virtual) subdirectories should be searched for resources;
		 *                     false otherwise (has an effect only for resource paths ending with "/")
		 * @param a_bStopAtFirstResource true if the search should stop with the first loaded resource;
		 *                               false otherwise
		 * @param a_loadedResources a Hashtable where the loaded and instantiated resources are stored
		 */
		public void loadResources(String a_strResourceSearchPath,
											File a_Directory,
											IResourceInstantiator a_instantiator,
											boolean a_bRecursive,
											boolean a_bStopAtFirstResource,
											Hashtable a_loadedResources);
	}
