/*
 Copyright (c) 2000, The JAP-Team
 All rights reserved.
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

 - Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

 - Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.

 - Neither the name of the University of Technology Dresden, Germany nor the names of its contributors
  may be used to endorse or promote products derived from this software without specific
  prior written permission.


 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
package anon.util;

import java.io.UnsupportedEncodingException;

public class URLCoder
	{
		/**
		 * Decodes a URL to a unicode. This method uses UTF-8-encoding.
		 * 
		 * @param a_strURL
		 *          a URL in UTF-8-encoding
		 * @return the URL in unicode encoding
		 */
		public static String decode(String a_strURL)
			{
				if (a_strURL == null)
					{
						return null;
					}

				StringBuffer output = new StringBuffer();
				byte[] enc = new byte[a_strURL.length()];
				int bytes = 0;
				int i = 0;
				char c;

				try
					{
						while (i < a_strURL.length())
							{
								c = a_strURL.charAt(i);
								if (c == '+')
									{
										output.append(' ');
									}
								else if (c == '%')
									{
										enc[bytes] = (byte) Integer.parseInt(a_strURL.substring(i + 1, i + 3), 16);
										bytes++;
										i += 2;
									}
								else
									{
										output.append(c);
									}
								i++;

								if ((i < a_strURL.length() && a_strURL.charAt(i) != '%') || i >= a_strURL.length())
									{
										output.append(new String(enc, 0, bytes, "UTF8"));
										bytes = 0;
									}
							}
					}

				catch (NumberFormatException a_e)
					{
						return null;
					}
				catch (UnsupportedEncodingException a_e)
					{
						return null;
					}

				return output.toString();
			}

		/**
		 * Taken from Apache Harmony* Copyright 2011 JAP-Team
		 * 
		 * Licensed under the Apache License, Version 2.0 (the "License"); you may
		 * not use this file except in compliance with the License. You may obtain a
		 * copy of the License at
		 * 
		 * http://www.apache.org/licenses/LICENSE-2.0
		 * 
		 * Unless required by applicable law or agreed to in writing, software
		 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
		 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
		 * License for the specific language governing permissions and limitations
		 * under the License.
		 */
		/**
		 * Translates a string into <code>application/x-www-form-urlencoded</code>
		 * using UTF-8 encoding
		 * 
		 * @param in
		 *          String to be encoded
		 * @throws UnsupportedEncodingException
		 */
		public static String encode(String s) throws UnsupportedEncodingException
			{
				if (s == null)
					return null;
				// Guess a bit bigger for encoded form
				StringBuffer buf = new StringBuffer(s.length() + 16);
				int start = -1;
				for (int i = 0; i < s.length(); i++)
					{
						char ch = s.charAt(i);
						if ((ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z') || (ch >= '0' && ch <= '9')
								|| " .-*_".indexOf(ch) > -1) { //$NON-NLS-1$
								if (start >= 0)
									{
										convert(s.substring(start, i), buf);
										start = -1;
									}
								if (ch != ' ')
									{
										buf.append(ch);
									}
								else
									{
										buf.append('+');
									}
							}
						else
							{
								if (start < 0)
									{
										start = i;
									}
							}
					}
				if (start >= 0)
					{
						convert(s.substring(start, s.length()), buf);
					}
				return buf.toString();
			}

		static final String digits = "0123456789ABCDEF"; //$NON-NLS-1$

		private static void convert(String s, StringBuffer buf) throws UnsupportedEncodingException
			{
				byte[] bytes = s.getBytes("UTF8");
				for (int j = 0; j < bytes.length; j++)
					{
						buf.append('%');
						buf.append(digits.charAt((bytes[j] & 0xf0) >> 4));
						buf.append(digits.charAt(bytes[j] & 0xf));
					}
			}
	}