package anon.anonudp.client.test;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import mybouncycastle.org.bouncycastle.crypto.engines.AESFastEngine;
import mybouncycastle.org.bouncycastle.crypto.modes.SICBlockCipher;
import mybouncycastle.org.bouncycastle.crypto.params.KeyParameter;
import mybouncycastle.org.bouncycastle.crypto.params.ParametersWithIV;

import anon.anonudp.mixmessage.crypto.CTRCipher;
import anon.anonudp.mixmessage.crypto.EccGroup713;
import anon.anonudp.mixpacket.DataPacket;
import anon.anonudp.mixpacket.PacketFactory;
import anon.util.Util;
import junit.framework.TestCase;

public class UDPMixDataPacketDecryptionTest extends TestCase
	{
		public void testDecryption() throws Exception
			{
				DataPacket packet1 = new DataPacket(0, new byte[2], new byte[DataPacket.SIZE]);
				DataPacket packet2 = new DataPacket(0, new byte[2], new byte[DataPacket.SIZE]);
				byte channelKeys[][] = new byte[3][EccGroup713.SYMMETRIC_KEY_LENGTH];
				
				for (int j = 0; j < channelKeys.length; j++)
					{
						packet1 = PacketFactory.process(packet1, channelKeys[j]);
					}				

				PacketFactory packetFactory=new PacketFactory(channelKeys);
				packet2=packetFactory.myprocess(packet2);
				assertTrue(Util.arraysEqual(packet1.getData(),packet2.getData()));
				
				DataPacket packet = new DataPacket(0, new byte[2], new byte[DataPacket.SIZE]);
				//Check if transformation done right...
				SICBlockCipher cipher = new SICBlockCipher(new AESFastEngine());

        cipher.init(true, new ParametersWithIV(new KeyParameter(channelKeys[0]), channelKeys[1]));
        
        Cipher sdkCipher=Cipher.getInstance("AES/CTR/NoPadding");
				IvParameterSpec ivSpec1 = new IvParameterSpec(channelKeys[1]);

				SecretKeySpec aesKey1 = new SecretKeySpec(channelKeys[0], "AES");
				//sdkCipher.init(Cipher.ENCRYPT_MODE, aesKey1, ivSpec1);
	      byte out[]=new byte[DataPacket.SIZE];
				int numRuns = 1000000;
				long startTime = System.currentTimeMillis();
				for (int k = 0; k < numRuns; k++)
					{
						for (int j = 0; j < channelKeys.length; j++)
							{
								packet = PacketFactory.process(packet, channelKeys[j]);
								//int off=cipher.processBytes(packet.getData(), 0, packet.getData().length, out, 0);
//								int off=sdkCipher.update(packet.getData(), 0, packet.getData().length, out, 0);
							}
					}
				
				
				
				long endTime = System.currentTimeMillis();
				int time = (int) (endTime - startTime);
				int x = (numRuns * 1000) / time;
				System.out.println(
						"Original method: Finished " + numRuns + " DataPacket decryption processings in " + time + " ms -- " + x + " pro second");

				packetFactory=new PacketFactory(channelKeys);
				startTime = System.currentTimeMillis();
				for (int k = 0; k < numRuns; k++)
					{
						packetFactory.myprocess(packet);
					}
				
				
				
				endTime = System.currentTimeMillis();
				time = (int) (endTime - startTime);
				x = (numRuns * 1000) / time;
				System.out.println(
						"My Cipher: Finished " + numRuns + " DataPacket decryption processings in " + time + " ms -- " + x + " pro second");
			
			}
	}
