package anon.client.crypto;

import mybouncycastle.org.bouncycastle.crypto.engines.AESFastEngine;
import mybouncycastle.org.bouncycastle.crypto.modes.GCMBlockCipher;
import mybouncycastle.org.bouncycastle.crypto.params.AEADParameters;
import mybouncycastle.org.bouncycastle.crypto.params.KeyParameter;

public class DataChannelCipherReferenceImplementation
{


		private GCMBlockCipher m_sendEngine;
		private GCMBlockCipher m_recvEngine;

		private long m_EncMsgCounter;
		private long m_DecMsgCounter;

		private byte[] m_sendKey;
		private byte[] m_recvKey;
		private byte[] m_sendIV;
		private byte[] m_recvIV;
		

		private boolean m_bSingleKey = false;

		private byte[] m_decryptHelperBuff=new byte[32];
		
		public DataChannelCipherReferenceImplementation()
			{
				m_sendEngine = new GCMBlockCipher(new AESFastEngine());
				m_recvEngine = new GCMBlockCipher(new AESFastEngine());
				m_sendIV = new byte[12];
				m_recvIV = new byte[12];
				m_EncMsgCounter = 0;
				m_DecMsgCounter = 0;
			}

		synchronized private int setSendKey(byte[] key, int offset, int len)
			{
				try
					{
						m_sendKey =new byte[len];
						System.arraycopy(key, offset, m_sendKey, 0, len);
						m_EncMsgCounter = 0;
						return 0;
					}
				catch (Exception e)
					{
						return -1;
					}
			}

		synchronized private int setRecvKey(byte[] key, int offset, int len)
			{
				try
					{
						m_recvKey =new byte[len];
						System.arraycopy(key, offset, m_recvKey, 0, len);
						m_DecMsgCounter = 0;
						return 0;
					}
				catch (Exception e)
					{
						return -1;
					}
			}

		/* (non-Javadoc)
		 * @see anon.client.crypto.IDataChannelChipher#setEncryptionKeysAES(byte[])
		 */
		synchronized public int setEncryptionKeysAES(byte[] keys)
			{
				return setEncryptionKeysAES(keys,0,keys.length);
			}

		/* (non-Javadoc)
		 * @see anon.client.crypto.IDataChannelChipher#setEncryptionKeysAES(byte[], int, int)
		 */
		synchronized public int setEncryptionKeysAES(byte[] keys,int offset,int keylen)
			{
				int ret1, ret2;
				try
					{
						if (keylen == 16)
							{
								m_bSingleKey = true;
								ret1 = setSendKey(keys, offset, 16);
								ret2 = setRecvKey(keys, offset, 16);
							}
						else
							{
								m_bSingleKey = false;
								ret1 = setSendKey(keys, offset, 16);
								ret2 = setRecvKey(keys, offset+16, 16);
							}
						if ((ret1 == 0) && (ret2 == 0))
							{
								return 0;
							}
						else
							{
								return -1;
							}
					}
				catch (Exception e)
					{
						return -1;
					}
			}

		/* (non-Javadoc)
		 * @see anon.client.crypto.IDataChannelChipher#getKeys()
		 */
		public byte[] getKeys()
			{
				try
					{
						if ((m_sendKey == null) || (m_recvKey == null))
							{
								return null;
							}
						else if (m_bSingleKey)
							{
								return m_sendKey;
							}
						else
							{
								byte[] keys = new byte[32];
								System.arraycopy(m_sendKey, 0, keys, 0, 16);
								System.arraycopy(m_recvKey, 0, keys, 16, 16);
								return keys;
							}
					}
				catch (Exception e)
					{
						return null;
					}
			}

		private static void createIV(byte[] iv, long counter)
			{
				iv[8] = (byte) (counter >> 24 & 0x00ff);
				iv[9] = (byte) (counter >> 16 & 0x00ff);
				iv[10] = (byte) (counter >> 8 & 0x00ff);
				iv[11] = (byte) (counter & 0x00ff);
			}

		/* (non-Javadoc)
		 * @see anon.client.crypto.IDataChannelChipher#encryptGCM1(byte[], int, byte[], int, int)
		 */
		public void encryptGCM1(byte[] from, int ifrom, byte[] to, int ito, int len) throws Exception
			{
				DataChannelCipherReferenceImplementation.createIV(m_sendIV, m_EncMsgCounter);
				m_EncMsgCounter++;
				AEADParameters sendKey = new AEADParameters(new KeyParameter(m_sendKey, 0, m_sendKey.length), 128, m_sendIV, null);
				
				m_sendEngine.init(true, sendKey);
				int outlen = m_sendEngine.processBytes(from, ifrom, len, to, ito);
				m_sendEngine.doFinal(to, ito + outlen);
			}

		/* (non-Javadoc)
		 * @see anon.client.crypto.IDataChannelChipher#decryptGCM2(byte[], int, byte[], int, int)
		 */
		public void decryptGCM2(byte[] from, int ifrom, byte[] to, int ito, int len) throws Exception
			{
				DataChannelCipherReferenceImplementation.createIV(m_recvIV, m_DecMsgCounter);
				m_DecMsgCounter++;
				AEADParameters recvKey = new AEADParameters(new KeyParameter(m_recvKey, 0, m_recvKey.length), 128, m_recvIV, null);
				
				m_recvEngine.init(false, recvKey);
				/* decrypt only the first 2 bytes to get the payload length */
				//Note: We have to process a full block + MAC_SIZE here!
				m_recvEngine.processBytes(from, ifrom, 32, m_decryptHelperBuff,0);
				int payloadLen = ((m_decryptHelperBuff[0] << 8) & 0x003F00) | (m_decryptHelperBuff[1] & 0x00FF);
				/* decrypt the remaining packet now */
				m_recvEngine.reset();
				int declen = m_recvEngine.processBytes(from, ifrom, payloadLen + 19, to, ito); //Size= payloadlen+ payloadHeaderSize+MacSize
				m_recvEngine.doFinal(to, ito + declen);
			}

		public static int getMACSize()
			{
				return 16;
			}
	}
