package anon.client.crypto;

import anon.util.Util;
import junit.framework.Assert;
import junit.framework.TestCase;

public class DataChannelCipherTest extends TestCase
	{

		public void testEncryptGCM1() throws Throwable
			{
					DataChannelCipher c=new DataChannelCipher();
					DataChannelCipherReferenceImplementation cr=new DataChannelCipherReferenceImplementation();
					byte[] keys=new byte[32]; 
					c.setEncryptionKeysAES(keys);
					cr.setEncryptionKeysAES(keys);
					byte[] from=new byte[128];
					byte[] to1=new byte[128];
					byte[] to2=new byte[128];
					c.encryptGCM1(from, 0, to1, 0, 100);
					c.encryptGCM1(from, 0, to2, 0, 100);
					Assert.assertFalse(Util.arraysEqual(to1,to2));

					byte[] r_to1=new byte[128];
					byte[] r_to2=new byte[128];
					cr.encryptGCM1(from, 0, r_to1, 0, 100);
					cr.encryptGCM1(from, 0, r_to2, 0, 100);
					Assert.assertFalse(Util.arraysEqual(r_to1,r_to2));

					Assert.assertTrue(Util.arraysEqual(to1,r_to1));
					Assert.assertTrue(Util.arraysEqual(to2,r_to2));

			}
		
		public void testDecryptGCM1() throws Throwable
			{
					DataChannelCipher c=new DataChannelCipher();
					DataChannelCipherReferenceImplementation cr=new DataChannelCipherReferenceImplementation();
					byte[] keys=new byte[32]; 
					c.setEncryptionKeysAES(keys);
					cr.setEncryptionKeysAES(keys);
					int len=100;
					byte[] from=new byte[128];
					from[1]=(byte)(len-3); // we have to subtract the header from the payload length...
					
					byte[] to1=new byte[128];
					byte[] to2=new byte[128];
					c.encryptGCM1(from, 0, to1, 0, len);
					c.encryptGCM1(from, 0, to2, 0, len);
					Assert.assertFalse(Util.arraysEqual(to1,to2));

					byte[] r_to1=new byte[128];
					byte[] r_to2=new byte[128];
					cr.encryptGCM1(from, 0, r_to1, 0, len);
					cr.encryptGCM1(from, 0, r_to2, 0, len);
					Assert.assertFalse(Util.arraysEqual(r_to1,r_to2));

					Assert.assertTrue(Util.arraysEqual(to1,r_to1));
					Assert.assertTrue(Util.arraysEqual(to2,r_to2));
					
					byte[] test=new byte[128];
					test[1]=(byte)(len-3);
					cr.decryptGCM2(r_to1, 0, from,0, 116);
					Assert.assertTrue(Util.arraysEqual(from,test));
					cr.decryptGCM2(r_to2, 0, from,0, 116);
					Assert.assertTrue(Util.arraysEqual(from,test));
					c.decryptGCM2(to1, 0, from,0, 116);
					Assert.assertTrue(Util.arraysEqual(from,test));
					c.decryptGCM2(to2, 0, from,0, 116);
					Assert.assertTrue(Util.arraysEqual(from,test));

			}
		

		public void testEncryptGCM1Performance() throws Throwable
			{
					DataChannelCipher c=new DataChannelCipher();
					DataChannelCipherReferenceImplementation cr=new DataChannelCipherReferenceImplementation();
					byte[] keys=new byte[32]; 
					c.setEncryptionKeysAES(keys);
					cr.setEncryptionKeysAES(keys);
					
					long runs=1000000;
					byte[] from=new byte[1024];
					byte[] to1=new byte[from.length+16];
					long start=System.currentTimeMillis();
					for	(long i=0;i<runs;i++)
						{
							cr.encryptGCM1(from, 0, to1, 0, from.length);
												
							
						}
					long end=System.currentTimeMillis();
					
					System.out.println("Reference Implementation: "+(from.length*runs)/(end-start)+" bytes/ms");
	
					 start=System.currentTimeMillis();
					for	(long i=0;i<runs;i++)
						{
							c.encryptGCM1(from, 0, to1, 0, from.length);
												
							
						}
					 end=System.currentTimeMillis();
					
					System.out.println("AN.ONLib Implementation: "+(from.length*runs)/(end-start)+" bytes/ms");
	
			}

	
	}
