package anon.client.test;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;

import HTTPClient.HTTPClientSocketFactory;
import anon.AnonChannel;
import anon.client.AnonClient;
import anon.crypto.SignatureVerifier;
import anon.infoservice.InfoServiceDBEntry;
import anon.infoservice.InfoServiceHolder;
import anon.infoservice.MixCascade;
import anon.infoservice.SimpleMixCascadeContainer;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

public class AnonClientConnectionWithDNSResolverTest extends TestCase
	{

		class TestSocketFactory implements HTTPClientSocketFactory
			{

				public Socket connect(InetAddress host, int port) throws IOException
					{
						return new Socket(host, port);
					}

			}

		protected void runTest()
			{
				try
					{
						TestSocketFactory factory = new TestSocketFactory();
						AnonClient anon = new AnonClient(null); //create AN.ON client
						anon.setSocketFactory(factory);

						InfoServiceHolder is = InfoServiceHolder.getInstance();
						is.setSocketFactory(factory);
						InfoServiceDBEntry service = new InfoServiceDBEntry("infoservice.inf.tu-dresden.de", 80);
						is.setPreferredInfoService(service);
						anon.setDNSResolver(is);
						
						//--> set to your cascade here!!
						MixCascade mixCascade = new MixCascade(null, null, "dud165.inf.tu-dresden.de", 6544); //manually select mix cascade
						mixCascade.setAccessControlPreSharedSecret("anonvpnrules");
						SignatureVerifier.getInstance().setCheckSignatures(false); //insecure, of course - but necessary for this simple manually added mix-cascade

						//initialize the client --> establish the connection
						anon.initialize(mixCascade, new SimpleMixCascadeContainer(mixCascade), null, true);

						//create a communication channel (like a connected socket)
						AnonChannel c = anon.createChannel(AnonChannel.HTTP);
						InputStream in = c.getInputStream();
						OutputStream out = c.getOutputStream();

						//write a simple HTTP request...
						out.write("GET http://google.de/ HTTP/1.0\n\n".getBytes());
						byte buff[] = new byte[1000];
						int len = 0;

						//get the answer and print the on the command line
						while ((len = in.read(buff)) > 0)
							{
								System.out.print(new String(buff, 0, len));
							}
					}
				catch (Exception e)
					{
						e.printStackTrace();
						assertEquals(true, false);
						return;
					}
				assertEquals(true, true);

			}

		private static Test[] createAllTests()
			{
				Test[] allTests = new Test[1];
				allTests[0] = new AnonClientConnectionWithDNSResolverTest();
				return allTests;
			}

		public static Test suite()
			{
				Test[] allTests = createAllTests();
				TestSuite suite = new TestSuite(AnonClientConnectionWithDNSResolverTest.class.getName());
				for (int i = 0; i < allTests.length; i++)
					{
						suite.addTest(allTests[i]);
					}
				return suite;
			}
	}
