package anon.client.test;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;

import HTTPClient.HTTPClientSocketFactory;
import anon.AnonChannel;
import anon.client.AnonClient;
import anon.client.AnonTCPChannel;
import anon.client.DummyTrafficControlChannel;
import anon.crypto.SignatureVerifier;
import anon.infoservice.MixCascade;
import anon.infoservice.SimpleMixCascadeContainer;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import logging.LogHolder;
import logging.LogLevel;
import logging.LogType;
import logging.SystemErrLog;

public class AnonTCPChannelTest extends TestCase
	{

		class TestSocketFactory implements HTTPClientSocketFactory
		{

			public Socket connect(InetAddress host, int port) throws IOException
				{
					return new Socket(host,port);
				}

		}

		protected void runTest()
			{
				try{
					SystemErrLog log=new SystemErrLog();
					log.setLogLevel(LogLevel.DEBUG);
					log.setLogType(LogType.ALL);
					LogHolder.setLogInstance(log);
					LogHolder.setDetailLevel(LogHolder.DETAIL_LEVEL_HIGHEST);
				AnonClient  anon=new AnonClient(null); //create AN.ON client
				anon.setSocketFactory(new TestSocketFactory());
				//--> set to your cascade here!!
				MixCascade mixCascade=new MixCascade(null, null, "141.76.46.165", 6544); //manually select mix cascade
				mixCascade.setAccessControlPreSharedSecret("anonvpnrules");
				SignatureVerifier.getInstance().setCheckSignatures(false); //insecure, of course - but necessary for this simple manually added mix-cascade

				//initialize the client --> establish the connection
				anon.initialize(mixCascade, new SimpleMixCascadeContainer(mixCascade), null, true);
				anon.setDummyTraffic(DummyTrafficControlChannel.DT_DISABLE);
				//create a communication channel (like a connected socket)
				AnonTCPChannel c=(AnonTCPChannel)anon.createChannel(AnonChannel.TCP);
				InputStream in=c.getInputStream();
				OutputStream out=c.getOutputStream();
				
				//c.connect("145.243.240.20",80 );
				c.connect("www.tu-dresden.de",80);
				//write a simple HTTP request...
				out.write("GET / HTTP/1.0\n\n".getBytes());
				byte buff[]=new byte[1000];
				int len=0;
				
				//get the answer and print the on the command line
				while((len=in.read(buff))>0)
					{
						System.out.print(new String(buff,0,len));
					}
				}
				catch(Exception e)
					{
						e.printStackTrace();
						assertEquals(true,false);
						return;
					}
				assertEquals(true,true);

			
			}
	
		private static Test[] createAllTests()
			{
				Test[] allTests = new Test[1];
				allTests[0]=new AnonTCPChannelTest();
				return allTests;
			}

		public static Test suite()
			{
				Test[] allTests = createAllTests();
				TestSuite suite = new TestSuite(AnonTCPChannelTest.class.getName());
				for (int i = 0; i < allTests.length; i++) 
				{
					suite.addTest(allTests[i]);
				}
				return suite;
			}
	}
