package anon.pay.xml;

import java.util.Calendar;
import java.util.GregorianCalendar;

import junitx.framework.extension.XtendedPrivateTestCase;




public class XMLBalanceTest extends XtendedPrivateTestCase
{	

	public XMLBalanceTest(String name)
	{
		super(name);
	}

	public void testCalculateEndDate()
	{
		Calendar calendarStart = new GregorianCalendar();
		Calendar calendarEnd = new GregorianCalendar();
		Calendar calendarEndCalculated;
		
		calendarStart.set(2010, 1, 23, 14, 1, 2);
		calendarEndCalculated = XMLBalance.calculateEndDate(calendarStart, 5, Calendar.MONTH);
		calendarEnd.set(2010, 6, 22, 23, 59, 59);
		assertEquals(calendarEnd, calendarEndCalculated);
		
		
		calendarStart.set(2010, 7, 10, 15, 0, 2);
		calendarEndCalculated = XMLBalance.calculateEndDate(calendarStart, 2, Calendar.MONTH);
		calendarEnd.set(2010, 9, 9, 23, 59, 59);
		assertEquals(calendarEnd, calendarEndCalculated);
	}
	
	private void assertEquals(GregorianCalendar a_calendarOne, GregorianCalendar a_calendarTwo)
	{
		assertEquals(a_calendarOne.get(Calendar.YEAR), a_calendarTwo.get(Calendar.YEAR));
		assertEquals(a_calendarOne.get(Calendar.MONTH), a_calendarTwo.get(Calendar.MONTH));
		assertEquals(a_calendarOne.get(Calendar.DAY_OF_MONTH), a_calendarTwo.get(Calendar.DAY_OF_MONTH));
		assertEquals(a_calendarOne.get(Calendar.HOUR_OF_DAY), a_calendarTwo.get(Calendar.HOUR_OF_DAY));
		assertEquals(a_calendarOne.get(Calendar.MINUTE), a_calendarTwo.get(Calendar.MINUTE));
		assertEquals(a_calendarOne.get(Calendar.SECOND), a_calendarTwo.get(Calendar.SECOND));
	}
	
	public void testIsLastMonthOfRate()
	{
		Calendar calendarNow = new GregorianCalendar();
		Calendar calendarStart = new GregorianCalendar();
		Calendar calendarEnd;
		long tStart, tNow, tEnd;
		
		calendarStart.set(2010, 1, 23, 14, 1, 2);
		calendarEnd = XMLBalance.calculateEndDate(calendarStart, 5, Calendar.MONTH);
		tStart = calendarStart.getTime().getTime();
		tEnd = calendarEnd.getTime().getTime();
		
		calendarNow.set(2010, 6, 20, 11, 5, 22);
		tNow = calendarNow.getTime().getTime();
		assertTrue(XMLBalance.isLastMonthOfRate(calendarEnd, calendarNow, calendarStart));
		
		// objects should not get changed
		assertEquals(tStart, calendarStart.getTime().getTime());
		assertEquals(tEnd, calendarEnd.getTime().getTime());
		assertEquals(tNow, calendarNow.getTime().getTime());
		
		
		calendarNow.set(2010, 6, 23, 11, 5, 22);
		tNow = calendarNow.getTime().getTime();
		assertTrue(!XMLBalance.isLastMonthOfRate(calendarEnd, calendarNow, calendarStart));
		
		assertEquals(tStart, calendarStart.getTime().getTime());
		assertEquals(tEnd, calendarEnd.getTime().getTime());
		assertEquals(tNow, calendarNow.getTime().getTime());
		
		calendarNow.set(2010, 6, 22, 23, 59, 59); // should be the last day of the rate
		calendarNow.set(Calendar.MILLISECOND, 0);
		assertTrue(XMLBalance.isLastMonthOfRate(calendarEnd, calendarNow, calendarStart));
		
		calendarNow.set(2010, 5, 22, 23, 59, 59); // should be the day before the last month
		calendarNow.set(Calendar.MILLISECOND, 0);
		assertTrue(!XMLBalance.isLastMonthOfRate(calendarEnd, calendarNow, calendarStart));
		
		calendarNow.set(2010, 5, 23, 0, 0, 0); // should be the first day of the last month of the rate
		calendarNow.set(Calendar.MILLISECOND, 0);
		tNow = calendarNow.getTime().getTime();
		assertTrue(XMLBalance.isLastMonthOfRate(calendarEnd, calendarNow, calendarStart));
		
		assertEquals(tStart, calendarStart.getTime().getTime());
		assertEquals(tEnd, calendarEnd.getTime().getTime());
		assertEquals(tNow, calendarNow.getTime().getTime());
		
		
		// tests for February and for switching years
		calendarStart.set(2009, 10, 1, 1, 30, 40);
		calendarEnd = XMLBalance.calculateEndDate(calendarStart, 4, Calendar.MONTH);
		
		calendarNow.set(2010, 0, 31, 23, 6, 12);
		assertTrue(calendarNow.get(Calendar.MONTH) != calendarEnd.get(Calendar.MONTH));
		assertTrue(!XMLBalance.isLastMonthOfRate(calendarEnd, calendarNow, calendarStart));
		
		calendarNow.set(2010, 1, 20, 11, 5, 22);
		assertTrue(XMLBalance.isLastMonthOfRate(calendarEnd, calendarNow, calendarStart));
		
		calendarNow.set(2010, 1, 28, 23, 59, 59);
		assertTrue(XMLBalance.isLastMonthOfRate(calendarEnd, calendarNow, calendarStart));
		
		calendarNow.set(2010, 2, 1, 1, 2, 3);
		assertTrue(!XMLBalance.isLastMonthOfRate(calendarEnd, calendarNow, calendarStart));
		
		
		// tests for February only
		calendarStart.set(2010, 0, 1, 1, 30, 40);
		calendarEnd = XMLBalance.calculateEndDate(calendarStart, 3, Calendar.MONTH);
		
		calendarNow.set(2010, 0, 15, 11, 43, 40);
		assertTrue(!XMLBalance.isLastMonthOfRate(calendarEnd, calendarNow, calendarStart));
		
		calendarNow.set(2010, 1, 25, 13, 0, 0);
		assertTrue(!XMLBalance.isLastMonthOfRate(calendarEnd, calendarNow, calendarStart));
		
		calendarNow.set(2010, 1, 28, 23, 59, 59);
		assertTrue(!XMLBalance.isLastMonthOfRate(calendarEnd, calendarNow, calendarStart));
		
		calendarNow.set(2010, 2, 1, 1, 2, 3);
		assertTrue(XMLBalance.isLastMonthOfRate(calendarEnd, calendarNow, calendarStart));
		
		
		//test for changing years
		calendarStart.set(2008, 4, 15, 0, 20, 30);
		calendarEnd = XMLBalance.calculateEndDate(calendarStart, 27, Calendar.MONTH);
		
		calendarNow.set(2008, 7, 1, 1, 2, 3);
		assertTrue(!XMLBalance.isLastMonthOfRate(calendarEnd, calendarNow, calendarStart));
		
		calendarNow.set(2009, 7, 1, 1, 2, 3);
		assertTrue(!XMLBalance.isLastMonthOfRate(calendarEnd, calendarNow, calendarStart));
		
		calendarNow.set(2010, 7, 1, 1, 2, 3);
		assertTrue(XMLBalance.isLastMonthOfRate(calendarEnd, calendarNow, calendarStart));
	}
	
	public void testCalculateRemainingRateMonths()
	{
		GregorianCalendar calendarNow = new GregorianCalendar();
		GregorianCalendar calendarEnd = new GregorianCalendar();

		calendarEnd.set(2010, 5, 15, 23, 59, 59);
		
		calendarNow.set(2010, 0, 23, 11, 43, 40);
		assertEquals(4, XMLBalance.calculateRemainingRateMonths(calendarEnd, calendarNow));
		
		calendarNow.set(2010, 5, 1, 2, 2, 2);
		assertEquals(0, XMLBalance.calculateRemainingRateMonths(calendarEnd, calendarNow));
		
		calendarNow.set(2009, 11, 30, 4, 0, 0);
		assertEquals(5, XMLBalance.calculateRemainingRateMonths(calendarEnd, calendarNow));
		
		calendarNow.set(2011, 11, 30, 4, 0, 0);
		assertEquals(0, XMLBalance.calculateRemainingRateMonths(calendarEnd, calendarNow));
	}
}
