package anon.proxy;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import anon.proxy.DecompressionProxyCallback.DecompressionKit;

/**
 * Test case to simulate the rare situation where a gzip header
 * is splitted up between two chunks. The testsuite returned
 * contains tests for each index that splits up the header as well
 * as tests for some other indices that do not.
 * @author simon
 */
public class GzipHeaderSplitTest extends TestCase
{
	private final static int CHUNK_SIZE = 2900;
	private final static int DEFAULT_READ_BYTES = 1000;
	private final static int GZIP_HEADER_START_OFFSET = 323;
	private String expectedResult = null;
	private InputStream compressedInputStream = null;
	StringBuffer resultBuffer = new StringBuffer();
	int splitChunkOffset;
	
	public GzipHeaderSplitTest(int splitChunkOffset) 
	{
		this.splitChunkOffset = splitChunkOffset;
		setName("test with split-offset "+splitChunkOffset);
	}

	protected void setUp() throws Exception 
	{
		StringBuffer inString = new StringBuffer();
		
		Reader reader = new InputStreamReader(
					GzipHeaderSplitTest.class.getResourceAsStream(
							"decompressedOut"), "UTF-8");
		char[] inputBuffer = new char[1000];
		int charsRead = 0;
		do
		{
			charsRead = reader.read(inputBuffer);
			if(charsRead > 0) inString.append(new String(inputBuffer, 0, charsRead));
		}
		while (charsRead > 0);
		expectedResult = inString.toString();
		compressedInputStream = GzipHeaderSplitTest.class.getResourceAsStream("compressedIn");
	}
	
	protected void runTest() throws Exception
	{
		DecompressionProxyCallback callback = new DecompressionProxyCallback();
		DecompressionKit decompressionKit = new DecompressionKit();
		decompressionKit.setNewInflater(true);
		decompressionKit.setResult(new byte[DecompressionProxyCallback.MAX_DECOMPRESSION_OUTPUT]);
		
		int chunkSplitLength = GZIP_HEADER_START_OFFSET + splitChunkOffset;
		
		byte[] chunk = new byte[CHUNK_SIZE];
		
		//the first read operation reads data until the specified split index is reached.
		compressedInputStream.read(chunk, 0, chunkSplitLength);
		ProxyCallbackBuffer buffer = 
			new ProxyCallbackBuffer(chunk, GZIP_HEADER_START_OFFSET, chunkSplitLength);
		
		//the gzip header might get splitted here. 
		callback.decompress(buffer, decompressionKit, true, true);
		resultBuffer.append(new String(buffer.getChunk(), 0, buffer.getPayloadLength()));
		
		int lenRead = 0;
		//process the rest of the data.
		while ((lenRead = compressedInputStream.read(chunk, 0, DEFAULT_READ_BYTES)) > 0 ) 
		{
			buffer = new ProxyCallbackBuffer(chunk, 0, lenRead);
			callback.decompress(buffer, decompressionKit, true, false);
			resultBuffer.append(new String(buffer.getChunk(), 0, buffer.getPayloadLength(), "UTF-8"));
		}
		
		assertEquals(expectedResult, resultBuffer.toString());
		compressedInputStream.close();
	}
	
	public static Test suite()
	{
		TestSuite suite = new TestSuite("SplitTest");
		//also test non split indices ( 0 and >= 10)
		for (int i = 0; i <= 20; i++) 
		{
			suite.addTest(new GzipHeaderSplitTest(i));
		}
		return suite;
	}
}
