/*
 Copyright (c) 2000 - 2004, The JAP-Team
 All rights reserved.
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

  - Redistributions of source code must retain the above copyright notice,
 this list of conditions and the following disclaimer.

  - Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation and/or
 other materials provided with the distribution.

  - Neither the name of the University of Technology Dresden, Germany nor the names of its contributors
 may be used to endorse or promote products derived from this software without specific
 prior written permission.


 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
 */
package logging;

/** Writes log output to two Log instances*/
public final class ChainedLog implements ILog
{
	private ILog m_Log1,m_Log2;
	private boolean m_bConstantLevel1, m_bConstantLevel2;
	private boolean m_bConstantType1, m_bConstantType2;
	

	public ChainedLog(ILog log1,ILog log2)
	{
		m_Log1=log1;
		m_Log2=log2;
	}

	public synchronized void log(int level, int type, String mesg)
	{
		m_Log1.log(level,type,mesg);
		m_Log2.log(level,type,mesg);
	}

	public synchronized void setLogType(int type)
	{
		if (!m_bConstantType1)
		{
			m_Log1.setLogType(type);
		}
		if (!m_bConstantType2)
		{
			m_Log2.setLogType(type);
		}
	}
	
	public synchronized void setConstantLogLevel(ILog a_log, boolean a_bConstant)
	{
		if (m_Log1 == a_log)
		{
			m_bConstantLevel1 = a_bConstant;
		}
		else if (m_Log2 == a_log)
		{
			m_bConstantLevel2 = a_bConstant;
		}
	}
	
	public synchronized void setConstantLogType(ILog a_log, boolean a_bConstant)
	{
		if (m_Log1 == a_log)
		{
			m_bConstantType1 = a_bConstant;
		}
		else if (m_Log2 == a_log)
		{
			m_bConstantType2 = a_bConstant;
		}
	}

	public boolean isLogged(int a_logLevel, int a_logType)
	{
		if (AbstractLog.isLogged(m_Log1, a_logLevel, a_logType) ||
			AbstractLog.isLogged(m_Log2, a_logLevel, a_logType))
		{
			return true;
		}
		return false;
	}
	
	public int getLogType()
	{
		if (!m_bConstantType1)
		{
			return m_Log1.getLogType();
		}
		
		return m_Log2.getLogType();
	}

	public synchronized void setLogLevel(int level)
	{
		if (!m_bConstantLevel1)
		{
			m_Log1.setLogLevel(level);
		}
		if (!m_bConstantLevel2)
		{
			m_Log2.setLogLevel(level);
		}
	}

	public int getLogLevel()
	{
		if (!m_bConstantLevel1)
		{
			return m_Log1.getLogLevel();
		}
		return m_Log2.getLogLevel();
	}
}
